#!/bin/csh
#########################################################################
# USAGE                                                                 #
#########################################################################
echo "Submission script for running XELL/EPOS/NEXuS + SPheRIO 4.0 on STAR PBS cluster"
if (($# < 1)||($# > 7)) then
cat << EOF
Usage:
  `basename $0` name_of_collision 
        [epos/nexus/xell] [number_of_events] [first_event] [number_of_nodes] [first_node] [starting_folder]
EOF
exit 1
endif
#########################################################################
# INPUT                                                                 #
#########################################################################
#set BIN = "~/bin"
echo "Number of arguments received:" $#
set cnamecoll = $1
echo "Name of collisions is: " $cnamecoll
set pbs_script = "shs-xellspherio.csh"
if ($# >= 2) then
  if ($2 == "epos") then
      echo "Use Initial Conditions from EPOS"
      set pbs_script = "shs-epospherio.csh"
  else if ($2 == "nexus") then
      echo "Use Initial Conditions from NEXuS"
      set pbs_script = "shs-nexspherio.csh"
  else if ($2 == "xell") then
      echo "SPheRIO standalone version"
      set pbs_script = "shs-xellspherio.csh"
  else
    echo "Argument error:" $2 "is not a valid keyword"
    echo "Please input either epos or nexus"
    exit 1
  endif
else
  echo "SPheRIO standalone version"
endif
echo "Name of the script used to submit the jobs: " $pbs_script
@ itotalevents = 1
if ($# >= 3) then
  @ itotalevents = $3
endif
echo "Total number of events to be run is: " $itotalevents
@ ifirstevent = 1
if ($# >= 4) then
  @ ifirstevent = $4
endif
echo "The first number of event is: " $ifirstevent
@ itnodes = 1
if ($# >= 5) then
  @ itnodes = $5
endif
echo "Number of nodes to be used is: " $itnodes
if ($itnodes >= 20) then
	echo "ERROR: too many nodes! Script stopped!"
        exit 1
endif
@ ifirstnode = 1
if ($# >= 6) then
  @ ifirstnode = $6
endif
echo "The first number of node is: " $ifirstnode
set starting_folder = `pwd`
if ($# >= 7) then
set starting_folder = $7
if (! -d $starting_folder) then
    echo "WARNING: " $starting_folder " does not exist!"
    exit 1
endif
endif
echo "Starting folder is: "$starting_folder
@ ievent_d = $itotalevents / $itnodes
echo "Number of events to be run on each server is: " $ievent_d
@ inode = $ifirstnode
#########################################################################
# SUBMIT                                                                #
#########################################################################
while ($inode < ($ifirstnode + $itnodes ))
  echo "Current node index: " $inode
  @ ievent_i = ($inode - $ifirstnode) * $ievent_d + $ifirstevent
  echo $ievent_d "events will be submitted on the node, and the number of first event is" $ievent_i
  cat >sub-eponexspherio-$cnamecoll.$inode <<ENDE
#!/bin/csh -f
$BIN/$pbs_script $cnamecoll $ievent_d $ievent_i .$inode $starting_folder
ENDE
  chmod +x sub-eponexspherio-$cnamecoll.$inode
#  echo qsub -l cput=48:00:00 -o $starting_folder/ -e $starting_folder/ -N sub-eponexspherio-$cnamecoll.$inode sub-eponexspherio-$cnamecoll.$inode
#  qsub -l cput=48:00:00 -o $starting_folder/ -e $starting_folder/ -N sub-eponexspherio-$cnamecoll.$inode sub-eponexspherio-$cnamecoll.$inode
#   echo qsub -o $starting_folder/ -e $starting_folder/ -N sub-eponexspherio-$cnamecoll.$inode sub-eponexspherio-$cnamecoll.$inode
#   qsub -o $starting_folder/ -e $starting_folder/ -N sub-eponexspherio-$cnamecoll.$inode sub-eponexspherio-$cnamecoll.$inode
  rm -f tela$inode.log
  echo $BIN/$pbs_script $cnamecoll $ievent_d $ievent_i .$inode $starting_folder
  $BIN/$pbs_script $cnamecoll $ievent_d $ievent_i .$inode $starting_folder >& tela$inode.log </dev/null &
  sleep 1
  rm -f sub-eponexspherio-$cnamecoll.$inode
	@ inode = $inode + 1
end
exit 0
