CCC   #################################################################
CCC   SUBROUTINE PROGRAM: bsqdn.f 
CCC   "bsqdn" finds baryon number, strangeness and charge density 
CCC   at na -th particle position.             
CCC   #################################################################
CCC   last updated 08172007
CCC   fix a bug
CCC
CCC   last updated 08162007
CCC   fix a error
CCC
CCC   last updated 08142007
CCC   modified and add check
CCC
CCC   last updated 08092007
CCC   adopt ihdmode 1
CCC
CCC   last udpated 02202007
CCC   adopt 2d case
CCC   #################################################################
      SUBROUTINE bsqd(tau,na)

      IMPLICIT NONE
      include 'spherio.inc'
	include 'spherio.quifz.inc'
	include 'spherio.debug.inc'
	include 'spherio.hydro.inc'
CCC   /////////////////////////////////////////////////////
      INTEGER na,nb,nn
      DOUBLE PRECISION QXY,QET,W2,W1
      DOUBLE PRECISION u0,tau
CCC   ///////////////////////////////////////////////////
	if (idimension.eq.2) then
	call bsqd_xy(tau,na)
	return
	endif


      bd(oflg,na)=zero
      sd(oflg,na)=zero
      qd(oflg,na)=zero
	pibd(oflg,na)=zero
	sgmd(oflg,na)=zero
CCC   #####################################################
      DO nn=1,np
         nb= kl(nn) 
         QXY= dsqrt((xx(oflg,na)-xx(oflg,nb))**2
     &            +(yy(oflg,na)-yy(oflg,nb))**2)/hxy   
         QET=   abs(et(oflg,na)-et(oflg,nb))    /het 
         CALL krln(QXY,W2,2,12)
         CALL krln(QET,W1,1,33)
         bd(oflg,na)= bd(oflg,na)+ br(oflg,nb)*W2*W1
         sd(oflg,na)= sd(oflg,na)+ sr(oflg,nb)*W2*W1
         qd(oflg,na)= qd(oflg,na)+ qr(oflg,nb)*W2*W1
         pibd(oflg,na)= pibd(oflg,na)+ pibn(oflg,nb)*W2*W1
         sgmd(oflg,na)= sgmd(oflg,na)+ sgmn(oflg,nb)*W2*W1
         select case (ihdmode)
	   case (0)
              continue
	   case (1)
	        if ((iflgce(nb).eq.2).and.(ihdfull.ne.1)) then
              stop'fatal ihdmode error in bsqd'
	        end if
	   case default
	        stop'ihdmode fatal error in bsqd'
	   end select
      END DO
      u0= COSH(al(oflg,na)-et(oflg,na))*COSH(bt(oflg,na))
      bd(oflg,na)=bd(oflg,na)/(tau*u0)
      sd(oflg,na)=sd(oflg,na)/(tau*u0)
      qd(oflg,na)=qd(oflg,na)/(tau*u0)
      qd(oflg,na)=qd(oflg,na)/(tau*u0)
      pibd(oflg,na)=pibd(oflg,na)/(tau*u0)
      sgmd(oflg,na)=sgmd(oflg,na)/(tau*u0)
CCC   #####################################################
      RETURN
      END



CCC   #################################################################
CCC   SUBROUTINE PROGRAM: bsqdn.f 
CCC   "bsqdn" finds baryon number, strangeness and charge density 
CCC   at na -th particle position.             
CCC   #################################################################
CCC   last udpated 03052007
CCC   add output and fix a bug concerning it
CCC
CCC   last udpated 02202007
CCC   inherit from bsqd()
CCC   #################################################################
      SUBROUTINE bsqd_xy(tau,na)

      IMPLICIT NONE
      include 'spherio.inc'
	include 'spherio.debug.inc'
CCC   /////////////////////////////////////////////////////
      INTEGER na,nb,nn
      DOUBLE PRECISION QXY,QET,W2
c      double precision W1
      DOUBLE PRECISION u0,tau
CCC   ///////////////////////////////////////////////////
c	if (bugflg.ne.15) stop'bugflg fatal error in bsdq_xy()'


      bd(oflg,na)=zero
      sd(oflg,na)=zero
      qd(oflg,na)=zero
	pibd(oflg,na)=zero
	sgmd(oflg,na)=zero
CCC   #####################################################
      DO nn=1,np
         nb= kl(nn) 
         QXY= dsqrt((xx(oflg,na)-xx(oflg,nb))**2
     &            +(yy(oflg,na)-yy(oflg,nb))**2)/hxy   
         QET=   abs(et(oflg,na)-et(oflg,nb))    /het 
         CALL krln(QXY,W2,2,12)
c         CALL krln(QET,W1,1,33)
         bd(oflg,na)= bd(oflg,na)+ br(oflg,nb)*W2 !rip W1 from these terms
         sd(oflg,na)= sd(oflg,na)+ sr(oflg,nb)*W2
         qd(oflg,na)= qd(oflg,na)+ qr(oflg,nb)*W2
         pibd(oflg,na)= pibd(oflg,na)+ pibn(oflg,nb)*W2
         sgmd(oflg,na)= sgmd(oflg,na)+ sgmn(oflg,nb)*W2
      END DO
      u0= COSH(al(oflg,na)-et(oflg,na))*COSH(bt(oflg,na))
      bd(oflg,na)=bd(oflg,na)/(tau*u0)
      sd(oflg,na)=sd(oflg,na)/(tau*u0)
      qd(oflg,na)=qd(oflg,na)/(tau*u0)
      pibd(oflg,na)=pibd(oflg,na)/(tau*u0)
      sgmd(oflg,na)=sgmd(oflg,na)/(tau*u0)
CCC   #####################################################
      RETURN
      END



CCC   #################################################################
CCC   SUBROUTINE PROGRAM: decay.f
CCC   decay()
CCC   "decay" calculates SPH particle coordinates and veosities 
CCC   after its decay. By the decay, one SPH particel decays 
CCC   into two SPH partices with half entropy.    
CCC   The separation of two decay SPH particles is setted by 
CCC   "decaysplit".
CCC   #################################################################
CCC   last updated 04182009
CCC   improvement of bt interpolation and debugging output
CCC
CCC   last updated 03222009
CCC   check the difference between different ways of averaging
CCC
CCC   last updated 12272008
CCC   more debug info
CCC
CCC   last updated 12262008
CCC   added more debug info
CCC
CCC   last updated 12152008
CCC   added debug info
CCC
CCC   last updated 07072008
CCC   fix a bug concerning vvxn
CCC
CCC   last updated 08312007
CCC   incorporate vvxn
CCC
CCC   last updated 05212007
CCC   incorporate uxn
CCC
CCC   last updated 12192006
CCC   modified to incorporate tempxn in new chemical fo
CCC
CCC   last updated 10152006
CCC   incorporate chemical fo
CCC   #################################################################
      SUBROUTINE decay(tau,na,nd)
      IMPLICIT NONE

      include 'spherio.inc'
	include 'epos.incsf'
	include 'spherio.debug.inc'
      include 'spherio.seedspo.inc'
	include 'spherio.tautranz.inc'
	include 'spherio.hydro.inc'

      INTEGER nd,na,na1,na2,nb,nn,kk,i 
      DOUBLE PRECISION tau,delx,dely,dele
      DOUBLE PRECISION theta,phi,ran3,costheta
	double precision ran2r
      DOUBLE PRECISION xxp,yyp,etp
      DOUBLE PRECISION QXY,QET,W2,W1  
      DOUBLE PRECISION u0b,u1b,u2b,u3b,phif 
      DOUBLE PRECISION u0dn,u1dn,u2dn,u3dn
      DOUBLE PRECISION v1dn,v2dn,v3dn
      DOUBLE PRECISION aldn,phdn,btdn,undn
      DOUBLE PRECISION snp,brp,srp,qrp
	DOUBLE PRECISION pibnp,sgmnp

	if (idimension.eq.2) then
	call decay_xy(tau,na,nd)
	return
	endif
CCC   #################################################################
c      if (tau.gt.(tautranz+10.d0*dtau)) return
CCC   #################################################################
      nd=nd+1                !! decay particle ID
CCC   #################################################################
      na1=  na 
      na2=  Nsp(1) +nd

      if(na2.gt.NPL) goto 225 
      ndpr(na2)=na 

CCC   #################################################################
CCC   Emissao Continua e Quimico fo
CCC   #################################################################
      iflgce(na2)=iflgce(na1)     !variables not changing with time, so ntsp independent
      iflgxn(na2)=iflgxn(na1)
	mbxn(na2)  =mbxn(na1)
	msxn(na2)  =msxn(na1)
	tempxn(na2)=tempxn(na1)
	ssxn(na2)  =ssxn(na1)
	bbxn(na2)  =bbxn(na1)
      do kk=0,3
	uxn(kk,na2)=uxn(kk,na1)
	enddo
	iflgfp(na2)=iflgfp(na1)
CCC   #################################################################
      delx= hxy *decaysplt 
      dely= hxy *decaysplt
      dele= het *decaysplt
cxx      dele= 0.d0
      
      xxp= xx(1,na)
      yyp= yy(1,na)
      etp= et(1,na)
      snp= sn(1,na)
      pibnp=pibn(1,na)
      sgmnp=sgmn(1,na)
      brp= br(1,na)
      srp= sr(1,na)
      qrp= qr(1,na)

      phi= ran3(idum)*two*pi
c$$$      theta= (two*ran3(idum)-uni)*pi
c      costheta = -1.d0+2.d0*ran3(idum)
      costheta = -1.d0+2.d0*ran2r(idum)
      theta = dacos(costheta)
CCC   #################################################################
      sn(0,na1)= snp *half
      pibn(0,na1)=pibnp*half
      sgmn(0,na1)=sgmnp*half
      br(0,na1)= brp *half
      sr(0,na1)= srp *half
      qr(0,na1)= qrp *half
      xx(0,na1)= xxp +delx*sin(theta)*cos(phi)
      yy(0,na1)= yyp +dely*sin(theta)*sin(phi)
      et(0,na1)= etp +dele*cos(theta)
CCC   #################################################################
      sn(0,na2)= snp *half
	pibn(0,na2)=pibnp*half
	sgmn(0,na2)=sgmnp*half
      br(0,na2)= brp *half
      sr(0,na2)= srp *half
      qr(0,na2)= qrp *half
      xx(0,na2)= xxp +delx*sin(pi-theta)*cos(phi+pi)
      yy(0,na2)= yyp +dely*sin(pi-theta)*sin(phi+pi)
      et(0,na2)= etp +dele*cos(pi-theta)
CCC   #################################################################
CCC   debug output
	      if (iauxi.ge.2) then
            if ((xx(0,na2).le.0.d0).or.(xx(0,na2).ge.0.d0)) then
	        continue
	      elseif (iauxi.ge.5) then
              open(11,file=fnnan(1:nfnnan),access='append')
		    write(11,*)'nancatchup xx(0,na2) in decay'
              write(11,*)'oflg=',oflg
      	    write(11,*)'SPH particle number na1 na2=',na1,na2
      	    write(11,*)'original particle na, NPL=',na,NPL
      	    write(11,'(a,3d30.20)') 'xx(0,na2),yy(0,na2),et(0,na2)=',
     &                                 xx(0,na2),yy(0,na2),et(0,na2)
      	    write(11,'(a,3d30.20)') 'xx(1,na),yy(1,na),et(1,na)=',
     &                                 xx(1,na),yy(1,na),et(1,na)
      	    write(11,'(a,4d30.20)') 'delx,costheta,theta,phi=',
     &                                 delx,costheta,theta,phi
      	    write(11,'(a,3d30.20)')'sn(1,na),mbxn(na),ssxn(na)=',
     &                                sn(1,na),mbxn(na),ssxn(na)
        		close(11)
	        stop'nancatchup xx(0,na2) in decay'
	      else
	        stop'nancatchup xx(0,na2) in decay'
	      endif
	      endif
CCC   #################################################################
CCC   #################################################################
      DO i=1,2
         if(i.eq.1) nn=na1
         if(i.eq.2) nn=na2 
         CALL prcn(xxp,yyp,etp)
         if(npflg.eq.0) goto 224  
         u0dn= zero
         u1dn= zero
         u2dn= zero 
         u3dn= zero
         aldn= zero
         btdn= zero
         phdn= zero
         undn= zero
         DO kk=1,np
            nb= kl(kk)
            QXY=dsqrt((xx(0,nn)-xx(1,nb))**2+(yy(0,nn)-yy(1,nb))**2)/hxy
            QET=  abs( et(0,nn)-et(1,nb) )/het
            CALL krln(QXY,W2,2,12)
            CALL krln(QET,W1,1,33)
            u0b=  cosh(al(1,nb)-et(1,nb)) *cosh(bt(1,nb))
            u1b=   cos(ph(1,nb))          *sinh(bt(1,nb))
            u2b=   sin(ph(1,nb))          *sinh(bt(1,nb))
            u3b=  sinh(al(1,nb)-et(1,nb)) *cosh(bt(1,nb))/tau
            u0dn= u0dn+ u0b *W2*W1
            u1dn= u1dn+ u1b *W2*W1
            u2dn= u2dn+ u2b *W2*W1
            u3dn= u3dn+ u3b *W2*W1
            aldn= aldn+ al(1,nb) *W2*W1
            btdn= btdn+ bt(1,nb) *W2*W1
            phdn= phdn+ ph(1,nb) *W2*W1
            undn= undn+ 1.d0 *W2*W1
          ENDDO
CCC   find logigudinal, transverse rapidities and their azimuthal ang.
CCC   #################################################################
CCC   NaN catchup
      if (iauxi.ge.5) then
	if (((uni+(tau*u3dn/u0dn))/(uni-(tau*u3dn/u0dn))).le.0.d0) then
      open(11,file=fnnan(1:nfnnan),access='append')
      write(11,*)'NaN in SPH particle decay!'
	write(11,'(a,2f16.8)')'u3dn,u0dn=',u3dn,u0dn
      close(11)
	endif
	endif
CCC   #################################################################
         ph(0,nn)= phif(u1dn,u2dn) !option1
        al(0,nn)= half*dlog((uni+(tau*u3dn/u0dn))/(uni-(tau*u3dn/u0dn)))
         al(0,nn)= al(0,nn)+ et(0,nn)
cxx         bt(0,nn)= (u1dn/u0dn)*(u1dn/u0dn)+(u2dn/u0dn)*(u2dn/u0dn) !original~ bad convergence at big bt
cxx         bt(0,nn)= dsqrt(bt(0,nn)) *dcosh(al(0,nn)-et(0,nn))
cxx         bt(0,nn)= half*dlog((uni+bt(0,nn))/(uni-bt(0,nn)))
         v1dn=u1dn/u0dn !normalization of velocity
         v2dn=u2dn/u0dn
         v3dn=u3dn/u0dn
	   u0dn=1.d0/dsqrt(1.d0-v1dn**2-v2dn**2-v3dn**2*tau**2)
	   u1dn=v1dn*u0dn
	   u2dn=v2dn*u0dn
	   u3dn=v3dn*u0dn
         bt(0,nn)= dsqrt(u1dn*u1dn+u2dn*u2dn) !option1~ to avoid bad convergence and energy conservation problem
         bt(0,nn)= dlog(dsqrt(1.d0+bt(0,nn)**2)+bt(0,nn))
cxx          if (imprs.eq.1) then
cxx         aldn=aldn/undn       !option2~
cxx         btdn=btdn/undn
cxx         phdn=phdn/undn        !this is no good when ph=0
cxx         al(0,nn)= aldn
cxx         bt(0,nn)= btdn
cxx         ph(0,nn)= phdn
c          al(0,nn)= al(1,na)   !option3~
c          bt(0,nn)= bt(1,na)
c          ph(0,nn)= ph(1,na)
cxx          endif
CCC   #################################################################
CCC   cancellation of decay
c      if(iauxi.ge.8)then
c	if((bt(1,na).ge.9.d0).and.
c     &  (abs(bt(0,nn)/bt(1,na)).ge.2.d0.or.
c     &   abs(bt(1,na)/bt(0,nn)).ge.2.d0)) then
c          open(11,file=fnnan(1:nfnnan),access='append')
c		WRITE(11,*)'bt abnormal in decay!'
c		WRITE(11,*)'na,nd=',na,nd
c	    WRITE(11,*)'oflg=',oflg
c		WRITE(11,*)'tau,nn,na1,na2=',tau,nn,na1,na2
c		WRITE(11,'(a,2d20.10)') 'bt(0,nn),bt(1,na)=',bt(0,nn),bt(1,na)
c		WRITE(11,'(a,4d20.10)') 'u0dn,u1dn,u2dn,u3dn=',
c     &                             u0dn,u1dn,u2dn,u3dn
c		WRITE(11,'(a,3d20.10)') 'xx,yy,et=',
c     &                             xx(0,nn),yy(0,nn),et(0,nn)
c		WRITE(11,'(a,2d20.10)') 'al,ph=',
c     &                             al(0,nn),ph(0,nn)
cxx		WRITE(11,'(a,4d20.10)') 'aldn,btdn,phdn,undn=',
cxx     &                             aldn,btdn,phdn,undn
c          WRITE(11,'(a)') ' '
c		CLOSE(11)
c	endif
c	endif
      if (((u0dn.gt.0.d0).and.(u0dn.le.1.d7)).and.
     &    (bt(0,nn).le.15.d0)) then
c      if (tau.le.(tautranz+10.d0*dtau)) then
	    continue
	else
          if(iauxi.ge.8)then
          open(11,file=fnnan(1:nfnnan),access='append')
		WRITE(11,*)'bt nancatchup / overflow in decay!'
		WRITE(11,*)'particle splitting will be called off'
		WRITE(11,*)'na,nd=',na,nd
	    WRITE(11,*)'oflg=',oflg
		WRITE(11,*)'tau,nn,na1,na2=',tau,nn,na1,na2
		WRITE(11,'(a,2d20.10)') 'bt(0,nn),bt(1,na)=',bt(0,nn),bt(1,na)
		WRITE(11,'(a,4d20.10)') 'u0dn,u1dn,u2dn,u3dn=',
     &                             u0dn,u1dn,u2dn,u3dn
		WRITE(11,'(a,3d20.10)') 'xx,yy,et=',
     &                             xx(0,nn),yy(0,nn),et(0,nn)
		WRITE(11,'(a,2d20.10)') 'al,ph=',
     &                             al(0,nn),ph(0,nn)
cxx		WRITE(11,'(a,4d20.10)') 'aldn,btdn,phdn,undn=',
cxx     &                             aldn,btdn,phdn,undn
          WRITE(11,'(a)') ' '
		CLOSE(11)
	    endif
          al(0,na)= al(1,na)
          bt(0,na)= bt(1,na)
          ph(0,na)= ph(1,na)
          xx(0,na)= xx(1,na)
          yy(0,na)= yy(1,na)
          et(0,na)= et(1,na)
          sn(0,na)= sn(1,na)
          pibn(0,na)= pibn(1,na)
          sgmn(0,na)= sgmn(1,na)
          br(0,na)= br(1,na)
          sr(0,na)= sr(1,na)
          qr(0,na)= qr(1,na)
          nd=nd-1
c		stop 'bt fatal error in decay!'
          return
      endif
CCC   #################################################################
      ENDDO
CCC   #################################################################
c     Nsp(0)= na2 
c     oflg=0 
c     i1= INT((xx(oflg,na2)-(-xymx(oflg)))/hxy)+1 
c     j1= INT((yy(oflg,na2)-(-xymx(oflg)))/hxy)+1 
c     k1= INT((et(oflg,na2)-(-etmx(oflg)))/het)+1 
c     lt(oflg,nn) = hd(oflg,i1,j1,k1)
c     hd(oflg,i1,j1,k1)= na2   
CCC   #################################################################
      RETURN 

CCC   #################################################################
 224  CONTINUE
      open(11,file=fnnsp(1:nfnnsp),access='append')
      write(11,*) 'np error in decay.f \(^^:)/'
      close(11)
      stop'SPheRIO: np error in decay.f'
ccc   ////////////////////////////////////////////////      
 225  CONTINUE 
      open(11,file=fnnsp(1:nfnnsp),access='append')
      write(11,*) 'Nsp > NPL error in dectrl \(^^:)/'
      close(11)
      stop'Nsp > NPL error in decay \(^^:)/'
CCC   #################################################################
      END       



CCC   #################################################################
CCC   #################################################################
CCC   #################################################################




CCC   #################################################################
CCC   SUBROUTINE PROGRAM: decay.f
CCC   decay()
CCC   "decay" calculates SPH particle coordinates and veosities 
CCC   after its decay. By the decay, one SPH particel decays 
CCC   into two SPH partices with half entropy.    
CCC   The separation of two decay SPH particles is setted by 
CCC   "decaysplit".
CCC   #################################################################
CCC   last updated 05022009
CCC   built based on the 3D version
CCC   #################################################################
      SUBROUTINE decay_xy(tau,na,nd)
      IMPLICIT NONE

      include 'spherio.inc'
	include 'epos.incsf'
	include 'spherio.debug.inc'
      include 'spherio.seedspo.inc'
	include 'spherio.tautranz.inc'
	include 'spherio.hydro.inc'
      INTEGER nd,na,na1,na2,nb,nn,kk,i 
      DOUBLE PRECISION tau,delx,dely,dele
      DOUBLE PRECISION theta,phi,ran3,costheta
	double precision ran2r
      DOUBLE PRECISION xxp,yyp,etp
      DOUBLE PRECISION QXY,QET,W2
      DOUBLE PRECISION u0b,u1b,u2b,u3b,phif 
      DOUBLE PRECISION u0dn,u1dn,u2dn,u3dn
      DOUBLE PRECISION v1dn,v2dn,v3dn
      DOUBLE PRECISION aldn,phdn,btdn,undn
      DOUBLE PRECISION snp,brp,srp,qrp

CCC   #################################################################
      nd=nd+1                !! decay particle ID
CCC   #################################################################
      na1=  na 
      na2=  Nsp(1) +nd

      if(iviscosity.ge.2) then
	    write(*,*) 'not implemented yet'
          stop 'not implemented yet'
	endif

      if(na2.gt.NPL) goto 227 
      ndpr(na2)=na 

CCC   #################################################################
CCC   Emissao Continua e Quimico fo
CCC   #################################################################
      iflgce(na2)=iflgce(na1)     !variables not changing with time, so ntsp independent
      iflgxn(na2)=iflgxn(na1)
	mbxn(na2)  =mbxn(na1)
	msxn(na2)  =msxn(na1)
	tempxn(na2)=tempxn(na1)
	ssxn(na2)  =ssxn(na1)
	bbxn(na2)  =bbxn(na1)
      do kk=0,3
	uxn(kk,na2)=uxn(kk,na1)
	enddo
	iflgfp(na2)=iflgfp(na1)
CCC   #################################################################
      delx= hxy *decaysplt 
      dely= hxy *decaysplt
cxx      dele= het *decaysplt
      dele= 0.d0
      
      xxp= xx(1,na)
      yyp= yy(1,na)
      etp= et(1,na)
      snp= sn(1,na)
      brp= br(1,na)
      srp= sr(1,na)
      qrp= qr(1,na)

      phi= ran3(idum)*two*pi
      costheta = -1.d0+2.d0*ran2r(idum)
      theta = dacos(costheta)
CCC   #################################################################
      sn(0,na1)= snp *half
      br(0,na1)= brp *half
      sr(0,na1)= srp *half
      qr(0,na1)= qrp *half
      xx(0,na1)= xxp +delx*sin(theta)*cos(phi)
      yy(0,na1)= yyp +dely*sin(theta)*sin(phi)
      et(0,na1)= etp +dele*cos(theta)
CCC   #################################################################
      sn(0,na2)= snp *half
      br(0,na2)= brp *half
      sr(0,na2)= srp *half
      qr(0,na2)= qrp *half
      xx(0,na2)= xxp +delx*sin(pi-theta)*cos(phi+pi)
      yy(0,na2)= yyp +dely*sin(pi-theta)*sin(phi+pi)
      et(0,na2)= etp +dele*cos(pi-theta)
CCC   #################################################################
CCC   debug output
	      if (iauxi.ge.2) then
            if ((xx(0,na2).le.0.d0).or.(xx(0,na2).ge.0.d0)) then
	        continue
	      elseif (iauxi.ge.5) then
              open(11,file=fnnan(1:nfnnan),access='append')
		    write(11,*)'nancatchup xx(0,na2) in decay'
              write(11,*)'oflg=',oflg
      	    write(11,*)'SPH particle number na1 na2=',na1,na2
      	    write(11,*)'original particle na, NPL=',na,NPL
      	    write(11,'(a,3d30.20)') 'xx(0,na2),yy(0,na2),et(0,na2)=',
     &                                 xx(0,na2),yy(0,na2),et(0,na2)
      	    write(11,'(a,3d30.20)') 'xx(1,na),yy(1,na),et(1,na)=',
     &                                 xx(1,na),yy(1,na),et(1,na)
      	    write(11,'(a,4d30.20)') 'delx,costheta,theta,phi=',
     &                                 delx,costheta,theta,phi
      	    write(11,'(a,3d30.20)')'sn(1,na),mbxn(na),ssxn(na)=',
     &                                sn(1,na),mbxn(na),ssxn(na)
        		close(11)
	        stop'nancatchup xx(0,na2) in decay_xy'
	      else
	        stop'nancatchup xx(0,na2) in decay_xy'
	      endif
	      endif
CCC   #################################################################
CCC   #################################################################
      DO i=1,2
         if(i.eq.1) nn=na1
         if(i.eq.2) nn=na2 
         CALL prcn(xxp,yyp,etp)
         if(npflg.eq.0) goto 226  
         u0dn= zero
         u1dn= zero
         u2dn= zero 
         u3dn= zero
         aldn= zero
         btdn= zero
         phdn= zero
         undn= zero
         DO kk=1,np
            nb= kl(kk)
            QXY=dsqrt((xx(0,nn)-xx(1,nb))**2+(yy(0,nn)-yy(1,nb))**2)/hxy
            QET=  abs( et(0,nn)-et(1,nb) )/het
            CALL krln(QXY,W2,2,12)
c            CALL krln(QET,W1,1,33)
            u0b=  cosh(al(1,nb)-et(1,nb)) *cosh(bt(1,nb))
            u1b=   cos(ph(1,nb))          *sinh(bt(1,nb))
            u2b=   sin(ph(1,nb))          *sinh(bt(1,nb))
            u3b=  sinh(al(1,nb)-et(1,nb)) *cosh(bt(1,nb))/tau
            u0dn= u0dn+ u0b *W2       !rip off W1 here
            u1dn= u1dn+ u1b *W2
            u2dn= u2dn+ u2b *W2
            u3dn= u3dn+ u3b *W2
            aldn= aldn+ al(1,nb) *W2
            btdn= btdn+ bt(1,nb) *W2
            phdn= phdn+ ph(1,nb) *W2
            undn= undn+ 1.d0 *W2
          ENDDO
CCC   find logigudinal, transverse rapidities and their azimuthal ang.
CCC   #################################################################
CCC   NaN catchup
      if (iauxi.ge.5) then
	if (((uni+(tau*u3dn/u0dn))/(uni-(tau*u3dn/u0dn))).le.0.d0) then
      open(11,file=fnnan(1:nfnnan),access='append')
      write(11,*)'NaN in SPH particle decay!'
	write(11,'(a,2f16.8)')'u3dn,u0dn=',u3dn,u0dn 
      close(11)
	endif
	endif
CCC   #################################################################
         ph(0,nn)= phif(u1dn,u2dn) !option1
        al(0,nn)= half*dlog((uni+(tau*u3dn/u0dn))/(uni-(tau*u3dn/u0dn))) 
         al(0,nn)= al(0,nn)+ et(0,nn) 
cxx         bt(0,nn)= (u1dn/u0dn)*(u1dn/u0dn)+(u2dn/u0dn)*(u2dn/u0dn) !original~ bad convergence at big bt
cxx         bt(0,nn)= dsqrt(bt(0,nn)) *dcosh(al(0,nn)-et(0,nn))
cxx         bt(0,nn)= half*dlog((uni+bt(0,nn))/(uni-bt(0,nn)))
         v1dn=u1dn/u0dn !normalization of velocity
         v2dn=u2dn/u0dn
         v3dn=u3dn/u0dn
	   u0dn=1.d0/dsqrt(1.d0-v1dn**2-v2dn**2-v3dn**2*tau**2)
	   u1dn=v1dn*u0dn
	   u2dn=v2dn*u0dn
	   u3dn=v3dn*u0dn
         bt(0,nn)= dsqrt(u1dn*u1dn+u2dn*u2dn) !option1~ to avoid bad convergence and energy conservation problem
         bt(0,nn)= dlog(dsqrt(1.d0+bt(0,nn)**2)+bt(0,nn))

      if (((u0dn.gt.0.d0).and.(u0dn.le.1.d7)).and.
     &    (bt(0,nn).le.15.d0)) then
	    continue
	else
          if(iauxi.ge.8)then
          open(11,file=fnnan(1:nfnnan),access='append')
		WRITE(11,*)'bt nancatchup / overflow in decay!'
		WRITE(11,*)'particle splitting will be called off'
		WRITE(11,*)'na,nd=',na,nd
	    WRITE(11,*)'oflg=',oflg
		WRITE(11,*)'tau,nn,na1,na2=',tau,nn,na1,na2
		WRITE(11,'(a,2d20.10)') 'bt(0,nn),bt(1,na)=',bt(0,nn),bt(1,na)
		WRITE(11,'(a,4d20.10)') 'u0dn,u1dn,u2dn,u3dn=',
     &                             u0dn,u1dn,u2dn,u3dn
		WRITE(11,'(a,3d20.10)') 'xx,yy,et=',
     &                             xx(0,nn),yy(0,nn),et(0,nn)
		WRITE(11,'(a,2d20.10)') 'al,ph=',
     &                             al(0,nn),ph(0,nn)
          WRITE(11,'(a)') ' '
		CLOSE(11)
	    endif
          al(0,na)= al(1,na)
          bt(0,na)= bt(1,na)
          ph(0,na)= ph(1,na)
          xx(0,na)= xx(1,na)
          yy(0,na)= yy(1,na)
          et(0,na)= et(1,na)
          sn(0,na)= sn(1,na)
          br(0,na)= br(1,na)
          sr(0,na)= sr(1,na)
          qr(0,na)= qr(1,na)
          nd=nd-1
c		stop 'bt fatal error in decay!'
          return
      endif
CCC   #################################################################
      ENDDO
CCC   #################################################################
      RETURN 

CCC   #################################################################
 226  CONTINUE
      open(11,file=fnnsp(1:nfnnsp),access='append')
      write(11,*) 'np error in decay.f \(^^:)/'
      close(11)
      stop'SPheRIO: np error in decay_xy'
ccc   ////////////////////////////////////////////////      
 227  CONTINUE 
      open(11,file=fnnsp(1:nfnnsp),access='append')
      write(11,*) 'Nsp > NPL error in dectrl \(^^:)/'
      close(11)
      stop'Nsp > NPL error in decay_xy \(^^:)/'
CCC   #################################################################
      END       



CCC   #################################################################
CCC   #################################################################
CCC   #################################################################




CCC   #################################################################
CCC   subroutine program decay-control, dectrl.f
CCC   dectrl()
CCC   "dectrl" controls SPH particle decay. 
CCC   lastest updated 05042006
CCC   'spherio.eos.inc' is included
CCC   #################################################################
CCC   last updated 04202009
CCC   new baryon density check for particle splitting is implemented
CCC
CCC   last updated 09022008
CCC   avoid low bd splitting of massless pions
CCC
CCC   last udpated 07072008
CCC   fix a bug concerning modified EOM
CCC
CCC   last updated 08312007
CCC   adopt vvxn
CCC
CCC   last updated 08212007
CCC   add more iflgce check
CCC
CCC   last updated 08202007
CCC   adopt improved ihdmode check
CCC
CCC   last updated 08172007
CCC   temporarily restore the modification of ihdmode
CCC   and it is switched back
CCC
CCC   last updated 08142007
CCC   add ihdmode check to avoid unnecessary decay
CCC
CCC   last updated 05212007
CCC   adopt uxn
CCC
CCC   last updated 01232007
CCC   add debug for massless pion+bag, comment the criterian where the
CCC   non-zero-baryon-number of sph particle is less than 27, split
CCC
CCC   last updated 12192006
CCC   modified to incorporate tempxn in new chemical fo
CCC
CCC   last updated 10152006
CCC   incorporate chemical fo
CCC   #################################################################
      SUBROUTINE dectrl(tau)
      IMPLICIT NONE
      include 'spherio.inc'
      include 'spherio.eos.inc'
      include 'spherio.quifz.inc'
      include 'spherio.debug.inc'
      include 'spherio.temps.inc'
	include 'spherio.sphsplit.inc'
      include 'epos.incsf'
      INTEGER Nspp,nd,na,ns1,ns2
      DOUBLE PRECISION tau
      DOUBLE PRECISION snumax,bramax
CCC   #################################################################
CCC   SPheRIO Particle decay part:
CCC   decay criteria: 
CCC   #################################################################
CCC   in preparation.... transfer to flg=0 mode:  

      nd=0      
      Nsp(0)= Nsp(1) 
      Nspp=   Nsp(1) 
      snumax= zero 
      bramax= zero 

      oflg=1 
      CALL mklt()
      do 2001 na=1,Nsp(0)  
c         if ((iflgce(na).eq.2).and.(ihdmode.ne.0)) cycle
         xx(0,na)= xx(1,na)  
         yy(0,na)= yy(1,na)  
         et(0,na)= et(1,na)  
         al(0,na)= al(1,na)  
         bt(0,na)= bt(1,na)  
         ph(0,na)= ph(1,na)  
         sn(0,na)= sn(1,na)
         pibn(0,na)=pibn(1,na)
         sgmn(0,na)=sgmn(1,na)
         br(0,na)= br(1,na) 
         sr(0,na)= sr(1,na)
         qr(0,na)= qr(1,na)
         if(snumax.lt.sn(1,na)) then
            snumax=   sn(1,na)
            ns1=na
         end if
         if(bramax.lt.abs(br(1,na))) then 
            bramax=   abs(br(1,na))
            ns2=na
         end if                  
 2001 continue
CCC   #################################################################
      oflg=1
      CALL mklt()
      do 680 na=1,Nspp
         if ((iflgce(na).eq.2).and.(ihdmode.ne.0)) cycle
         eos(2)= ss(1,na)
         eos(3)= bd(1,na)
         call eosph(eos,2)
	    if (eos(5).ge.0.d0) then
		    continue
	    else
              if (iauxi.ge.8) then
              open(11,file=fnnan(1:nfnnan),access='append')
                WRITE(11,*)'eos(5) problem/nancatchup in cleantfin'
                WRITE(11,*)'current particle index,tau=',na,tau
                WRITE(11,*)'oflg=',oflg
                WRITE(11,'(a,4d20.10)') 'eos(1-4)',eos(1),eos(2),
     &                                           eos(3),eos(4)
                WRITE(11,'(a,3d20.10)') 'eos(5-7)',eos(5),eos(6),eos(7)
                WRITE(11,'(a,6d20.10)') 'xx,yy,et,ph,al,bt=',
     &                xx(oflg,na),yy(oflg,na),et(oflg,na),
     &                ph(oflg,na),al(oflg,na),bt(oflg,na)
                WRITE(11,'(a,4d20.10)') 'ss,sn,bd,br=',
     &                ss(oflg,na),sn(oflg,na),bd(oflg,na),br(oflg,na)
                CLOSE(11)
c                stop 'eos(5) fatal error in dectrl!'
              endif
              eos(5)=-1.d0
          endif
         if(eos(5).lt.Tfin*1.05d+00) cycle
         oflg=1
         CALL prcn(xx(1,na),yy(1,na),et(1,na))
	   if(((npflg.eq.0).and.(ihdmode.eq.0)).or.
     &      ((npflg.eq.0).and.(ihdmode.ne.0).and.(iflgce(na).eq.0))) !!the decoupled out SPH particles may stay single
     &                               goto 691     !! error message
CCC   #################################################################
         if(np.gt.512) cycle
         if((np.lt.npmn).and.
     &      ((abs(br(1,na)).gt.brmin).or.
     &       (ieosmode.ne.1.and.ieosmode.ne.2))) then
            CALL decay(tau,na,nd)
            cycle
         end if
       if((abs(br(1,na)).gt.brmax).and.(ieosmode.eq.1.or.ieosmode.eq.2))
     &   then
            CALL decay(tau,na,nd)
            cycle
         end if
CCC   #################################################################
CCC   comment the following since 01232007
c         nc=0 
c         do nn=1,np
c            nb=kl(nn) 
c            if(abs(br(1,nb)).gt.1.d-99) nc=nc+1
c         end do
c         if(nc.lt.npmn) then 
c            CALL decay(tau,na,nd) 
c            goto 680 
c         end if
CCC   #################################################################
         
CCC   #################################################################
c         oflg=1 
c         sumw=zero 
c         do nn=1,np
c            nb=kl(nn) 
c            QXY= dsqrt((xx(oflg,na)-xx(oflg,nb))**2
c     &           +(yy(oflg,na)-yy(oflg,nb))**2)/hxy
c            QET= abs(et(oflg,na)-et(oflg,nb))/het 
c            CALL krln(QXY,W2,2,12)
c            CALL krln(QET,W1,1,33)
c            sumw=sumw+W2*W1 
c         end do 
c         gamma= cosh(al(1,na)-et(1,na))*cosh(bt(1,na))
c         avsn= ss(1,na)  *tau *gamma /sumw 
c         avbr= bd(1,na)  *tau *gamma /sumw 
c         
c         dispsn= zero
c         dispbr= zero 
c         do nn=1,np
c            dispsn= dispsn+ (sn(1,nn)-avsn)**2 
c            dispbr= dispbr+ (br(1,nn)-avbr)**2             
c         end do
c         dispsn= dsqrt(dispsn/dble(np))
c         dispbr= dsqrt(dispbr/dble(np)) 
c
c         if(abs(sn(1,na)-avsn).gt.dispsn*2.d+00) then
c            CALL decay(tau,na,nd)
c            goto 680
c         end if
c         
c         if(abs(br(1,na)-avbr).gt.dispbr*2.d+00) then
c            CALL decay(tau,na,nd)
c            goto 680
c         end if
CCC   #################################################################
 680  continue
      

CCC   #################################################################
      Nsp(1)=Nsp(1)+nd
      Nsp(2)=Nsp(1)       !it is needed in solvediff() when oflg is set to 2
      if(Nsp(1).gt.NPL) goto 690  !! error message
CCC   #################################################################
      do na=1,Nsp(1)              !! back to flg=1 mode:  
CCC   #################################################################
c         if ((iflgce(na).eq.2).and.(ihdmode.ne.0)) cycle
         xx(1,na)= xx(0,na)
         yy(1,na)= yy(0,na)
         et(1,na)= et(0,na)
         al(1,na)= al(0,na)
         bt(1,na)= bt(0,na)
         ph(1,na)= ph(0,na)
         sn(1,na)= sn(0,na)
         pibn(1,na)=pibn(0,na)
         sgmn(1,na)=sgmn(0,na)
         br(1,na)= br(0,na)
         sr(1,na)= sr(0,na)
         qr(1,na)= qr(0,na)
CCC   #################################################################
         sn(2,na)= sn(1,na)       !used in solvediff() when oflg is set to 2 and fins() is called
         pibn(2,na)= pibn(1,na)
         sgmn(2,na)= sgmn(1,na)
         br(2,na)= br(1,na)
         sr(2,na)= sr(1,na)
         qr(2,na)= qr(1,na)
CCC   #################################################################
      end do
      oflg= 1
      CALL mklt()
CCC   #################################################################
      RETURN 

CCC   #################################################################
 690  CONTINUE
      open(11,file=fnnsp(1:nfnnsp),access='append')
      write(11,*) 'Nsp > NPL error in dectrl \(^^:)/'
      stop'SPheRIO:Nsp > NPL error' 
      close(11)      
CCC   #################################################################
 691  CONTINUE
      open(11,file=fnnsp(1:nfnnsp),access='append')
      write(11,*) 'np error in dectrl \(^^:)/'
	write(11,'(a,2i9)')'na,iflgce(na)=',na,iflgce(na)
	write(11,'(a,1i9)')'ihdmode=',ihdmode
      stop'SPheRIO: np error in dectrl'  
      close(11)
CCC   #################################################################
      END 



CCC   #################################################################
CCC   #################################################################
CCC   #################################################################



CCC   ##############################################################
CCC   subroutine program decay-control, dectrl_off.f 
CCC   "dectrl" controls SPH particle decay for massless pion
CCC   ##############################################################
CCC   last updated 08312007
CCC   adopt vvxn
CCC
CCC   last updated 02092007
CCC   updated to implement for general cases
CCC   renamed to dectrl_off()
CCC
CCC   last updated 09172006
CCC   ##############################################################
      SUBROUTINE dectrl_off(tau)
      IMPLICIT NONE
      include 'spherio.inc' 

      INTEGER na
      DOUBLE PRECISION tau
CCC   ##############################################################
      Nsp(2)=Nsp(1)
      do na=1,Nsp(1)
         sn(2,na)= sn(1,na)
         pibn(2,na)=pibn(1,na)
         sgmn(2,na)=sgmn(1,na)
         br(2,na)= br(1,na)
         sr(2,na)= sr(1,na)
         qr(2,na)= qr(1,na)
      end do
      oflg= 1
      CALL mklt()
CCC   ##############################################################
      RETURN 
      END 

CCC   ##############################################################
CCC   ##############################################################

CCC   #################################################################
CCC   SUBROUTINE PROGRAM: finds.f 
CCC   "finds" finds entropy density at 'na' -th particle position. 
CCC   #################################################################
CCC   last updated 08172007
CCC   fix a bug
CCC
CCC   last updated 08162007
CCC   fix a error
CCC
CCC   last updated 08142007
CCC   modified and add check
CCC
CCC   last updated 08092007
CCC   adopt ihdmode 1
CCC
CCC   last updated 02202007
CCC   adopt 2d case
CCC   #################################################################
      SUBROUTINE fins(tau,na)

      IMPLICIT NONE
      include 'spherio.inc'
	include 'spherio.quifz.inc'
	include 'spherio.debug.inc'
	include 'spherio.hydro.inc'
CCC   ////////////////////////////////////
      INTEGER nn,na,nb
      DOUBLE PRECISION QXY,QET,W2,W1
      DOUBLE PRECISION u0,tau
CCC   ////////////////////////////////////
	if (idimension.eq.2) then
	call fins_xy(tau,na)
	return
	endif


      ss(oflg,na)=zero 
CCC   #####################################################
      DO nn=1,np
         nb= kl(nn) 
         QXY= dsqrt((xx(oflg,na)-xx(oflg,nb))**2
     &            +(yy(oflg,na)-yy(oflg,nb))**2)/hxy
         QET= abs(et(oflg,na)-et(oflg,nb))/het
         CALL krln(QXY,W2,2,12)
         CALL krln(QET,W1,1,33)
         ss(oflg,na)= ss(oflg,na)+ sn(oflg,nb)*W2*W1
         select case (ihdmode)
	   case (0)
              continue
	   case (1)
	        if ((iflgce(nb).eq.2).and.(ihdfull.ne.1)) then
              stop'fatal ihdmode error in fins'
	        end if
	   case default
	        stop'ihdmode fatal error in fins'
	   end select
      END DO
      u0= dcosh(al(oflg,na)-et(oflg,na))*dcosh(bt(oflg,na))
      ss(oflg,na)=ss(oflg,na)/(tau*u0)
CCC   #####################################################
      RETURN
      END 



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



CCC   #################################################################
CCC   SUBROUTINE PROGRAM: findsb.f 
CCC   #################################################################
CCC   last updated 10192007
CCC   fix a bug concerning link table
CCC
CCC   last updated 10182007
CCC   fixed a bug in the definition of ppp
CCC
CCC   last updated 10172007
CCC   modified to adopt ppp
CCC
CCC   last updated 10092007
CCC   add more output
CCC
CCC   last updated 08172007
CCC   fix a bug
CCC
CCC   last updated 08162007
CCC   fix a error
CCC
CCC   last updated 08142007
CCC   modified and add check
CCC
CCC   last updated 08092007
CCC   adopt ihdmode 1
CCC
CCC   last updated 04192007
CCC   unp is redefined
CCC
CCC   last udpated 04182007
CCC   add the output for vx, vy and ve
CCC
CCC   last updated 02222007
CCC   modified according to 2d case
CCC
CCC   last updated 02202007
CCC   modified for 2d case
CCC
CCC   last updated 02190227
CCC   inherit from fins()
CCC   #################################################################
      SUBROUTINE finsb_gen(tau,xxp,yyp,etp,ssp,bbp,vx,vy,ve,ppp)
      IMPLICIT NONE
      include 'spherio.inc'
	include 'epos.incsf'
      include 'spherio.ce.inc'
	include 'spherio.quifz.inc'
	include 'spherio.debug.inc'
	include 'spherio.hydro.inc'
      INTEGER nn,nb
      DOUBLE PRECISION QXY,QET,W2,W1  
c      DOUBLE PRECISION u0
      DOUBLE PRECISION tau
	DOUBLE PRECISION xxp,yyp,etp,ssp,bbp
	DOUBLE PRECISION dpp,ppp
c	DOUBLE PRECISION alp,btp
      DOUBLE PRECISION unp
      DOUBLE PRECISION u0p,u1p,u2p,u3p
	DOUBLE PRECISION vx,vy,ve
CCC   ////////////////////////////////////
	if (idimension.eq.2) then
	call finsb_gen_xy(tau,xxp,yyp,etp,ssp,bbp,vx,vy,ve,ppp)
	return
	endif

      ssp=zero
	bbp=zero
	ppp=zero
c	alp=zero
c	btp=zero
      u0p=zero
	u1p=zero
	u2p=zero
	u3p=zero

	if (npflg.eq.0) return

CCC   #####################################################
      DO nn=1,np
         nb= kl(nn) 
         QXY= dsqrt((xxp-xx(oflg,nb))**2
     &            +(yyp-yy(oflg,nb))**2)/hxy
         QET= abs(etp-et(oflg,nb))/het
         CALL krln(QXY,W2,2,12)
         CALL krln(QET,W1,1,33)
          ssp= ssp+ sn(oflg,nb)*W2*W1
	    bbp= bbp+ br(oflg,nb)*W2*W1
cc        alp= alp+ al(oflg,nb)*W2*W1
cc        btp= btp+ bt(oflg,nb)*W2*W1
cc        unp= unp+ uni*W2*W1
          u0p= u0p+cosh(al(oflg,nb)-et(oflg,nb))*cosh(bt(oflg,nb))*W2*W1
          u1p= u1p+cos(ph(oflg,nb))*sinh(bt(oflg,nb))*W2*W1
          u2p= u2p+sin(ph(oflg,nb))*sinh(bt(oflg,nb))*W2*W1
          u3p= u3p+sinh(al(oflg,nb)-et(oflg,nb))*cosh(bt(oflg,nb))*W2*W1
     &                                                            /tau
         select case (ihdmode)
	   case (0)
          continue
	   case (1)
          if ((iflgce(nb).eq.2).and.(ihdfull.ne.1)) then
           open(22,file=fnfce(1:nfnfce),access='append')
           write(22,*)' '
           write(22,*)'fatal ihdmode error in findsb_gen!'
           write(22,*)'nn,np,nb,iflgce(nb),ihdfull,tau=',
     &          nn,np,nb,iflgce(nb),ihdfull,tau
           close(22)
           stop'fatal ihdmode error in findsb_gen'
          end if
	   case default
           stop'ihdmode fatal error in findsb_gen'
	   end select
      END DO

      if (ssp.ne.zero) then
	    unp=dsqrt(u0p**2-u1p**2-u2p**2-tau**2*u3p**2)
	    u0p=u0p/unp
	    u1p=u1p/unp
	    u2p=u2p/unp
	    u3p=u3p/unp
c         alp=alp/unp
c         btp=btp/unp
c         u0= dcosh(alp-etp)*dcosh(btp)
          ssp=ssp/(tau*u0p)
	    bbp=bbp/(tau*u0p)
          vx=u1p/u0p
	    vy=u2p/u0p
	    ve=u3p/u0p
	else
          ssp=zero
          bbp=zero
          vx=zero
          vy=zero
          ve=zero
	endif

      if (icemode.ne.0) then
          call probce_gen(tau,xxp,yyp,etp,u0p,vx,vy,ve,dpp,ppp)
      end if

CCC   #####################################################
      RETURN
      END 



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



CCC   #################################################################
CCC   SUBROUTINE PROGRAM: findsb.f 
CCC   #################################################################
CCC   last udpated 10192007
CCC   fix a bug concerning link table
CCC
CCC   last updated 10182007
CCC   fixed a bug in the definition of ppp
CCC
CCC   last updated 10172007
CCC   modified to adopt ppp
CCC
CCC   last updated 04192007
CCC   unp is redefined
CCC
CCC   last udpated 04182007
CCC   add output for vx, vy and ve
CCC
CCC   last updated 02202007
CCC   inherit from finsb_gen()
CCC   #################################################################
      SUBROUTINE finsb_gen_xy(tau,xxp,yyp,etp,ssp,bbp,vx,vy,ve,ppp)
      IMPLICIT NONE
      include 'spherio.inc'
	include 'spherio.debug.inc'
      include 'spherio.ce.inc'
      INTEGER nn,nb
      DOUBLE PRECISION QXY,QET,W2
c      DOUBLE PRECISION u0,W1
      DOUBLE PRECISION tau
	DOUBLE PRECISION xxp,yyp,etp,ssp,bbp,unp
      DOUBLE PRECISION ppp
c	DOUBLE PRECISION alp,btp,dpp
      DOUBLE PRECISION u0p,u1p,u2p,u3p
	DOUBLE PRECISION vx,vy,ve
CCC   ////////////////////////////////////
c	if (bugflg.ne.15) stop'bugflg fatal error in finsb_gen_xy()'
      ssp=zero
	bbp=zero
	ppp=zero
c	alp=zero
c	btp=zero
      u0p=zero
	u1p=zero
	u2p=zero
	u3p=zero
	    
      if (icemode.ne.0) then
          stop'probce not implemented for 2D case yet!'
      end if
CCC   #####################################################
      DO nn=1,np
         nb= kl(nn) 
         QXY= dsqrt((xxp-xx(oflg,nb))**2
     &            +(yyp-yy(oflg,nb))**2)/hxy
         QET= abs(etp-et(oflg,nb))/het
         CALL krln(QXY,W2,2,12)
c         CALL krln(QET,W1,1,33)
         ssp= ssp+ sn(oflg,nb)*W2  !rip W1 from these terms
	   bbp= bbp+ br(oflg,nb)*W2
c	   alp= alp+ al(oflg,nb)*W2
c	   btp= btp+ bt(oflg,nb)*W2
c        unp= unp+uni*W2
         u0p= u0p+cosh(al(oflg,nb)-et(oflg,nb))*cosh(bt(oflg,nb))*W2
         u1p= u1p+cos(ph(oflg,nb))*sinh(bt(oflg,nb))*W2
         u2p= u2p+sin(ph(oflg,nb))*sinh(bt(oflg,nb))*W2
         u3p= u3p+sinh(al(oflg,nb)-et(oflg,nb))*cosh(bt(oflg,nb))*W2
     &                                                            /tau
      END DO
	unp=dsqrt(u0p**2-u1p**2-u2p**2-tau**2*u3p**2)
	u0p=u0p/unp
	u1p=u1p/unp
	u2p=u2p/unp
	u3p=u3p/unp
c	alp=alp/unp
c	btp=btp/unp
c      u0= dcosh(alp-etp)*dcosh(btp)
      ssp=ssp/(tau*u0p)
	bbp=bbp/(tau*u0p)
      vx=u1p/u0p
	vy=u2p/u0p
	ve=u3p/u0p
CCC   #####################################################
      RETURN
      END 



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



CCC   #################################################################
CCC   SUBROUTINE PROGRAM: finds.f 
CCC   "finds" finds entropy density at 'na' -th particle position. 
CCC   #################################################################
CCC   last updated 02202007
CCC   inhert from fins()
CCC   #################################################################
      SUBROUTINE fins_xy(tau,na)

      IMPLICIT NONE
      include 'spherio.inc'
	include 'spherio.debug.inc'
CCC   ////////////////////////////////////
      INTEGER nn,na,nb
      DOUBLE PRECISION QXY,QET,W2
c	double precision W1
      DOUBLE PRECISION u0,tau
CCC   ////////////////////////////////////
c	if (bugflg.ne.15) stop'bugflg fatal error in fins_xy()'


      ss(oflg,na)=zero 
CCC   #####################################################
      DO nn=1,np
         nb= kl(nn) 
         QXY= dsqrt((xx(oflg,na)-xx(oflg,nb))**2
     &            +(yy(oflg,na)-yy(oflg,nb))**2)/hxy
         QET= abs(et(oflg,na)-et(oflg,nb))/het
         CALL krln(QXY,W2,2,12)
c         CALL krln(QET,W1,1,33)
         ss(oflg,na)= ss(oflg,na)+ sn(oflg,nb)*W2  !rip the W1 part
      END DO
      u0= dcosh(al(oflg,na)-et(oflg,na))*dcosh(bt(oflg,na))
      ss(oflg,na)=ss(oflg,na)/(tau*u0)
CCC   #####################################################
      RETURN
      END 



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



CCC   #################################################################
CCC   subroutine program for calculating of kernel function
CCC   'krln' and 'krld' calculate kernel function and its derivative. 
CCC   #################################################################
CCC   last updated 06012007
CCC   modified to adopt bugflg 30
CCC
CCC   last updated 03052007
CCC   use a different expression to rip off the singularity at zero
CCC   #################################################################
      SUBROUTINE krln(Q,WW,dflg,hflg)

      IMPLICIT NONE
      include 'spherio.inc'
	include 'epos.incsf'
      DOUBLE PRECISION hh,Q,WW,fnr
      INTEGER dflg,hflg
      IF(hflg.eq.12) THEN
         hh=hxy
      ELSE IF(hflg.eq.33) THEN
         hh=het
      ELSE
         GOTO 900
      END IF
      IF(dflg.eq.1) THEN
         fnr=2.d0/3.d0/hh
      ELSE IF(dflg.eq.2) THEN
         fnr=10.d0/(7.d0*pi)/(hh*hh)
      ELSE IF(dflg.eq.3) THEN
         fnr=1.d0/pi/(hh*hh*hh)
      ELSE
         GOTO 900
      END IF
      IF(Q.LE.uni) THEN
         WW=1.d0-1.5d0*(Q*Q)+0.75d0*(Q*Q*Q)
      ELSE IF(Q.LE.two) THEN
         WW=(2.d0-Q)*(2.d0-Q)*(2.d0-Q)/4.d0
      ELSE
         WW=0.0d0
      END IF
      WW=WW*fnr
      RETURN
      
 900  open(11,file=fnnan(1:nfnnan),access='append')
      write(11,*) 'somthing wrong in kernel function' 
      close(11)
      stop'sth wrong in kernel function'
      END 

CCC   #################################################################
CCC   For calculation of derivative of Kernel function W 
      SUBROUTINE krld(Q,DW,dflg,hflg,za,zb,kflg)
      IMPLICIT NONE
      include 'spherio.inc'
	include 'epos.incsf'
      include 'spherio.debug.inc'
      DOUBLE PRECISION hh,Q,DW,fnr 
      DOUBLE PRECISION za(3),zb(3) 
      INTEGER kflg,hflg,dflg

      IF (bugflg.eq.30) THEN
          CALL krld_sin(Q,DW,dflg,hflg,za,zb,kflg)
	    RETURN
	END IF

      IF(hflg.eq.12) THEN
         hh= hxy
      ELSE if(hflg.eq.33) THEN
         hh= het 
      ELSE
         GOTO 950
      END IF
      IF(dflg.eq.1) THEN
         fnr=2.d0/3.d0/hh
      ELSE IF(dflg.eq.2) THEN
         fnr=10.d0/(7.d0*pi)/(hh*hh)
      ELSE IF(dflg.eq.3) THEN
         fnr=1.d0/pi/(hh*hh*hh)
      ELSE
         GOTO 950
      END IF
      IF(Q.LE.uni) THEN
         DW=-3.d0+2.25d0*Q
      ELSE IF(Q.LE.two) THEN
         DW=-0.75d0*(2.d0-Q)*(2.d0-Q)
      ELSE
         DW=0.0d0
      END IF
      IF(Q.LE.uni) THEN
         DW=DW*(za(kflg)-zb(kflg))
      ELSE IF(Q.LE.two) THEN 
         DW=DW*(za(kflg)-zb(kflg))/Q
      ELSE
         DW=0.0d0
      END IF
      DW=DW*fnr/(hh*hh)
      RETURN
 950  open(11,file=fnnan(1:nfnnan),access='append')
      write(11,*) 'somthing else wrong in kernel function'
      close(11)
      stop'sth else wrong in kernel function'
      END
CCC   #################################################################
CCC   #################################################################
CCC   #################################################################






CCC   #################################################################
CCC   subroutine program for calculating of kernel function
CCC   'krln' and 'krld' calculate kernel function and its derivative. 
CCC   #################################################################
CCC   last updated 06012007
CCC   old kernal function with singularity
CCC   #################################################################
      SUBROUTINE krld_sin(Q,DW,dflg,hflg,za,zb,kflg)
      IMPLICIT NONE
      include 'spherio.inc' 
      DOUBLE PRECISION hh,Q,DW,fnr 
      DOUBLE PRECISION za(3),zb(3) 
      INTEGER kflg,hflg,dflg 
CCC   /////////////////////////////////////////////////////////////
      IF(Q.LE.uni) THEN 
         DW = -3.00d+00*Q + (9.00d+00/4.00d+00)*(Q*Q)
      ELSE IF(Q.LE.two) THEN 
         DW = -0.75d+00*(2.00d+00-Q)*(2.00d+00-Q)
      ELSE
         DW = 0.000d+00 
      END IF
      IF(Q.LE.epsilon) THEN 
         DW= ZERO
      ELSE
         DW= DW* (za(kflg)-zb(kflg))/Q
      END IF
c     //////////////////////////////////////////////
      if(hflg.eq.12) then 
         hh= hxy
      else if(hflg.eq.33) then 
         hh= het 
      else
         goto 966
      end if
c     //////////////////////////////////////////////
      if(dflg.eq.1)  then 
         fnr=  (2.d+00)/(3.d+00)    /(hh) 
      else if(dflg.eq.2) then 
         fnr= (10.d+00)/(7.d+00*pi) /(hh*hh) 
      else if(dflg.eq.3) then 
         fnr=  (1.d+00)/(pi)        /(hh*hh*hh) 
      else
         goto 966
      end if
c     //////////////////////////////////////////////
      DW= DW* fnr 
      RETURN
 966  open(11,file='log.file',status='unknown')
      write(6,*) 'somthing wrong in kernel_sin function' 
      close(10) 
      stop 900
CCC   ####################################################################
      END 
CCC   #################################################################
CCC   #################################################################
CCC   #################################################################






CCC   #################################################################
CCC   SUBROUTINE PROGRAM: makelist.f: 
CCC   "mklt" makes a list for List-Linked-Method
CCC   #################################################################
CCC   last updated 10092007
CCC   fixed a very old bug(?)
CCC
CCC   last updated 08172007
CCC   temporarily remarked the modification concerning idmode
CCC   and it is switched back
CCC
CCC   last updated 08162007
CCC   adopt ihdfull and fix a error
CCC
CCC   last updated 08142007
CCC   adopt ihdmode 1                 
CCC   #################################################################
      SUBROUTINE mklt()
      IMPLICIT NONE 
      include 'spherio.inc'
      include 'spherio.quifz.inc'
	include 'epos.incsf'
	include 'spherio.debug.inc'
ccc   ------------------------------------------
      INTEGER nn,i1,j1,k1,i2,j2,k2,imx,jmx,kmx
      DOUBLE PRECISION rho,eta
ccc   ------------------------------------------
      xymx(oflg)= zero
      etmx(oflg)= zero
      DO nn=1,Nsp(oflg)
CCC   #################################################################
CCC   debug output
c	      if (iauxi.ge.2) then
c            if ((xx(oflg,nn).le.0.d0).or.(xx(oflg,nn).ge.0.d0)) then
c              continue
c            elseif (iauxi.ge.5) then
c              open(11,file=fnnan(1:nfnnan),access='append')
c		    write(11,*)'nancatchup xx(1,nn) in mklt'
c		    write(11,*)'oflg=',oflg
c		    write(11,*)'SPH particle number nn=',nn
c		    write(11,'(a,3d30.20)') 'xx(-2,nn),yy(-2,nn),et(-2,nn)=',
c     &                                 xx(-2,nn),yy(-2,nn),et(-2,nn)
c		    write(11,'(a,3d30.20)') 'xx(-1,nn),yy(-1,nn),et(-1,nn)=',
c     &                                 xx(-1,nn),yy(-1,nn),et(-1,nn)
c		    write(11,'(a,3d30.20)') 'xx(0,nn),yy(0,nn),et(0,nn)=',
c     &                                 xx(0,nn),yy(0,nn),et(0,nn)
c		    write(11,'(a,3d30.20)') 'xx(1,nn),yy(1,nn),et(1,nn)=',
c     &                                 xx(1,nn),yy(1,nn),et(1,nn)
c		    write(11,'(a,3d30.20)') 'xx(2,nn),yy(2,nn),et(2,nn)=',
c     &                                 xx(2,nn),yy(2,nn),et(2,nn)
c		    write(11,'(a,3d30.20)') 'xx(oflg,nn-1),yy(oflg,nn-1),',
c     &                                'et(oflg,nn-1)=',
c     &                                 xx(oflg,nn-1),yy(oflg,nn-1),
c     &                                 et(oflg,nn-1)
c        		close(11)
c              stop'nancatchup xx(oflg,nn) in mklt'
c	      else
c              stop'nancatchup xx(oflg,nn) in mklt'
c            endif
c	      endif
CCC   #################################################################
         rho= dsqrt(xx(oflg,nn)*xx(oflg,nn)+yy(oflg,nn)*yy(oflg,nn))
         eta= abs(et(oflg,nn))
         If(rho.ge.xymx(oflg)) xymx(oflg)= rho
         IF(eta.ge.etmx(oflg)) etmx(oflg)= eta
      END DO

c      DO nn=1,Nsp(oflg)
c         i1= INT((xx(oflg,nn)-(-xymx(oflg)))/hxy)+1
c         j1= INT((yy(oflg,nn)-(-xymx(oflg)))/hxy)+1
c         k1= INT((et(oflg,nn)-(-etmx(oflg)))/het)+1
c         DO 101 i2=i1-RL,i1+RL
c            if(i2.le.0)  goto 101
c            if(i2.gt.LX) goto 101
c            DO 102 j2=j1-RL,j1+RL
c               if(j2.le.0)  goto 102
c               if(j2.gt.LX) goto 102
c               DO 103 k2=k1-RL,k1+RL
c                  if(k2.le.0)  goto 103
c                  if(k2.gt.LX) goto 103
ccc   =========================================
c                  if(oflg.lt.0) then
c                     hd(oflg,i2,j2,k2)= 0
c                  else
c                     hd(0,i2,j2,k2)= 0
c                  end if
ccc   =========================================
c 103           CONTINUE
c 102        CONTINUE
c 101     CONTINUE
c      END DO 


      DO i2=1,LX
            DO j2=1,LX
               DO k2=1,LX
ccc   =========================================
                  if(oflg.lt.0) then
                     hd(oflg,i2,j2,k2)= 0
                  else
                     hd(0,i2,j2,k2)= 0
                  end if
ccc   =========================================
               ENDDO
            ENDDO
      ENDDO


      imx=0 
      jmx=0
      kmx=0
      DO nn=1,Nsp(oflg)          
         i1= INT((xx(oflg,nn)-(-xymx(oflg)))/hxy)+1
         j1= INT((yy(oflg,nn)-(-xymx(oflg)))/hxy)+1
         k1= INT((et(oflg,nn)-(-etmx(oflg)))/het)+1

         if(i1.gt.imx) imx= i1
         if(j1.gt.imx) jmx= j1
         if(k1.gt.imx) kmx= k1
CCC   #################################################################
CCC   debug output
            if(imx.gt.LX) then
	      if (iauxi.ge.2) then
              open(11,file=fnnan(1:nfnnan),access='append')
		    write(11,*)'imx overflow in mklt'
		    write(11,'(a,2i7,d30.20)')'oflg,imx,xymx(oflg)=',
     &                                 oflg,imx,xymx(oflg)
		    write(11,*)'SPH particle number nn=',nn
		    write(11,'(a,3d30.20)') 'xx(-2,nn),yy(-2,nn),et(-2,nn)=',
     &                                 xx(-2,nn),yy(-2,nn),et(-2,nn)
		    write(11,'(a,3d30.20)') 'xx(-1,nn),yy(-1,nn),et(-1,nn)=',
     &                                 xx(-1,nn),yy(-1,nn),et(-1,nn)
		    write(11,'(a,3d30.20)') 'xx(0,nn),yy(0,nn),et(0,nn)=',
     &                                 xx(0,nn),yy(0,nn),et(0,nn)
		    write(11,'(a,3d30.20)') 'xx(1,nn),yy(1,nn),et(1,nn)=',
     &                                 xx(1,nn),yy(1,nn),et(1,nn)
		    write(11,'(a,3d30.20)') 'xx(2,nn),yy(2,nn),et(2,nn)=',
     &                                 xx(2,nn),yy(2,nn),et(2,nn)
        		close(11)
            endif
              stop 'nancatchup imx in mklt'
	      endif
CCC   #################################################################

ccc   ////////////////////////////////////////////
         select case (ihdmode)
         case (0)
                continue
         case (1)
                if ((iflgce(nn).eq.2).and.(ihdfull.eq.0)) cycle
	   case default
	        stop'ihdmode fatal error in makelist'
         end select

         if(oflg.lt.0) then 
            lt(oflg,nn) = hd(oflg,i1,j1,k1)
            hd(oflg,i1,j1,k1)= nn 
         else 
            lt(0,nn) = hd(0,i1,j1,k1)
            hd(0,i1,j1,k1)= nn
         end if
ccc   ////////////////////////////////////////////
      END DO      
ccc   ############################################
ccc   error massages: 
      if(imx.gt.LX) then
         open(11,file=fnnsp(1:nfnnsp),access='append')
         write(11,*) ' imx > LX in makelist.f,imx=',imx
         CLOSE(11)
         STOP' imx > LX in makelist.f'
      end if
      if(jmx.gt.LX) then
         open(11,file=fnnsp(1:nfnnsp),access='append')
         WRITE(11,*) ' jmx > LX in makelist.f, jmx=',jmx
         CLOSE(11)
         STOP  ' jmx > LX in makelist.f'
      end if
      if(kmx.gt.LX) then
         open(11,file=fnnsp(1:nfnnsp),access='append')
         WRITE(11,*) ' kmx > LX in makelist.f, kmx=',kmx
         CLOSE(11)
         STOP' kmx > LX in makelist.f'
      end if
ccc   ############################################
      RETURN 
      END 


C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



CCC   #################################################################
CCC   SUBROUTINE PROGRAM: parcon.f   
CCC   "prcn" picks up SPH prticles around near 'xxp','yyp','etp'
CCC   using link-list. 
CCC   --> 'np' and 'kl' are outputs of this routine:
CCC       'np' is the totsl number of SPH particles 
CCC        within 2h around 'xxp','yyp','etp'.  
CCC   --> 'kl(n)' gives the particle number 
CCC        assigned in SPheRIO.  
CCC   #################################################################
CCC   last updated 08212007
CCC   turn off debug output for null npflg
CCC
CCC   last updated 01212007
CCC   modified and added debugging info
CCC   #################################################################
      SUBROUTINE prcn(xxp,yyp,etp)
      IMPLICIT NONE
      include 'spherio.inc'
	include 'spherio.quifz.inc'
	include 'spherio.debug.inc'
	include 'epos.incsf'

      INTEGER nnb(NPX),indx(NPX),nb,nn,mm
      INTEGER i1,j1,k1,i2,j2,k2
      DOUBLE PRECISION xxp,yyp,etp
      DOUBLE PRECISION QXY,QET,W2,W1 
      DOUBLE PRECISION arr(NPX) 
ccc   ///////////////////////////////////////////////////
      npflg=1  !! npflg initialize: npflg=0 <--> np=0 
      i1= INT((xxp-(-xymx(oflg)))/hxy)+1 
      j1= INT((yyp-(-xymx(oflg)))/hxy)+1 
      k1= INT((etp-(-etmx(oflg)))/het)+1 
ccc   //////////////////////////////////////////////////
      
      np=0 
      DO 101 i2=i1-RL,i1+RL
         if(i2.le.0) goto 101
         if(i2.gt.LX) goto 101 
         DO 102 j2=j1-RL,j1+RL
            if(j2.le.0) goto 102
            if(j2.gt.LX) goto 102 
            DO 103 k2=k1-RL,k1+RL            
               if(k2.le.0) goto 103
               if(k2.gt.LX) goto 103
ccc   ===========================================
               if(oflg.lt.0) then 
                  nb= hd(oflg,i2,j2,k2)
               else 
                  nb= hd(0,i2,j2,k2)
               end if
ccc   ===========================================
 201           CONTINUE               
               IF(nb.eq.0) GOTO 103 
ccc   //////////////////////////////////////////////
               QXY= dsqrt((xxp-xx(oflg,nb))**2
     &                  +(yyp-yy(oflg,nb))**2)/hxy
               QET=  abs(etp-et(oflg,nb))/het 
               IF(QXY.GE.two) GOTO 150  
               IF(QET.GE.two) GOTO 150  
               np=np+1               
               CALL krln(QXY,W2,2,12) 
               CALL krln(QET,W1,1,33) 
ccc   //////////////////////////////////////////////
               arr(np)= W2*W1 
               nnb(np)= nb 
               if(np.gt.NPX) goto 221  
 150           CONTINUE
ccc   ====================================
               if(oflg.lt.0) then 
                  nb=lt(oflg,nb)
               else 
                  nb=lt(0,nb)
               end if
ccc   ====================================
               GOTO 201 
 103        CONTINUE
 102     CONTINUE
 101  CONTINUE
ccc   /////////////////////////////
      CALL indexx(np,NPX,arr,indx)
      mm=0 
      do nn=np,1,-1 
         mm=mm+1 
         kl(mm)=nnb(indx(nn))
      end do 
ccc   //////////////////////////
ccc   ##########################
      if(np.eq.0) then
	npflg=0 
      if ((iauxi.ge.5).and.(ihdmode.eq.0)) then
      open(11,file=fnnsp(1:nfnnsp),access='append')
      write(11,*) 'npflg=0 in parcon.f \(^^:)/ '
      write(11,'(a,3d30.20)') 'xxp,yyp,etp=',xxp,yyp,etp
      write(11,*) 'np=',np
	write(11,'(a,2d30.20)') 'xymx,etmx=',xymx(oflg),etmx(oflg)
	write(11,*) 'i1,j1,k1=',i1,j1,k1
      close(11)
	endif
	endif
ccc   ##########################
      RETURN
ccc   ////// ERROR MESSAGE  //////////////////////
 221  continue 
      open(11,file=fnnsp(1:nfnnsp),access='append')
      write(11,*) 'np>NPX error in parcon.f \(^^:)/ '
      write(11,'(a,3d30.20)') 'xxp,yyp,etp=',xxp,yyp,etp
      write(11,*) 'np=',np
	write(11,'(a,2d30.20)') 'xymx,etmx=',xymx(oflg),etmx(oflg)
	write(11,*) 'i1,j1,k1=',i1,j1,k1
      close(11)
      stop'SPheRIO: np> NPX error in parcon '
ccc   ///////////////////////////////////////////
      END





C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



