CCC   #################################################################
CCC   SUBROUTINE PROGRAM: dqfind.f 
CCC   EOM to calulate the bulk viscosity
CCC   In collaboration with Philipe Mota and Takashi Kodama
CCC   #################################################################
CCC   Convention of dq_vis
CCC   1 - u1 - du1/dtau
CCC   2 - u2 - du2/dtau
CCC   3 - u3 - du3/dtau
CCC   4 - entropy ratio (sn/sgmn) - d(sn/sgmn)/dtau
CCC   5 - bulk viscosity ratio (pibn/sgmn) - d(pibn/sgmn)/dtau
CCC   6 - 10 shear viscosity (reserved)
CCC   #################################################################
CCC   the equation to solve is
CCC   aa-matrix * dq_vis-vector = bb-vector
CCC   we use m-matrix to temporarily store aa-matrix and bb-matrix
CCC   before sending them to gaussj() to solve
CCC   for convenience
CCC   m(i,j)=aa(i,j)
CCC   m(i,0)=-r(i)
CCC   but there are so many terms
CCC   first let's define some frequently used term
CCC   1) ddotu(0:nrank)
CCC   ddotu=partial.dot.u=partial_\mu*u^\mu
CCC   ddotu(0)->r() NOT including the negative sign
CCC   ddotu(1)->m(,1) contribution proportional to du1
CCC   dtrhos is d(rho*)/dtau where tau is t' not proper time
CCC   divv is div_i*v^i
CCC   2) udotde(0:nrank)
CCC   udotde=u.dot.de
CCC   udotde(0)->r() NOT including the negative sign
CCC   udotde(1)->m(,1) contribution proportional to du1
CCC   3) udotdpi(0:nrank)
CCC   4) udotdsgm(0:nrank)
CCC   5) udotdui(0:nrank)
CCC   #################################################################
CCC   last updated 06182009
CCC   modified to fix an error in the formulae
CCC
CCC   last updated 06112009
CCC   the framework is built
CCC   #################################################################
      SUBROUTINE dqfn_bulk(tau,na,dq_vis)

      IMPLICIT NONE
      include 'spherio.inc' 
      include 'spherio.eos.inc'
	include 'spherio.quifz.inc'
	include 'spherio.debug.inc'
	include 'spherio.hydro.inc'
	include 'spherio.viscosity.inc'
	include 'epos.incsf'
      DOUBLE PRECISION QXY,QET,DW2,DW1,W2,W1,tau
	double precision dtrhos,divv
      DOUBLE PRECISION Pa,Ptla,edna,Ta,ua(0:3),za(3),csa2
	double precision uasub(1:3)
      DOUBLE PRECISION Pb,Ptlb,ednb,ub(0:3),zb(3)
	double precision dqb
	integer nrank
      parameter (nrank=5)
      DOUBLE PRECISION m(1:nrank,0:nrank)
      double precision aa(1:nrank,1:nrank),bb(1:nrank)
      DOUBLE PRECISION dq_vis(NPL,5)
      INTEGER na,nb,nn,kflg
	integer iloop
	double precision ddotu(0:nrank),udotde(0:nrank)
	double precision udotdpi(0:nrank),udotdsgm(0:nrank)
      double precision udotdui(0:nrank)

     	if (idimension.eq.2) then
      write(*,*) 'dqfn_bulk_xy() is not supported yet'
	stop 'dqfn_bulk_xy() is not supported yet'
	endif

	do kflg=1,nrank
      dq_vis(na,kflg)= zero
	do iloop=0,nrank
	m(kflg,iloop)= zero
	enddo
      enddo

	do iloop=0,nrank
	ddotu(iloop)= zero
	udotde(iloop)= zero
	udotdpi(iloop)= zero
	udotdsgm(iloop)= zero
	udotdui(iloop)= zero
      enddo

      za(1)= xx(oflg,na)
      za(2)= yy(oflg,na)
      za(3)= et(oflg,na)

      ua(0) = cosh(al(oflg,na)-et(oflg,na))*cosh(bt(oflg,na))
      ua(1) = cos(ph(oflg,na))*sinh(bt(oflg,na))
      ua(2) = sin(ph(oflg,na))*sinh(bt(oflg,na))
      ua(3) = sinh(al(oflg,na)-et(oflg,na))*cosh(bt(oflg,na))/tau

      uasub(1) = -ua(1)
      uasub(2) = -ua(2)
      uasub(3) = -ua(3)*tau**2

      eos(2)= ss(oflg,na)
      eos(3)= bd(oflg,na)
      call eosph(eos,2)
      edna= eos(1)
      Pa=   eos(4)
	Ptla = Pa+pibd(oflg,na)
	Ta=   eos(5)
      call eosph_cs2(eos,2,csa2)
CCC   #################################################################
CCC   ddotu(0:nrank)
      ddotu(0)=ua(0)/tau
	do kflg=1,3
	ddotu(kflg)=-uasub(kflg)/ua(0)
      enddo
      ddotu(0)=ddotu(0)+ua(3)**2*tau/ua(0)

      DO nn=1,np
         nb=kl(nn)
         dtrhos= zero
         zb(1)= xx(oflg,nb)
         zb(2)= yy(oflg,nb)
         zb(3)= et(oflg,nb)
         ub(0) = cosh(al(oflg,nb)-et(oflg,nb))*cosh(bt(oflg,nb))
         ub(1) = cos(ph(oflg,nb))*sinh(bt(oflg,nb))
         ub(2) = sin(ph(oflg,nb))*sinh(bt(oflg,nb))
         ub(3) = sinh(al(oflg,nb)-et(oflg,nb))*cosh(bt(oflg,nb))/tau
         QXY= sqrt((za(1)-zb(1))**2+(za(2)-zb(2))**2)/hxy
         QET=  abs(za(3)-zb(3))/het
         DO kflg=1,3 
            if(kflg.ne.3) then 
               call krld(QXY,DW2,2,12,za,zb,kflg)
               CALL krln(QET,W1,1,33)
               dtrhos=divv+sgmn(oflg,nb)*(ua(kflg)/ua(0)-ub(kflg)/ub(0))
     &             *DW2*W1
            else if(kflg.eq.3) then
               call krln(QXY,W2,2,12)
               call krld(QET,DW1,1,33,za,zb,kflg)
               dtrhos=divv+sgmn(oflg,nb)*(ua(kflg)/ua(0)-ub(kflg)/ub(0))
     &             *W2*DW1
            endif
         ENDDO ! of kflg
      ENDDO ! of nn
      divv= -1.d0/sgmd(oflg,na)/(tau*ua(0))*dtrhos
	ddotu(0)= ddotu(0)+ua(0)*divv

CCC   #################################################################
CCC   udotde(0:nrank)
	do kflg=0,5
	udotde(kflg)= udotde(kflg)-(edna+Pa)*ddotu(kflg)
      enddo
      udotde(5)= Ta*sgmd(oflg,na)*ua(0)

CCC   #################################################################
CCC   udotdpi(0:nrank)
      udotdpi(0)= -pibd(oflg,na)/vtaur
	do kflg=0,5
	udotdpi(kflg)= 
     &    udotdpi(kflg)-(vzeta/vtaur+pibd(oflg,na))*ddotu(kflg)
      enddo

CCC   #################################################################
CCC   udotdsgm(0:nrank)
      udotdsgm(0)= -sgmd(oflg,na)*ua(0)/tau
	do kflg=0,5
	udotdsgm(kflg)= udotdsgm(kflg)-sgmd(oflg,na)*ddotu(kflg)
      enddo

CCC   #################################################################
CCC   udotdui(0:nrank)
      do kflg=1,2
      udotdui(kflg)=-ua(0)
	enddo
      udotdui(0)=-2.d0*tau*ua(0)*ua(3)
      udotdui(3)=-ua(0)*tau**2

CCC   #################################################################
CCC   equation 1-3
      do kflg=1,nrank
      m(kflg,kflg)=m(kflg,kflg)+(edna+Pa+pibd(oflg,na))*udotdui(kflg)
	enddo
      do iloop=1,3
      do kflg=0,nrank
      m(iloop,kflg)=m(iloop,kflg)+
     &                     (1.d0+csa2**2)*uasub(iloop)*udotde(kflg)
	m(iloop,kflg)=m(iloop,kflg)+uasub(iloop)*udotdpi(kflg)
	m(iloop,kflg)=m(iloop,kflg)+uasub(iloop)*
     &                     (edna+Pa+pibd(oflg,na))*ddotu(kflg)
	enddo
	enddo
      DO nn=1,np
         nb=kl(nn) 
         zb(1)= xx(oflg,nb)
         zb(2)= yy(oflg,nb)
         zb(3)= et(oflg,nb)
         ub(0) = cosh(al(oflg,nb)-et(oflg,nb))*cosh(bt(oflg,nb))
         eos(2)= ss(oflg,nb)
         eos(3)= bd(oflg,nb)
         call eosph(eos,2)
         ednb= eos(1)
         Pb=   eos(4)
         Ptlb= Pb+pibd(oflg,nb)
         dqb = -sgmn(oflg,nb)*
     &         (Ptla/(sgmd(oflg,na)**2 *ua(0)**2)+
     &          Ptlb/(sgmd(oflg,nb)**2 *ub(0)**2))/tau
	   dqb = dqb*sgmd(oflg,na)*ua(0)  !!the factor different from r.h.s of old EOM
         QXY= sqrt((za(1)-zb(1))**2+(za(2)-zb(2))**2)/hxy
         QET=  abs(za(3)-zb(3))/het 
         DO kflg=1,3 
            if(kflg.ne.3) then 
               call krld(QXY,DW2,2,12,za,zb,kflg)
               CALL krln(QET,W1,1,33)
               m(kflg,0)=m(kflg,0)+ dqb* DW2*W1
            else if(kflg.eq.3) then
               call krln(QXY,W2,2,12)
               call krld(QET,DW1,1,33,za,zb,kflg)               
               m(kflg,0)=m(kflg,0)+ dqb* W2*DW1
            endif
         ENDDO ! of kflg
      ENDDO ! of nn

CCC   #################################################################
CCC   equation 4
      m(4,4)= Ta*sgmd(oflg,na)*ua(0)
      do kflg=0,5
	m(4,kflg)= m(4,kflg)+pibd(oflg,na)*ddotu(kflg)
	enddo

CCC   #################################################################
CCC   equation 5
      m(5,5)= 1.d0
      do kflg=0,5
	m(5,kflg)= m(5,kflg)-udotdpi(kflg)/sgmd(oflg,na)/ua(0)
	m(5,kflg)= m(5,kflg)+pibd(oflg,na)*udotdsgm(kflg)
     &                              /sgmd(oflg,na)**2/ua(0)
	enddo

	do kflg=1,nrank
	bb(kflg)= -m(kflg,0)
	do iloop=0,nrank
	aa(kflg,iloop) =m(kflg,iloop)
	enddo
      enddo

      call gaussj(aa,nrank,nrank,bb,1,1)
      do kflg=1,nrank
      dq_vis(na,kflg)= bb(kflg)
      enddo

CCC   #################################################################
CCC   debug purpose only
cxx   do kflg=1,nrank
cxx   dq_vis(na,kflg)= zero 
cxx   enddo
CCC   #################################################################

      RETURN 
      END



CCC   #################################################################
CCC   #################################################################
CCC   #################################################################





CCC   #################################################################
CCC   SUBROUTINE PROGRAM: dqfind.f 
CCC   "dqq" calculates dq_i/dtau 
CCC   A term appears in the 'momentum equation' of SPH.  
CCC
CCC   dq_a                 1      P_a+Q_a       1       P_b+Q_b         d W
CCC   ---- = - sum_b [--------- --------- + ---------- ---------]*sn_b* ----  
CCC   dtau            tau*u0_a^2   s_a^2    tau*u0_b^2   s_b^2          dx_a
CCC
CCC   #################################################################
CCC   last updated 08172007
CCC   temporarily switch back idmode modification
CCC   and it is switched back
CCC
CCC   last updated 08162007
CCC   switch back last modification
CCC
CCC   last updated 08092007
CCC   modified to adopt ihdmode 1
CCC
CCC   last updated 06012007
CCC   modified to adopt new bugflg 30
CCC
CCC   last updated 02202007
CCC   fix a bug and disable bugflg=7
CCC   adop bugflg=8 2d case
CCC
CCC   last updated 02192007
CCC   add more debugging info
CCC   modified to adopt transverse expansion case
CCC
CCC   last updated 09082006
CCC   add debugging output to check the difference between two sets of
CCC   hydro eq.
CCC
CCC   last updated 07052006
CCC   fix the bug in formulae
CCC
CCC   last updated 05042006
CCC   include 'spherio.eos.inc' to incorporate the strangeness
CCC   #################################################################
      SUBROUTINE dqfn(tau,na,dq,ncall)

      IMPLICIT NONE
      include 'spherio.inc' 
      include 'spherio.eos.inc'
	include 'spherio.quifz.inc'
	include 'spherio.debug.inc'
	include 'spherio.hydro.inc'
	include 'epos.incsf'
	integer ncall
c     ////////////////////////////////////////////
      DOUBLE PRECISION QXY,QET,DW2,DW1,W2,W1,tau
      DOUBLE PRECISION Pa,edna,u0a,za(3)
      DOUBLE PRECISION Pb,ednb,u0b,zb(3)
      DOUBLE PRECISION dqb,dq(NPL,3) 
      INTEGER na,nb,nn,kflg

c	double precision dqbold,dqbnew
c	double precision dqold(3),dqnew(3)
c	double precision uu0,gammat
c     ////////////////////////////////////////////
     	if (idimension.eq.2) then
      call dqfn_xy(tau,na,dq,ncall)
	return
	endif

      za(1)= xx(oflg,na)
      za(2)= yy(oflg,na)
      za(3)= et(oflg,na)  
CCC   //////////////////////////////////////////////////////////////
      u0a = cosh(al(oflg,na)-et(oflg,na))*cosh(bt(oflg,na))
      eos(2)= ss(oflg,na)
      eos(3)= bd(oflg,na)
      call eosph(eos,2)
      edna= eos(1)
      Pa=   eos(4)
      dq(na,1)= zero 
      dq(na,2)= zero 
      dq(na,3)= zero 
c	dqnew(1)= zero
c	dqnew(2)= zero
c	dqnew(3)= zero
c	dqold(1)= zero
c	dqold(2)= zero
c	dqold(3)= zero
      

      DO nn=1,np
         nb=kl(nn) 
         zb(1)= xx(oflg,nb)
         zb(2)= yy(oflg,nb)
         zb(3)= et(oflg,nb)
         u0b = cosh(al(oflg,nb)-et(oflg,nb))*cosh(bt(oflg,nb))
         eos(2)= ss(oflg,nb)
         eos(3)= bd(oflg,nb)
         call eosph(eos,2)
         ednb= eos(1)
         Pb=   eos(4)
CCC   #################################################################
CCC   modified at 07052006
CCC   #################################################################
          select case (ihdmode)
	    case (0)
              dqb = -sn(oflg,nb)*
     &            (Pa/(ss(oflg,na)**2 *u0a**2)+
     &            Pb/(ss(oflg,nb)**2 *u0b**2))/tau 
cc     &           Pb/(ss(oflg,na)**2 *u0b**2))/tau	!bug(?)
          case (1)
	        if ((iflgce(nb).eq.0)) then
              dqb = -sn(oflg,nb)*
     &            (Pa/(ss(oflg,na)**2 *u0a**2)+
     &            Pb/(ss(oflg,nb)**2 *u0b**2))/tau
	        else
	        dqb = 0.d0
	        end if
	        if ((iflgce(na).eq.2)) dqb=0.d0
	    case default
	        stop 'fatal error in ihdmode in dqfind'
	    end select
         if (bugflg.eq.30) then
         dqb = -sn(oflg,nb)*
     &         (Pa/(ss(oflg,na)**2 *u0a**2)+
     &          Pb/(ss(oflg,na)**2 *u0b**2))/tau
	   end if
CCC   #################################################################
CCC   debugging output
CCC   modified at 09092006
CCC   #################################################################
c         if (bugflg.eq.7) then
c	      dqbnew=-sn(oflg,nb)*
c     &         (Pa/(ss(oflg,na)**2 *u0a**2)+
c     &          Pb/(ss(oflg,nb)**2 *u0b**2))/tau
c	      dqbold=-sn(oflg,nb)*
c     &         (Pa/(ss(oflg,na)**2 *u0a**2)+
c     &          Pb/(ss(oflg,na)**2 *u0b**2))/tau
c         endif
CCC   #################################################################
         QXY= sqrt((za(1)-zb(1))**2+(za(2)-zb(2))**2)/hxy
         QET=  abs(za(3)-zb(3))/het 
CCC   #################################################################
         DO kflg=1,3 
            if(kflg.ne.3) then 
               call krld(QXY,DW2,2,12,za,zb,kflg)
               CALL krln(QET,W1,1,33)
               dq(na,kflg)=dq(na,kflg)+ dqb* DW2*W1
            else if(kflg.eq.3) then
               call krln(QXY,W2,2,12)
               call krld(QET,DW1,1,33,za,zb,kflg)               
               dq(na,kflg)=dq(na,kflg)+ dqb* W2*DW1
            endif
CCC   #################################################################
CCC   debugging output
      if (iauxi.ge.8) then
	if ((dq(na,kflg).ge.0.d0).or.(dq(na,kflg).le.0.d0)) then
		continue
	else
         open(11,file=fnnan(1:nfnnan),access='append')
		WRITE(11,*)'nancatchup in dqfind,oflg,ncall=',oflg,ncall
		WRITE(11,*)'source particle number na=',na
		WRITE(11,*)'neighbor particle number np=',np
		WRITE(11,*)'total & current neighbor particle index,nb=',nn,nb
		WRITE(11,'(a,4d20.10)') 'eos(1-4)',eos(1),eos(2),eos(3),eos(4)
		WRITE(11,'(a,3d20.10)') 'eos(5-7)',eos(5),eos(6),eos(7)
		WRITE(11,'(a,6d20.10)') 'xx,yy,et,ph,al,bt of a=',
     &      xx(oflg,na),yy(oflg,na),et(oflg,na),
     &      ph(oflg,na),al(oflg,na),bt(oflg,na)
		WRITE(11,'(a,6d20.10)') 'xx,yy,et,ph,al,bt of b=',
     &      xx(oflg,nb),yy(oflg,nb),et(oflg,nb),
     &      ph(oflg,nb),al(oflg,nb),bt(oflg,nb)
		WRITE(11,'(a,5d20.10)') 'dqb,W1,DW1,W2,DW2=',
     &      dqb,W1,DW1,W2,DW2
		WRITE(11,'(a,5d20.10)') 'ss,sn,u0b=',
     &      ss(oflg,nb),sn(oflg,nb),u0b
          WRITE(11,*)'If this happens for the first SPH particle ever'
          WRITE(11,*)'It might due to the IC distribution problem'
		CLOSE(11)
		stop 'dqfind fatal error!'
	endif
	endif
CCC   #################################################################
CCC   #################################################################
CCC   debugging output
CCC   modified at 09092006
CCC   #################################################################
c               if (bugflg.eq.7) then
c                   if(kflg.ne.3) then 
c                      call krld(QXY,DW2,2,12,za,zb,kflg)
c                      CALL krln(QET,W1,1,33)
c                      dqnew(kflg)=dqnew(kflg)+ dqbnew* DW2*W1
c                      dqold(kflg)=dqold(kflg)+ dqbold* DW2*W1
c                   else if(kflg.eq.3) then
c                      call krln(QXY,W2,2,12)
c                      call krld(QET,DW1,1,33,za,zb,kflg)
c                      dqnew(kflg)=dqnew(kflg)+ dqbnew* W2*DW1
c                      dqold(kflg)=dqold(kflg)+ dqbold* W2*DW1
c                   endif
c               endif
CCC   #################################################################
ccc   //////////////////////////////////////////////
          ENDDO ! of kflg
      ENDDO ! of nn
CCC   #################################################################
CCC   debugging output
CCC   modified at 09092006
CCC   #################################################################
c         if (bugflg.eq.7) then
c	      DO 320 kflg=1,2 
c            if ((dabs(dqnew(kflg))/dabs(dqold(kflg)).ge.1.d+2).or.
c     &          (dabs(dqnew(kflg))/dabs(dqold(kflg)).le.1.d-2)) then
c               open(11,file='dqdifcheck.data',access='append') 
c               write(11,*)'index number',na
c               write(11,*)'link number',np
c               write(11,'(a)')'dqnew dqold kflg'
c               write(11,'(2d16.8,i2)')dqnew(kflg),dqold(kflg),kflg
c               write(11,'(a)')'xxna yyna btna snna ssna bdna u0 gammat'
c               uu0=cosh(al(1,na)-et(1,na))*cosh(bt(1,na))
c               gammat=1.d0/dsqrt(1.d0-(sinh(bt(1,na))/uu0)**2)
c               write(11,'(7d16.8)')xx(1,na),yy(1,na),bt(1,na),sn(1,na),
c     &          ss(1,na),bd(1,na),uu0,gammat
c	         write(11,*)'   '
c               close(11)
c            endif
c320         CONTINUE
c         endif
CCC   #################################################################


CCC   #################################################################

      RETURN 
      END



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@





CCC   #################################################################
CCC   SUBROUTINE PROGRAM: dqfind.f 
CCC   "dqq" calculates dq_i/dtau 
CCC   A term appears in the 'momentum equation' of SPH.  
CCC
CCC   dq_a                 1      P_a+Q_a       1       P_b+Q_b         d W
CCC   ---- = - sum_b [--------- --------- + ---------- ---------]*sn_b* ----  
CCC   dtau            tau*u0_a^2   s_a^2    tau*u0_b^2   s_b^2          dx_a
CCC
CCC   #################################################################
CCC   last updated 05032009
CCC   abandon the line number for loop
CCC
CCC   last updated 03042007
CCC   debugging output added
CCC
CCC   last updated 02202007
CCC   inherit from dqfn()
CCC   #################################################################
      SUBROUTINE dqfn_xy(tau,na,dq,ncall)

      IMPLICIT NONE
      include 'spherio.inc' 
      include 'spherio.eos.inc'
	include 'spherio.debug.inc'
	include 'epos.incsf'
c     ////////////////////////////////////////////
      DOUBLE PRECISION QXY,QET,DW2,DW1,W2,tau
c	double precision W1
      DOUBLE PRECISION Pa,edna,u0a,za(3)
      DOUBLE PRECISION Pb,ednb,u0b,zb(3)
      DOUBLE PRECISION dqb,dq(NPL,3)
      INTEGER na,nb,nn,kflg
	integer ncall
	CHARACTER fname*30,chtau*10
c     ////////////////////////////////////////////

c      if (bugflg.ne.15) stop 'bugflg fatal error in dqfn_xy'

CCC   #################################################################
CCC   debug output
	if ((tau/0.197d0.eq.1.0d0).and.(bugflg.eq.15).and.(na.eq.999)
     &    .and.(oflg.eq.1)) then
	write(chtau,'(f10.7)')(tau/0.197d0)
      fname='char_dqfn'//chtau//'.data'
      open(11,file=fname,status='replace')
      write(11,'(a)')'na sna ssa u0a Pa kflg dw2 xa ya za
     &        nb snb ssb u0b Pb xb yb zb'
      write(11,'(a,d16.8)')'t=',tau/0.197d0
	close(11)
	end if
CCC   #################################################################


      za(1)= xx(oflg,na)
      za(2)= yy(oflg,na)
      za(3)= et(oflg,na)  
CCC   //////////////////////////////////////////////////////////////
      u0a = cosh(al(oflg,na)-et(oflg,na))*cosh(bt(oflg,na))
      eos(2)= ss(oflg,na)
      eos(3)= bd(oflg,na)
      call eosph(eos,2)
      edna= eos(1)
      Pa=   eos(4)
      dq(na,1)= zero
      dq(na,2)= zero
      dq(na,3)= zero
      

      DO nn=1,np
         nb=kl(nn)
c      DO nn=1,Nsp(1)
c         nb=nn
         zb(1)= xx(oflg,nb)
         zb(2)= yy(oflg,nb)
         zb(3)= et(oflg,nb)
         u0b = cosh(al(oflg,nb)-et(oflg,nb))*cosh(bt(oflg,nb))
         eos(2)= ss(oflg,nb)
         eos(3)= bd(oflg,nb)
         call eosph(eos,2)
         ednb= eos(1)
         Pb=   eos(4)
CCC   #################################################################
CCC   modified at 07052006
CCC   #################################################################
         dqb = -sn(oflg,nb)*
     &         (Pa/(ss(oflg,na)**2 *u0a**2)+
     &          Pb/(ss(oflg,nb)**2 *u0b**2))/tau
CCC   #################################################################
         QXY= sqrt((za(1)-zb(1))**2+(za(2)-zb(2))**2)/hxy
         QET=  abs(za(3)-zb(3))/het
CCC   #################################################################
         DO kflg=1,3
            if(kflg.ne.3) then
               call krld(QXY,DW2,2,12,za,zb,kflg)
c               CALL krln(QET,W1,1,33)
               dq(na,kflg)=dq(na,kflg)+ dqb* DW2  !rip W1 here
            else if(kflg.eq.3) then
               call krln(QXY,W2,2,12)
               call krld(QET,DW1,1,33,za,zb,kflg)
               dq(na,kflg)=dq(na,kflg)+ dqb* W2*DW1  !DW1 should be zero
	         if (DW1.ne.0.d0) then
	            write(*,*)'DW1 fatal error in dqfind_xy!'
                  stop 'DW1 fatal error in dqfind_xy!'
	         endif
            endif
CCC   #################################################################
CCC   debug output
	if ((tau/0.197d0.eq.1.0d0).and.(bugflg.eq.15).and.(na.eq.999)
     &    .and.(oflg.eq.1)) then
      open(11,file=fname,access='append')
	       write(11,'(i8,4d20.10,i8,4d20.10,i8,7d20.10)')
     &     na,sn(oflg,na),ss(oflg,na),u0a,Pa,kflg,dw2,
     &     za(1),za(2),za(3),nb,
     &     sn(oflg,nb),ss(oflg,nb),u0b,Pb,zb(1),zb(2),zb(3)
      close(11)
	end if
CCC   #################################################################
          ENDDO     ! of kflg
      ENDDO   ! of nn
CCC   #################################################################
CCC   #################################################################
CCC   debugging output
      if (iauxi.ge.18) then
	if ((dq(na,1).eq.0.d0).or.(dq(na,2).eq.0.d0)) then
         open(11,file=fnnan(1:nfnnan),access='append')
		WRITE(11,*)'nancatchup in dqfind_xy,oflg,ncall=',oflg,ncall
		WRITE(11,*)'neighbor particle number np=',np
		WRITE(11,*)'current particle index,nb=',nn,nb
		WRITE(11,'(a,4d20.10)') 'eos(1-4)',eos(1),eos(2),eos(3),eos(4)
		WRITE(11,'(a,3d20.10)') 'eos(5-7)',eos(5),eos(6),eos(7)
		WRITE(11,'(a,6d20.10)') 'xx,yy,et,ph,al,bt of a=',
     &      xx(oflg,na),yy(oflg,na),et(oflg,na),
     &      ph(oflg,na),al(oflg,na),bt(oflg,na)
		WRITE(11,'(a,6d20.10)') 'xx,yy,et,ph,al,bt of b=',
     &      xx(oflg,nb),yy(oflg,nb),et(oflg,nb),
     &      ph(oflg,nb),al(oflg,nb),bt(oflg,nb)
		WRITE(11,'(a,5d20.10)') 'dqb,DW1,W2,DW2=',
     &      dqb,DW1,W2,DW2
		WRITE(11,'(a,5d20.10)') 'ss,sn,u0b=',
     &      ss(oflg,nb),sn(oflg,nb),u0b
		CLOSE(11)
c		stop 'dqfind_xy fatal error!'
	endif
	endif
CCC   #################################################################
CCC   #################################################################
      RETURN 
      END

C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@





CCC   #################################################################
CCC   SUBROUTINE PROGRAM: dqfind.f 
CCC   EOM to calulate the bulk and shear viscosity
CCC   #################################################################
CCC   last updated 06112009
CCC   #################################################################
      SUBROUTINE dqfn_shear()

      IMPLICIT NONE
	write(*,*)' Not done yet..'
      stop 'Not done yet...'

CCC   #################################################################

      RETURN 
      END



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@





CCC   ################################################################
CCC   "findbt" is called in dq2v.f for finding 'bt'. 
CCC   It checks the consistency of SPH-momentum eqs.  
CCC   given 'bt'(=xbt), 'al','ph' and 'q'(=qxfn,qyfn,qefn)
CCC   and 'al','ph' and 'q'(=qxfn,qyfn,qefn), i.e., 
CCC             qe = (Pr+edn)/ssp *u0 *(tau*tau) *ve 
CCC   is OK or not.  
CCC   ################################################################
CCC   last updated 08172007
CCC   modified the output
CCC
CCC   last udpated 05132007
CCC   fixed a minor bug
CCC
CCC   last updated 05042006
CCC   include 'spherio.eos.inc' to incorporate the strangeness
CCC   ################################################################
      DOUBLE PRECISION FUNCTION findbt(xbt)

      IMPLICIT NONE
      include 'spherio.inc'
      include 'spherio.eos.inc'
      include 'spherio.debug.inc'
	include 'spherio.hydro.inc'
      include 'epos.incsf'
CCC   ///////////////////////////////////////////////////////
      DOUBLE PRECISION xbt 
      DOUBLE PRECISION QXY,QET,W2,W1
      DOUBLE PRECISION ssp,bbp,Pr,edn
      DOUBLE PRECISION vxfn,vyfn,vefn,fbt1,fbt2,fbt3 
      DOUBLE PRECISION chi,u0fn,alfn
      DOUBLE PRECISION xfn,yfn,efn,phfn,tafn
      DOUBLE PRECISION qxfn,qyfn,qefn
      DOUBLE PRECISION findbt_xy
      INTEGER nb,mm
      INTEGER orfn,lowtflg,lowbflg,lowsflg
CCC   ///////////////////////////////////////////////////////      
      COMMON/XFUN/xfn,yfn,efn,phfn,tafn,orfn 
      COMMON/QFUN/qxfn,qyfn,qefn
      COMMON/LOWT/lowtflg,lowbflg,lowsflg
CCC   ///////////////////////////////////////////////////////
     	if (idimension.eq.2) then
      findbt=findbt_xy(xbt)
      return
      endif

      oflg= orfn 
      CALL prcn(xfn,yfn,efn)
      if(npflg.eq.0) goto 900 !! goto error message... 
      
c     -------------------------------------------------------
      if(abs(cos(phfn)).gt.epsilon) then
         chi=  cos(phfn)*tanh(xbt)*(qefn/qxfn)/tafn
      else
         chi=  sin(phfn)*tanh(xbt)*(qefn/qyfn)/tafn
      endif
CCC   #################################################################
CCC   NaN catchup
      if (iauxi.ge.5) then
	if ((chi+dsqrt(chi*chi+uni)).le.0.d0) then
	if ((iauxi.ge.10).or.(chi.gt.-0.58d9)) then
      open(11,file=fnnan(1:nfnnan),access='append')
      write(11,*)'NaN in findbt!'
	write(11,'(a,2d30.20)')'chi,(chi+sqrt(chi*chi+uni))=',
     &           chi,(chi+dsqrt(chi*chi+uni))
      close(11)
      endif
      endif
      endif
CCC   #################################################################
      alfn= log(chi+sqrt(chi*chi+uni)) +efn
CCC   #################################################################
CCC   NaN catchup
c      if (iauxi.ge.5) then
      if (chi.lt.-0.58d9) then		!!this also gives log(0.d0) in alfn
	alfn= -1.d0/2.d0/chi +efn
      endif
c      endif
CCC   #################################################################
      u0fn= cosh(xbt)*cosh(alfn-efn)
      vxfn= tanh(xbt)*cos(phfn)/cosh(alfn-efn)
      vyfn= tanh(xbt)*sin(phfn)/cosh(alfn-efn)
      vefn= tanh(alfn-efn)/(tafn)
c     -------------------------------------------------------
      ssp= zero
      bbp= zero
CCC   ///////////////////////////////////////////////////////
      DO mm=1,np
         nb=kl(mm)
         QXY= sqrt((xfn-xx(orfn,nb))**2+(yfn-yy(orfn,nb))**2)/hxy
         QET= abs(efn- et(orfn,nb))/het
         CALL krln(QXY,W2,2,12)
         CALL krln(QET,W1,1,33)
         ssp= ssp+ sn(orfn,nb) *W2*W1
         bbp= bbp+ br(orfn,nb) *W2*W1
      END DO
      ssp= ssp/(u0fn*tafn)
      bbp= bbp/(u0fn*tafn)
      eos(2)= ssp
      eos(3)= bbp
      call eosph(eos,2)
      if(eos(5).lt.1.d-02) lowtflg=1 !! [T < 10 MeV ]
      edn= eos(1)
      Pr=  eos(4)
      fbt1= (Pr+edn)/ssp *u0fn *vxfn
      fbt2= (Pr+edn)/ssp *u0fn *vyfn
      fbt3= (Pr+edn)/ssp *u0fn *(tafn*tafn)*vefn
CCC   ///////////////////////////////////////////////////////
!!there is an ambiguity to find the solution along the line phfn= 4/3*pi
      findbt= (qyfn- fbt2) + (qxfn- fbt1) 
      
!!this is an attempt to solve the problem
      if ((phfn.ge.pi/two.and.phfn.lt.pi).or.     
     &    (phfn.ge.three*pi/two.and.phfn.lt.two*pi))
     &findbt= (qyfn- fbt2) - (qxfn- fbt1)

CCC   #################################################################
CCC   debugging output
	if ((findbt.ge.0.d0).or.(findbt.le.0.d0)) then
	    continue
      elseif (iauxi.lt.5) then
		stop 'findbt fatal error!'
	else
          open(11,file=fnnan(1:nfnnan),access='append')
		WRITE(11,*)'findbt has unphysical value'
		WRITE(11,*)'neighbor particle number np=',np
	    WRITE(11,'(a,4d30.20)') 'eos(1-4)',eos(1),eos(2),eos(3),eos(4)
		WRITE(11,'(a,3d30.20)') 'eos(5-7)',eos(5),eos(6),eos(7)
		WRITE(11,'(a,2d30.20)') 'xbt,findbt=',xbt,findbt
		WRITE(11,'(a,3d30.20)') 'xfn,yfn,efn=',xfn,yfn,efn
		WRITE(11,'(a,2d30.20,i3)') 'phfn,tafn,orfn=',phfn,tafn,orfn
		WRITE(11,'(a,3d30.20)') 'qxfn,qyfn,qefn=',qxfn,qyfn,qefn
		WRITE(11,'(a,3d30.20)') 'fbt3,fbt2,fbt1=',fbt3,fbt2,fbt1
	    WRITE(11,'(a,3d30.20)') 'chi,alfn,u0fn=',chi,alfn,u0fn
	    WRITE(11,'(a,3d30.20)') 'vxfn,vyfn,vefn=',vxfn,vyfn,vefn
		CLOSE(11)
		stop 'findbt fatal error!'
	endif
CCC   #################################################################
      RETURN

c     ########################################################
 900  continue 
      open(11,file=fnnsp(1:nfnnsp),access='append')
      write(11,*) 'np error in findbt \(^^:)/'
      write(11,'(4d30.20)')  tafn, xfn, yfn, efn
	write(11,'(3d30.20)')  qxfn,qyfn,qefn
      close(11) 
      stop 'SPhrRIO: np error in findbt'
c     ########################################################
      END 



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



CCC   ################################################################
CCC   "findbt" is called in dq2v.f for finding 'bt'. 
CCC   It checks the consistency of SPH-momentum eqs.  
CCC   given 'bt'(=xbt), 'al','ph' and 'q'(=qxfn,qyfn,qefn)
CCC   and 'al','ph' and 'q'(=qxfn,qyfn,qefn), i.e., 
CCC             qe = (Pr+edn)/ssp *u0 *(tau*tau) *ve 
CCC   is OK or not.  
CCC   ################################################################
CCC   last udpated 05132007
CCC   fixed a minor bug
CCC
CCC   last updated 03022007
CCC   inherit from the old version
CCC   ################################################################
      DOUBLE PRECISION FUNCTION findbt_xy(xbt)

      IMPLICIT NONE
      include 'spherio.inc'
      include 'spherio.eos.inc'
	include 'spherio.debug.inc'
	include 'epos.incsf'
CCC   ///////////////////////////////////////////////////////
      DOUBLE PRECISION xbt 
      DOUBLE PRECISION QXY,QET,W2
c      double precision W1
      DOUBLE PRECISION ssp,bbp,Pr,edn
      DOUBLE PRECISION vxfn,vyfn,vefn,fbt1,fbt2,fbt3 
      DOUBLE PRECISION chi,u0fn,alfn
      DOUBLE PRECISION xfn,yfn,efn,phfn,tafn
      DOUBLE PRECISION qxfn,qyfn,qefn
      INTEGER nb,mm
      INTEGER orfn,lowtflg,lowbflg,lowsflg
CCC   ///////////////////////////////////////////////////////      
      COMMON/XFUN/xfn,yfn,efn,phfn,tafn,orfn
      COMMON/QFUN/qxfn,qyfn,qefn
      COMMON/LOWT/lowtflg,lowbflg,lowsflg
CCC   ///////////////////////////////////////////////////////
      oflg= orfn 
      CALL prcn(xfn,yfn,efn)
      if(npflg.eq.0) goto 901 !! goto error message... 
      
c     -------------------------------------------------------
      if(abs(cos(phfn)).gt.epsilon) then
         chi=  cos(phfn)*tanh(xbt)*(qefn/qxfn)/tafn
      else
         chi=  sin(phfn)*tanh(xbt)*(qefn/qyfn)/tafn
      end if
CCC   #################################################################
CCC   NaN catchup
      if (iauxi.ge.5) then
	if ((chi+dsqrt(chi*chi+uni)).le.0.d0) then
	if ((iauxi.ge.10).or.(chi.gt.-0.58d9)) then
      open(11,file=fnnan(1:nfnnan),access='append')
      write(11,*)'NaN in findbt_xy!'
	write(11,'(a,2d30.20)')'chi,(chi+sqrt(chi*chi+uni))=',
     &           chi,(chi+dsqrt(chi*chi+uni))
      close(11)
	endif
	endif
	endif
CCC   #################################################################
      alfn= log(chi+sqrt(chi*chi+uni)) +efn
CCC   #################################################################
CCC   NaN catchup
      if (iauxi.ge.5) then
      if (chi.lt.-0.58d9) then		!!this also gives log(0.d0) in alfn
	alfn= -1.d0/2.d0/chi +efn
	endif
	endif
CCC   #################################################################
      u0fn= cosh(xbt)*cosh(alfn-efn)
      vxfn= tanh(xbt)*cos(phfn)/cosh(alfn-efn)
      vyfn= tanh(xbt)*sin(phfn)/cosh(alfn-efn)
      vefn= tanh(alfn-efn)/(tafn)
c     -------------------------------------------------------
      ssp= zero
      bbp= zero
CCC   ///////////////////////////////////////////////////////
      DO mm=1,np
         nb=kl(mm)
         QXY= sqrt((xfn-xx(orfn,nb))**2+(yfn-yy(orfn,nb))**2)/hxy 
         QET= abs(efn- et(orfn,nb))/het
         CALL krln(QXY,W2,2,12)
c         CALL krln(QET,W1,1,33)
         ssp= ssp+ sn(orfn,nb) *W2
         bbp= bbp+ br(orfn,nb) *W2
      END DO
      ssp= ssp/(u0fn*tafn)
      bbp= bbp/(u0fn*tafn)
      eos(2)= ssp
      eos(3)= bbp 
      call eosph(eos,2)
      if(eos(5).lt.1.d-02) lowtflg=1 !! [T < 10 MeV ]
      edn= eos(1)
      Pr=  eos(4)
      fbt1= (Pr+edn)/ssp *u0fn *vxfn
      fbt2= (Pr+edn)/ssp *u0fn *vyfn
      fbt3= (Pr+edn)/ssp *u0fn *(tafn*tafn)*vefn
CCC   ///////////////////////////////////////////////////////
      findbt_xy= (qyfn- fbt2) + (qxfn- fbt1)
CCC   #################################################################
CCC   debugging output
      if (iauxi.ge.5) then
	if ((findbt_xy.ge.0.d0).or.(findbt_xy.le.0.d0)) then
		continue
	else
          open(11,file=fnnan(1:nfnnan),access='append')
		WRITE(11,*)'findbt_xy has unphysical value'
		WRITE(11,*)'neighbor particle number np=',np
	    WRITE(11,'(a,4d30.20)') 'eos(1-4)',eos(1),eos(2),eos(3),eos(4)
		WRITE(11,'(a,3d30.20)') 'eos(5-7)',eos(5),eos(6),eos(7)
		WRITE(11,'(a,2d30.20)') 'xbt,findbt_xy=',xbt,findbt_xy
		WRITE(11,'(a,3d30.20)') 'xfn,yfn,efn=',xfn,yfn,efn
		WRITE(11,'(a,2d30.20,i3)') 'phfn,tafn,orfn=',phfn,tafn,orfn
		WRITE(11,'(a,3d30.20)') 'qxfn,qyfn,qefn=',qxfn,qyfn,qefn
		WRITE(11,'(a,3d30.20)') 'fbt3,fbt2,fbt1=',fbt3,fbt2,fbt1
	    WRITE(11,'(a,2d30.20)') 'alfn,u0fn=',alfn,u0fn
	    WRITE(11,'(a,3d30.20)') 'vxfn,vyfn,vefn=',vxfn,vyfn,vefn
		CLOSE(11)
		stop 'findbt_xy fatal error!'
	endif
	endif
CCC   #################################################################
      RETURN

c     ########################################################
 901  continue 
      open(11,file=fnnsp(1:nfnnsp),access='append')
      write(11,*) 'np error in findbt \(^^:)/'
      write(11,'(4d30.20)')  tafn, xfn, yfn, efn
	write(11,'(3d30.20)')  qxfn,qyfn,qefn
      close(11) 
      stop 'SPhrRIO: np error in findbt'
c     ########################################################
      END 



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



CCC   ####################################################################
CCC   FUNCTION: findbt2.f 
CCC   New EOM in terms of four velocity
CCC   ####################################################################
CCC   last updated 04202009
CCC   added low entropy and baryon density check
CCC
CCC   last updated 03282009
CCC   built based on Kodama's suggestion three years ago
CCC   ####################################################################
      DOUBLE PRECISION FUNCTION findbt2()
      IMPLICIT NONE
      include 'spherio.inc'
      include 'spherio.eos.inc'
      include 'spherio.debug.inc'
	include 'spherio.sphsplit.inc'
	include 'spherio.hydro.inc'
      include 'epos.incsf'
      DOUBLE PRECISION xfn,yfn,efn,phfn,tafn
      DOUBLE PRECISION qxfn,qyfn,qefn
      INTEGER orfn
      COMMON/XFUN/xfn,yfn,efn,phfn,tafn,orfn
      COMMON/QFUN/qxfn,qyfn,qefn
      INTEGER lowtflg,lowbflg,lowsflg
      COMMON/LOWT/lowtflg,lowbflg,lowsflg
      DOUBLE PRECISION u0fn,u1fn,u2fn,u3fn
      COMMON/UFUN/u0fn
      DOUBLE PRECISION Pr,edn
      DOUBLE PRECISION ssp,bbp,QXY,QET,W2,W1
      DOUBLE PRECISION xtemp,ytemp,etemp
      INTEGER mm,nb
	double precision findbt2_xy

     	if (idimension.eq.2) then
      findbt2=findbt2_xy()
      return
      endif
      
      xtemp=xfn
      ytemp=yfn
      etemp=efn
      lowtflg=0
	lowbflg=0
      lowsflg=0
      CALL prcn(xtemp,ytemp,etemp)
      if(npflg.eq.0) stop 'npflg=0 in findbt2 place 1'
      ssp= zero
      bbp= zero
      DO mm=1,np
         nb=kl(mm)
         QXY= sqrt((xfn-xx(orfn,nb))**2+(yfn-yy(orfn,nb))**2)/hxy
         QET= abs(efn- et(orfn,nb))/het
         CALL krln(QXY,W2,2,12)
         CALL krln(QET,W1,1,33)
         ssp= ssp+ sn(orfn,nb) *W2*W1
         bbp= bbp+ br(orfn,nb) *W2*W1
      END DO
      ssp= ssp/(u0fn*tafn)
      bbp= bbp/(u0fn*tafn)
      eos(2)= ssp
      eos(3)= bbp
      call eosph(eos,2)
      edn= eos(1)
      Pr=  eos(4)
      if(eos(5).lt.1.d-02) lowtflg=1 !! [T < 10 MeV ]
      if(eos(2).lt.ssmin) lowsflg=1
      if(abs(eos(3)).lt.bdmin) lowbflg=1
      u1fn=qxfn/(Pr+edn)*ssp
      u2fn=qyfn/(Pr+edn)*ssp
      u3fn=qefn/(Pr+edn)*ssp/tafn**2
      u0fn=dsqrt(u1fn**2+u2fn**2+tafn**2*u3fn**2+1.d0)  !for use the next time
      findbt2= dsqrt(u1fn*u1fn+u2fn*u2fn) !to avoid bad convergence
      findbt2= dlog(dsqrt(1.d0+findbt2**2)+findbt2)
CCC   ####################################################################
CCC   debug output
CCC   ####################################################################
      if (findbt2.lt.15.d0) then
         continue
      else
	    if (iauxi.ge.8) then
          open(11,file=fnnan(1:nfnnan),access='append')
          write(11,*)'findbt2 has unphysical value'
          write(11,*)'neighbor particle number np=',np
          write(11,'(a,4d30.20)') 'eos(1-4)',eos(1),eos(2),eos(3),eos(4)
          write(11,'(a,3d30.20)') 'eos(5-7)',eos(5),eos(6),eos(7)
          write(11,'(a,2d30.20)') 'u0fn,findbt2=',u0fn,findbt2
          write(11,'(a,3d30.20)') 'u1fn,u2fn,u3fn=',u1fn,u2fn,u3fn
          write(11,'(a,3d30.20)') 'xfn,yfn,efn=',xfn,yfn,efn
          write(11,'(a,3d30.20)') 'qxfn,qyfn,qefn=',qxfn,qyfn,qefn
          write(11,'(a,2d30.20,i3)') 'phfn,tafn,orfn=',phfn,tafn,orfn
	    write(11,'(a)') ' '
          close(11)
	    endif
	    if (findbt2.ge.0.d0) then
	        continue
	    else
              stop 'findbt2 fatal error!'
	    endif
      endif
CCC   ####################################################################
CCC   ####################################################################
      RETURN 
      END 




CCC   ####################################################################
CCC   FUNCTION: findbt2.f 
CCC   New EOM in terms of four velocity
CCC   ####################################################################
CCC   last updated 052032009
CCC   built based on findbt2()
CCC   ####################################################################
      DOUBLE PRECISION FUNCTION findbt2_xy()
      IMPLICIT NONE
      include 'spherio.inc'
      include 'spherio.eos.inc'
      include 'spherio.debug.inc'
	include 'spherio.sphsplit.inc'
	include 'spherio.hydro.inc'
      include 'epos.incsf'
      DOUBLE PRECISION xfn,yfn,efn,phfn,tafn
      DOUBLE PRECISION qxfn,qyfn,qefn
      INTEGER orfn
      COMMON/XFUN/xfn,yfn,efn,phfn,tafn,orfn
      COMMON/QFUN/qxfn,qyfn,qefn
      INTEGER lowtflg,lowbflg,lowsflg
      COMMON/LOWT/lowtflg,lowbflg,lowsflg
      DOUBLE PRECISION u0fn,u1fn,u2fn,u3fn
      COMMON/UFUN/u0fn
      DOUBLE PRECISION Pr,edn
      DOUBLE PRECISION ssp,bbp,QXY,QET,W2
      DOUBLE PRECISION xtemp,ytemp,etemp
      INTEGER mm,nb

      xtemp=xfn
      ytemp=yfn
      etemp=efn
      lowtflg=0
	lowbflg=0
      lowsflg=0
      CALL prcn(xtemp,ytemp,etemp)
      if(npflg.eq.0) stop 'npflg=0 in findbt2_xy place 1'
      ssp= zero
      bbp= zero
      DO mm=1,np
         nb=kl(mm)
         QXY= sqrt((xfn-xx(orfn,nb))**2+(yfn-yy(orfn,nb))**2)/hxy
         QET= abs(efn- et(orfn,nb))/het
         CALL krln(QXY,W2,2,12)
!         CALL krln(QET,W1,1,33)
         ssp= ssp+ sn(orfn,nb) *W2    !rip off W1 here
         bbp= bbp+ br(orfn,nb) *W2
      END DO
      ssp= ssp/(u0fn*tafn)
      bbp= bbp/(u0fn*tafn)
      eos(2)= ssp
      eos(3)= bbp
      call eosph(eos,2)
      edn= eos(1)
      Pr=  eos(4)
      if(eos(5).lt.1.d-02) lowtflg=1 !! [T < 10 MeV ]
      if(eos(2).lt.ssmin) lowsflg=1
      if(abs(eos(3)).lt.bdmin) lowbflg=1
      u1fn=qxfn/(Pr+edn)*ssp
      u2fn=qyfn/(Pr+edn)*ssp
      u3fn=qefn/(Pr+edn)*ssp/tafn**2
      u0fn=dsqrt(u1fn**2+u2fn**2+tafn**2*u3fn**2+1.d0)  !for use the next time
      findbt2_xy= dsqrt(u1fn*u1fn+u2fn*u2fn) !to avoid bad convergence
      findbt2_xy= dlog(dsqrt(1.d0+findbt2_xy**2)+findbt2_xy)
CCC   ####################################################################
CCC   debug output
CCC   ####################################################################     
      if (findbt2_xy.lt.15.d0) then
         continue
      else
	    if (iauxi.ge.8) then
          open(11,file=fnnan(1:nfnnan),access='append')
          write(11,*)'findbt2_xy has unphysical value'
          write(11,*)'neighbor particle number np=',np
          write(11,'(a,4d30.20)') 'eos(1-4)',eos(1),eos(2),eos(3),eos(4)
          write(11,'(a,3d30.20)') 'eos(5-7)',eos(5),eos(6),eos(7)
          write(11,'(a,2d30.20)') 'u0fn,findbt2_xy=',u0fn,findbt2_xy
          write(11,'(a,3d30.20)') 'u1fn,u2fn,u3fn=',u1fn,u2fn,u3fn
          write(11,'(a,3d30.20)') 'xfn,yfn,efn=',xfn,yfn,efn
          write(11,'(a,3d30.20)') 'qxfn,qyfn,qefn=',qxfn,qyfn,qefn
          write(11,'(a,2d30.20,i3)') 'phfn,tafn,orfn=',phfn,tafn,orfn
	    write(11,'(a)') ' '
          close(11)
	    endif
	    if (findbt2_xy.ge.0.d0) then
	        continue
	    else
              stop 'findbt2_xy fatal error!'
	    endif
      endif
CCC   ####################################################################
      if (iauxi.ge.18) then
	if (findbt2_xy.eq.0.d0) then
          open(11,file=fnnan(1:nfnnan),access='append')
          write(11,*)'findbt2_xy has zero value'
          write(11,*)'neighbor particle number np=',np
          write(11,'(a,4d30.20)') 'eos(1-4)',eos(1),eos(2),eos(3),eos(4)
          write(11,'(a,3d30.20)') 'eos(5-7)',eos(5),eos(6),eos(7)
          write(11,'(a,2d30.20)') 'u0fn,findbt2_xy=',u0fn,findbt2_xy
          write(11,'(a,3d30.20)') 'u1fn,u2fn,u3fn=',u1fn,u2fn,u3fn
          write(11,'(a,3d30.20)') 'xfn,yfn,efn=',xfn,yfn,efn
          write(11,'(a,3d30.20)') 'qxfn,qyfn,qefn=',qxfn,qyfn,qefn
          write(11,'(a,2d30.20,i3)') 'phfn,tafn,orfn=',phfn,tafn,orfn
	    write(11,'(a)') ' '
          close(11)
	endif
	endif
CCC   ####################################################################
      RETURN 
      END 




CCC   #################################################################
CCC   subroutine program firststep.f
CCC   "firststep" is used in just first step of SPheRIO.
CCC   The tasks of the 'firststep' is
CCC   1) collect 'sn' to reproduce the initial condition well.   
CCC   2) rearrange SPH particles in a descending order of temperature. 
CCC   #################################################################
CCC   last updated 06112009
CCC   adopt the bulk viscosity
CCC
CCC   last updated 09022008
CCC   modified expression of etot
CCC
CCC   last updated 08192008
CCC   implemented etot and eftot
CCC
CCC   last udpated 11262007
CCC   add more entropy output
CCC
CCC   last udpated 08312007
CCC   incorporate vvxn
CCC
CCC   last updated 05212007
CCC   incorporate uxn
CCC
CCC   last updated 02202007
CCC   modified to ensure two oflg set
CCC
CCC   last updated 12192006
CCC   modified to incorporate tempxn in new chemical fo
CCC
CCC   last updated 10162006
CCC   incorporate chemical fo
CCC
CCC   last updated 05042006
CCC   include 'spherio.eos.inc' to incorporate the strangeness
CCC   #################################################################
      SUBROUTINE firststep(tau)
      
      IMPLICIT NONE
      include 'spherio.inc' 
      include 'spherio.eos.inc'
	include 'spherio.temps.inc'
	include 'epos.incsf'

CCC   #################################################################
      INTEGER Nspp,na,nn,mm
	integer kk
      DOUBLE PRECISION tau
      INTEGER indx(NPL)
      DOUBLE PRECISION arr(NPL)
	double precision gamma
      
CCC   #################################################################
ccc   sn correction part:  Firsr step ONLY 
      oflg=1

      Nspp= Nsp(oflg)
      CALL mklt()

CCC   #################################################################
      do na=1,Nspp 
         CALL prcn(xx(1,na),yy(1,na),et(1,na))
         if(npflg.eq.0) goto 172 
         CALL fins(tau,na)
	 CALL bsqd(tau,na) 
      end do
      
CCC   #################################################################
ccc   rearranging of SPH particles: 
CCC   #################################################################
      do nn=1,Nspp
         eos(2)=ss(1,nn)
         eos(3)=bd(1,nn)
         call eosph(eos,2)
         arr(nn)=eos(5)
      end do

      call indexx(Nspp,NPL,arr,indx)
      na=0
      do nn=Nspp,1,-1
         na=na+1
         mm=indx(nn)

CCC   #################################################################
CCC   Emissao Continua e Quimico fo
         iflgce_aux(na) = iflgce(mm)
         iflgxn_aux(na) = iflgxn(mm)
	   mbxn_aux(na)   = mbxn(mm)
	   msxn_aux(na)   = msxn(mm)
	   tempxn_aux(na) = tempxn(mm)
	   ssxn_aux(na)   = ssxn(mm)
	   bbxn_aux(na)   = bbxn(mm)
         do kk=0,3
	   uxn_aux(kk,na) = uxn(kk,mm)
	   enddo
         iflgfp_aux(na) = iflgfp(mm)
CCC   #################################################################
         xx(0,na)= xx(1,mm)
         yy(0,na)= yy(1,mm)
         et(0,na)= et(1,mm)
         al(0,na)= al(1,mm)
         bt(0,na)= bt(1,mm)
         ph(0,na)= ph(1,mm)
         sn(0,na)= sn(1,mm)
         ss(0,na)= ss(1,mm)
         pibn(0,na)=pibn(1,mm)
         pibd(0,na)=pibd(1,mm)
         sgmn(0,na)=sgmn(1,mm)
         sgmd(0,na)=sgmd(1,mm)
CCC   #################################################################
         br(0,na)= br(1,mm)
         sr(0,na)= sr(1,mm)
         qr(0,na)= qr(1,mm)
         bd(0,na)= bd(1,mm)
         sd(0,na)= sd(1,mm)
         qd(0,na)= qd(1,mm)
CCC   #################################################################
      end do

      stot= zero
	stoteta=zero
      btot= zero 
	etot= zero
      ez= zero

      do na=1,Nspp

CCC   #################################################################
CCC   Emissao Continua e Quimico fo
         iflgce(na) = iflgce_aux(na)
         iflgxn(na) = iflgxn_aux(na)
	   mbxn(na)   = mbxn_aux(na)
	   msxn(na)   = msxn_aux(na)
	   tempxn(na) = tempxn_aux(na)
	   ssxn(na)   = ssxn_aux(na)
	   bbxn(na)   = bbxn_aux(na)
         do kk=0,3
	   uxn(kk,na) = uxn_aux(kk,na)
	   enddo
         iflgfp(na) = iflgfp_aux(na)
CCC   #################################################################
         xx(1,na)= xx(0,na)
         yy(1,na)= yy(0,na)
         et(1,na)= et(0,na)
         al(1,na)= al(0,na)
         bt(1,na)= bt(0,na)
         ph(1,na)= ph(0,na)
         ss(1,na)= ss(0,na)
         sn(1,na)= sn(0,na)
         pibn(1,na)=pibn(0,na)
         pibd(1,na)=pibd(0,na)
         sgmn(1,na)=sgmn(0,na)
         sgmd(1,na)=sgmd(0,na)
CCC   #################################################################
         bd(1,na)= bd(0,na)
         sd(1,na)= sd(0,na)
         qd(1,na)= qd(0,na) 
         br(1,na)= br(0,na)
         sr(1,na)= sr(0,na)
         qr(1,na)= qr(0,na)
CCC   #################################################################
c     also flg=2:
         sn(2,na)= sn(1,na)
         br(2,na)= br(1,na)
         sr(2,na)= sr(1,na)
         qr(2,na)= qr(1,na)
CCC   #################################################################
         stot=stot+sn(1,na)
	   if (abs(et(1,na)).le.1.d0) stoteta=stoteta+sn(1,na)
         btot=btot+br(1,na)
         eos(2)=   ss(1,na)
         eos(3)=   bd(1,na)
         call eosph(eos,2)
	   gamma= cosh(al(1,na)-et(1,na))*cosh(bt(1,na))
	   etot=etot+sn(1,na)/(ss(1,na)*gamma)
     &        *((eos(1)+eos(4))*gamma**2-eos(4))
         if(eos(5).lt.Tfin) then
            Nsp(1)=na    !! <== SPH particle number in 1.st step.
	      oflg=1
            CALL mklt()
            goto 160 
         end if
      end do
	oflg=1
      CALL mklt()
 160  continue
CCC   #################################################################
      RETURN
      
ccc   ERROR MESSAGE
 172  CONTINUE
      open(11,file=fnnsp(1:nfnnsp),access='append')
      write(11,*) 'np error 1 in firststep \(^^:)/'
      close(11)
      stop 'SPheRIO: np error 1 in firststep'      
 174  CONTINUE
      open(11,file=fnnsp(1:nfnnsp),access='append')
      write(11,*) 'np error 2 in firststep \(^^:)/'
      close(11)
      stop 'SPheRIO: np error 2 in firststep'

      END

 


CCC   #################################################################
CCC   #################################################################
CCC   #################################################################


CCC   #################################################################
CCC   fluid observer
CCC   for check of hydrodynamical evolution
CCC   #################################################################
CCC   last updated 12152008
CCC   change al to et
CCC
CCC   last updated 08162006
CCC   #################################################################
      SUBROUTINE fldobsrv() 
      IMPLICIT NONE 
      include 'spherio.inc'
	include 'spherio.debug.inc'
      INTEGER flg,mm,nn,mbin 
      DOUBLE PRECISION xmin,xmax,x,x1,x2,dx
      DOUBLE PRECISION bnsph(100),snsph(100),nnsph(100)
      
      if (iauxi.ge.15) then

      xmin=-10.00d+00
      xmax=+10.00d+00
      mbin=20
      dx=(xmax-xmin)/dble(mbin) 

      do mm=1,mbin
         bnsph(mm)=zero
	   snsph(mm)=zero
	   nnsph(mm)=zero
      end do
      
      flg=1
      do nn=1,Nsp(flg)
c         mm= int( (al(flg,nn)-xmin)/dx ) +1 
         mm= int( (et(flg,nn)-xmin)/dx ) +1 
c         if(al(flg,nn).ge.xmax) mm=mbin
c         if(al(flg,nn).le.xmin) mm=1
         if(et(flg,nn).ge.xmax) mm=mbin
         if(et(flg,nn).le.xmin) mm=1
         bnsph(mm)=bnsph(mm)+br(flg,nn)
	   snsph(mm)=snsph(mm)+sn(flg,nn)
	   nnsph(mm)=nnsph(mm)+1.d0
      end do
c     ============================================================
      OPEN(10,FILE='zhist.sph.data',status='replace')
c	write(10,'(a)')'alpha bnsph snsph nnsph'
      write(10,'(a)')'eta bnsph entropysph nnsph dsdeta'
      do mm=1,mbin 
         x1= (mm-1)*dx+ xmin
         x2= (mm  )*dx+ xmin 
         x=  0.5d+00*(x1+x2)
        write(10,'(5d16.8)')x,bnsph(mm),snsph(mm),nnsph(mm),snsph(mm)/dx
      end do
      CLOSE(10) 
c     ============================================================

	endif

      RETURN
      END



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



CCC   #################################################################
CCC   fluid observer at last minute
CCC   for check of hydrodynamical evolution
CCC   #################################################################
CCC   last updated 12152008
CCC   change al to et
CCC
CCC   last updated 08162006 
CCC   #################################################################
      SUBROUTINE fldobsrv_fin(nf) 
      IMPLICIT NONE 
      include 'spherio.inc'
	include 'spherio.debug.inc'
      INTEGER flg,mm,nn,mbin 
	INTEGER nf
      DOUBLE PRECISION xmin,xmax,x,x1,x2,dx
      DOUBLE PRECISION bnsph(100),snsph(100),nnsph(100)
      

      if (iauxi.ge.15) then

      xmin=-10.00d+00
      xmax=+10.00d+00
      mbin=20
      dx=(xmax-xmin)/dble(mbin) 

      do mm=1,mbin
         bnsph(mm)=zero
	   snsph(mm)=zero
	   nnsph(mm)=zero
      end do
      
      flg=1
      do nn=1,nf
c         mm= int( (rfrs(3,nn)-xmin)/dx ) +1
         mm= int( (xfrs(3,nn)-xmin)/dx ) +1
c         if(rfrs(3,nn).ge.xmax) mm=mbin
c         if(rfrs(3,nn).le.xmin) mm=1
         if(xfrs(3,nn).ge.xmax) mm=mbin
         if(xfrs(3,nn).le.xmin) mm=1
         bnsph(mm)=bnsph(mm)+bnfz(nn)
	   snsph(mm)=snsph(mm)+snfz(nn)
	   nnsph(mm)=nnsph(mm)+1.d0
      end do
c     ============================================================
      OPEN(10,FILE='zhistfin.sph.data',status='replace')
c	write(10,'(a)')'alpha bnsph snsph nnsph'
      write(10,'(a)')'eta bnsph entropysph nnsph dsdeta'
      do mm=1,mbin 
         x1= (mm-1)*dx+ xmin
         x2= (mm  )*dx+ xmin 
         x=  0.5d+00*(x1+x2)
        write(10,'(5d16.8)')x,bnsph(mm),snsph(mm),nnsph(mm),snsph(mm)/dx
      end do
      CLOSE(10) 
c     ============================================================

	endif

      RETURN
      END



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



CCC   #################################################################
CCC   transverse fluid observer
CCC   subroutine outp_raw_pion()
CCC   for check of hydrodynamical evolution
CCC   #################################################################
CCC   last updated 02202007
CCC   inherit from 2d code
CCC   #################################################################
      SUBROUTINE outp_raw(ntsp,tau)
      IMPLICIT NONE
      include 'spherio.inc'
	include 'spherio.debug.inc'
	include 'spherio.eos.inc'
	integer ntsp
	double precision tau
      INTEGER nn
      DOUBLE PRECISION vx,vy,ve,q1,q2,q3,u0
      DOUBLE PRECISION edn,Pr
      DOUBLE PRECISION rr,x1,x2
      DOUBLE PRECISION pp,p1,p2
	character fname*30,chtau*10

c	if (bugflg.ne.15) stop 'bugflg fatal error in outp_raw'
	write(chtau,'(f10.7)')(tau/0.197d0)
      fname='char_rawfullnps'//chtau//'.data'
      open(11,file=fname,access='append')
      write(11,'(a)')'rr x1 x2 p1 p2 pp sn index'
      write(11,'(a,d16.8)')'tau=',tau/0.197d0
      do nn=1,Nsp(1)
             vx= tanh(bt(1,nn))*cos(ph(1,nn))/cosh(al(1,nn)-et(1,nn))
             vy= tanh(bt(1,nn))*sin(ph(1,nn))/cosh(al(1,nn)-et(1,nn))
             ve= tanh(al(1,nn)-et(1,nn))/tau
             u0= cosh(bt(1,nn))*cosh(al(1,nn)-et(1,nn))
             eos(2)= ss(1,nn)
             eos(3)= bd(1,nn)
             call eosph(eos,2)
             edn= eos(1)
             Pr=  eos(4)
             q1=(Pr+edn)/ss(1,nn)*u0*vx
             q2=(Pr+edn)/ss(1,nn)*u0*vy
             q3=(Pr+edn)/ss(1,nn)*u0*ve*tau*tau
	       x1=xx(1,nn)
	       x2=yy(1,nn)
	       rr=dsqrt(x1**2+x2**2)
	       p1=q1
	       p2=q2
	       pp=dsqrt(p1**2+p2**2)
	       write(11,'(7d20.10,i8)')rr/.197d0,x1/.197d0,x2/.197d0,
     &               p1,p2,pp,sn(1,nn),nn
      end do
      close(11)

	return
	end
CCC   #################################################################







CCC   #################################################################
CCC   SUBROUTINE PROGRAM: rung.f 
CCC   'rung' solves equation of motion of SPH-particles:    
CCC   Using results by rngkt.f, it renews quantity 'q' and 
CCC   finds rapidities corresponded. 
CCC   The difinition of q is the following; 
CCC      q=[ (P+edn)/s*u0*vx, (P+edn)/s*u0*vy, (P+edn)/s*u0*tau^2 v_e ] 
CCC   We also solve  dx/d tau = v  [ =(vx,vy,ve)]  
CCC   v=(vx,vy,ve), where 
CCC   vx= (tanh bt cos ph)/cosh(al-et),
CCC   vy= (tanh bt sin ph)/cosh(al-et),
CCC   ve= tanh(al-et)/tau by Runge-Kutta 2nd order.  
CCC   #################################################################
CCC   to find al,bt,ph from q, we use
CCC             chi= 1/tau* q_e/q_x *cos(ph)*tanh(bt) ,
CCC             al = log(chi+sqrt(chi*chi+1))+et  , 
CCC   and use the following algorithms:   
CCC        1)    get 'ph' using q_x and q_y
CCC        2)    input trial 'bt' and get 'al'
CCC        3)    check those 'al','bt' and 'ph' consistent with 
CCC              q or not. 
CCC   #################################################################
CCC   last updated 09192010
CCC   a hidden bug concerning q2x=q2y=0 is fixed
CCC
CCC   last udpated 05042009
CCC   fixed the lowbflg hidden conflict
CCC
CCC   last updated 03282009
CCC   implemented findbt2()
CCC
CCC   last updated 08212007
CCC   add more debug output
CCC   fix an error in formula
CCC
CCC   last updated 08202007
CCC   implement free particle free stream
CCC
CCC   last updated 08172007
CCC   add debug output
CCC
CCC   last updated 03042007
CCC   add common block
CCC   add deugging output
CCC
CCC   last updated 03012007
CCC   add debugging output for transverse expansion
CCC
CCC   last updated 02122007
CCC   modified the precision of findbt
CCC
CCC   last updated 05042006
CCC   include 'spherio.eos.inc' to incorporate the strangeness
CCC   #################################################################
      SUBROUTINE rung(tau,dq1,dq2)
       
      IMPLICIT NONE
      include 'spherio.inc'
      include 'spherio.eos.inc'
      include 'spherio.quifz.inc'
      include 'spherio.debug.inc'
      include 'spherio.temps.inc'
	include 'spherio.hydro.inc'
      include 'epos.incsf'
ccc   ---------------------------------------------------
      DOUBLE PRECISION k1x(NPL),k1y(NPL),k1e(NPL)
      DOUBLE PRECISION k2x(NPL),k2y(NPL),k2e(NPL)
      DOUBLE PRECISION q1x(NPL),q1y(NPL),q1e(NPL)
      DOUBLE PRECISION q2x(NPL),q2y(NPL),q2e(NPL)
      DOUBLE PRECISION dq1(NPL,3),dq2(NPL,3)
ccc   ---------------------------------------------------
      INTEGER Nspp,nn
      DOUBLE PRECISION tau,tau1,tau2
      DOUBLE PRECISION vx,vy,ve,u0 
ccc   ---------------------------------------------------
      DOUBLE PRECISION chi,rtflsp1,x1,x2,phif,xacc
      DOUBLE PRECISION Tm,Pr,edn
ccc   ---------------------------------------------------
      DOUBLE PRECISION xfn,yfn,efn,phfn,tafn
      DOUBLE PRECISION qxfn,qyfn,qefn
      DOUBLE PRECISION athbt
      INTEGER orfn
      INTEGER exflg,lowtflg,lowbflg,lowsflg

      COMMON/XFUN/xfn,yfn,efn,phfn,tafn,orfn
      COMMON/QFUN/qxfn,qyfn,qefn
      DOUBLE PRECISION u0fn
      COMMON/UFUN/u0fn
      COMMON/LOWT/lowtflg,lowbflg,lowsflg
      COMMON/EXF/exflg
      COMMON/QKRUNG/q1x,q1y,q1e,q2x,q2y,q2e,k1x,k1y,k1e,k2x,k2y,k2e
      DOUBLE PRECISION findbt2,bttemp1,bttempx
c	DOUBLE PRECISION findbt_new,findbt_old,bttemp2
      EXTERNAL findbt2
      DOUBLE PRECISION findbt
      EXTERNAL findbt 
      INTEGER nout,nfbt2,lpfbt2
	double precision xaccfbt2
c      SAVE q1x,q1y,q1e
c      SAVE q2x,q2y,q2e
c      SAVE k1x,k1y,k1e
c      SAVE k2x,k2y,k2e
ccc   ///////////////////////////////////////////////////
      
      exflg=0
      nfbt2=100
      xaccfbt2=1.d-5
      tau1=tau
      tau2=tau+dtau
      
ccc   2.nd order Runge-Kutta the first part:
CCC   #################################################################
      IF(oflg.eq.1) THEN
         Nspp= Nsp(1)
CCC   #################################################################
CCC   debug output
c	if ((tau/0.197d0.le.1.04d0).and.(bugflg.eq.15)) then
c	write(chtau,'(f10.7)')(tau/0.197d0)
c      fname1='char_rk1step'//chtau//'.data'
c      open(11,file=fname1,status='replace')
c      write(11,'(a)')'rr sn q1x q1y pp ff1 vv1 index'
c      write(11,'(a,d16.8)')'t=',tau/0.197d0
c	close(11)
c	end if
CCC   #################################################################

         DO 160 nn=1,Nspp
            vx= tanh(bt(1,nn))*cos(ph(1,nn))/cosh(al(1,nn)-et(1,nn))
            vy= tanh(bt(1,nn))*sin(ph(1,nn))/cosh(al(1,nn)-et(1,nn))
            ve= tanh(al(1,nn)-et(1,nn))/tau1
            u0= cosh(bt(1,nn))*cosh(al(1,nn)-et(1,nn))

            k1x(nn)= vx* dtau
            k1y(nn)= vy* dtau
            k1e(nn)= ve* dtau
            xx(2,nn)= xx(1,nn) + k1x(nn)
            yy(2,nn)= yy(1,nn) + k1y(nn)
            et(2,nn)= et(1,nn) + k1e(nn)
CCC   #################################################################
CCC   debug output
	      if (iauxi.ge.2) then
            if ((xx(2,nn).le.0.d0).or.(xx(2,nn).ge.0.d0)) then
	        continue
	      elseif (iauxi.ge.5) then
              open(11,file=fnnan(1:nfnnan),access='append')
		    write(11,*)'nancatchup1 xx(2,nn) in rung'
		    write(11,*)'SPH particle number nn=',nn
		    write(11,'(a,3d30.20)') 'xx(2,nn),yy(2,nn),et(2,nn)=',
     &                                 xx(2,nn),yy(2,nn),et(2,nn)
		    write(11,'(a,3d30.20)') 'ph(1,nn),bt(1,nn),al(1,nn)=',
     &                                 ph(1,nn),bt(1,nn),al(1,nn)
		    write(11,'(a,3d30.20)') 'k1x(nn),k1y(nn),k1e(nn)=',
     &                                 k1x(nn),k1y(nn),k1e(nn)
		    write(11,'(a,4d30.20)') 'vx,vy,ve,u0=',vx,vy,ve,u0
		    write(11,'(a,1i8)') 'iflgce(nn)=',iflgce(nn)
        		close(11)
	        stop 'nancatchup1 xx(2,nn) in rung'
	      else
	        stop 'nancatchup1 xx(2,nn) in rung'
	      endif
	      endif
CCC   #################################################################
            if ((ihdmode.ne.0).and.(iflgce(nn).eq.2)) then
              ph(2,nn)= ph(1,nn)
              cycle
            end if
            eos(2)= ss(1,nn)
            eos(3)= bd(1,nn)
            call eosph(eos,2)
            edn= eos(1)
            Pr=  eos(4)
            q1x(nn)=(Pr+edn)/ss(1,nn)*u0*vx
            q1y(nn)=(Pr+edn)/ss(1,nn)*u0*vy
            q1e(nn)=(Pr+edn)/ss(1,nn)*u0*ve*tau1*tau1

            q2x(nn)= q1x(nn) + dq1(nn,1) *dtau
            q2y(nn)= q1y(nn) + dq1(nn,2) *dtau
            q2e(nn)= q1e(nn) + dq1(nn,3) *dtau
            ph(2,nn)= phif(q2x(nn),q2y(nn))
CCC   #################################################################
CCC   debug output
c	if ((tau/0.197d0.le.1.04d0).and.(bugflg.eq.15)) then
c      open(11,file=fname1,access='append')
c	       rr=dsqrt((xx(1,nn))**2+(yy(1,nn))**2)
c	       pp=dsqrt((q1x(nn))**2+(q1y(nn))**2)
c	       ff1=dsqrt((dq1(nn,1))**2+(dq1(nn,2))**2)
c	       vv1=dsqrt((k1x(nn))**2+(k1y(nn))**2)
c	       write(11,'(7d20.10,i8)')rr/.197d0,sn(1,nn),
c     &               q1x(nn),q1y(nn),pp,ff1,
c     &               vv1/.197d0/dtau,nn
c      close(11)
c	end if
CCC   #################################################################
CCC   debugging output
      if (iauxi.ge.2) then
	if ((q2e(nn).ge.0.d0).or.(q2e(nn).le.0.d0)) then
		continue
	elseif (iauxi.ge.5) then
          open(11,file=fnnan(1:nfnnan),access='append')
		WRITE(11,*)'q2e(nn) has unphysical value cp1'
		WRITE(11,*)'SPH particle number nn=',nn
		WRITE(11,'(a,3d30.20)') 'q2x(nn),q2y(nn),q2e(nn)=',
     &                             q2x(nn),q2y(nn),q2e(nn)
		WRITE(11,'(a,3d30.20)') 'q1x(nn),q1y(nn),q1e(nn)=',
     &                             q1x(nn),q1y(nn),q1e(nn)
		WRITE(11,'(a,3d30.20)') 'dq1(nn,1),dq1(nn,2),dq1(nn,3)=',
     &                             dq1(nn,1),dq1(nn,2),dq1(nn,3)
		WRITE(11,'(a,3d30.20)') 'Pr,edn,ss(1,nn)=',Pr,edn,ss(1,nn)
		WRITE(11,'(a,4d30.20)') 'u0,vx,vy,ve=',u0,vx,vy,ve
     		CLOSE(11)
	    stop 'fatal error at cp1 rung'
	else
	    stop 'fatal error at cp1 rung'
	endif
	endif
CCC   #################################################################
ccc   ///////////////////////////////////////////////////
 160     CONTINUE  !! oflg=1 => oflg=2
         oflg=2
         CALL mklt()
         tafn=tau2
         orfn=oflg
         nout=0
         DO 162 nn=1,Nspp
CCC   #################################################################
         if ((ihdmode.ne.0).and.(iflgce(nn).eq.2)) then
              al(2,nn)= 0.5d0*log((uni+tau1*ve)/(uni-tau1*ve)) +et(2,nn)  !should be tau1 not tau2!
	        if (abs(tau1*ve).ge.1.d0) stop 've fatal error in rung'
              athbt= sqrt(vx**2+vy**2) *cosh(al(2,nn)-et(2,nn)) 
CCC   #################################################################
CCC   NaN catchup
                  if (iauxi.ge.5) then
	            if (.not.(((uni+athbt)/(uni-athbt)).gt.0.d0)) then
                  open(11,file=fnnan(1:nfnnan),access='append')
                  write(11,*)'NaN concerning athbt in rung()!'
	            write(11,*)'athbt=',athbt
	            write(11,'(a,f16.8)')'((uni+athbt)/(uni-athbt))='
     &                                 ,((uni+athbt)/(uni-athbt))
                  close(11)
	            endif
	            endif
CCC   #################################################################
              bt(2,nn)= 0.5d0*log((uni+athbt)/(uni-athbt))
              cycle
            endif
CCC   #################################################################
CCC   new we calculate bt, hum
            eos(2)= ss(1,nn)
            eos(3)= bd(1,nn)
            call eosph(eos,2)
            Tm= eos(5)
            if(Tm.lt.Tfin) then
c     == very low temperature... == 
               al(2,nn)=al(1,nn)
               bt(2,nn)=bt(1,nn)
               goto 162
            end if
            xfn=  xx(2,nn)
            yfn=  yy(2,nn)
            efn=  et(2,nn)
            phfn= ph(2,nn)
            qxfn= q2x(nn)
            qyfn= q2y(nn)
            qefn= q2e(nn)
            select case (iviscosity)
            case (0)
               if(bt(1,nn).gt.1.d-01) then
               x1= bt(1,nn) *0.9500d+00
               x2= bt(1,nn) *1.0500d+00
               xacc= 0.001*bt(1,nn)
               else
               x1=   -1.d-01
               x2=   +1.d-01
               xacc=  2.d-05 
               end if
 180        continue
               bt(2,nn)= rtflsp1(findbt,x1,x2,xacc)
               if(lowtflg.ne.1.and.exflg.eq.1) then
               x1=x1*0.75d+00
               x2=x2*1.25d+00
                 if((x2.ge.0.d0).or.(x2.le.0.d0)) then
                   continue
                 else
                   if (iauxi.ge.8) then
                   open(11,file=fnnan(1:nfnnan),access='append')
                   write(11,*)'nancatchup x2 in rung place 1'
                   write(11,*)'x1,x2=',x1,x2
                   close(11)
                   endif
                   stop 'rung fatal error x2 place 1!'
                 endif
               goto 180
               endif
            case(1)
c               if(bt(1,nn).gt.1.d-01) then
c               x1= bt(1,nn) *0.9500d+00
c               x2= bt(1,nn) *1.0500d+00
c               xacc= 0.001*bt(1,nn)
c               else
c               x1=   -1.d-01
c               x2=   +1.d-01
c               xacc=  2.d-05 
c               end if
c 183        continue
c               bttemp2= rtflsp1(findbt,x1,x2,xacc)
c               if(lowtflg.ne.1.and.exflg.eq.1) then
c               x1=x1*0.75d+00
c               x2=x2*1.25d+00
c                 if((x2.ge.0.d0).or.(x2.le.0.d0)) then
c                   continue
c                 else
c                   if (iauxi.ge.8) then
c                   open(11,file=fnnan(1:nfnnan),access='append')
c                   write(11,*)'nancatchup x2 in rung place 1'
c                   write(11,*)'x1,x2=',x1,x2
c                   close(11)
c                   endif
c                   stop 'rung fatal error x2 place 1!'
c                 endif
c               goto 183
c               endif
c                 findbt_old=findbt(bttemp2)
c                 bttemp1=0.2d0
c                 findbt_new=0.2d0
                 u0fn=u0
	           bttemp1=bt(1,nn)
                 do lpfbt2=1,nfbt2
	              bttempx= bttemp1
                    bttemp1= findbt2()
	              if (abs((bttempx-bttemp1)/(bttempx+bttemp1))
     &                  .le.xaccfbt2) exit
                 enddo
c                 findbt_new= findbt(bttemp1)
                 nout=nout+1
                 bt(2,nn)=bttemp1
c                 if (nout.le.2) then
c                 open(22,file=fnfce(1:nfnfce),access='append')
c                 write(22,*)'  '
c                 write(22,*)'1st: tau, nout=',tafn, nout
c                 write(22,'(a,2d30.20)')'new bt value, findbt():',
c     &                       bttemp1,findbt_new
c                 write(22,'(a,2d30.20)')'old bt value, findbt():',
c     &                       bttemp2,findbt_old
c                 write(22,*)'neighbor particle number np=',np
c                 write(22,'(a,4d30.20)') 'eos(1-4)',
c     &                       eos(1),eos(2),eos(3),eos(4)
c                 write(22,'(a,3d30.20)') 'eos(5-7)',eos(5),eos(6),eos(7)
c                 write(22,'(a,1d30.20)') ',u0fn=',u0fn
c                 write(22,'(a,3d30.20)') 'xfn,yfn,efn=',xfn,yfn,efn
c                 write(22,'(a,2d30.20,i3)') 'phfn,tafn,orfn=',
c     &                       phfn,tafn,orfn
c                 write(22,'(a,3d30.20)') 'qxfn,qyfn,qefn=',
c     &                       qxfn,qyfn,qefn
c                 close(22)
c                 endif
            case default
              stop 'iviscosity fatal error 1 in rung'
            end select
            if((lowtflg.eq.1.or.lowsflg.eq.1).and.
     &         (ieosmode.eq.1.or.ieosmode.eq.2)) then
c            if(lowtflg.eq.1) then
               bt(2,nn)=bt(1,nn) !! very low temperature entropy density
            end if
            if(bt(2,nn).lt.zero) then
               bt(2,nn)=abs(bt(2,nn))
               ph(2,nn)= ph(2,nn)+pi
            end if
            if(abs(cos(ph(2,nn))).gt.epsilon) then
               if (q2x(nn).ne.0.d0) then   
               chi= cos(ph(2,nn))*tanh(bt(2,nn))
               chi= chi* (q2e(nn)/q2x(nn))/tau2
               else !this happens when everything is zero
               chi=0.d0
               end if
            else
               if (q2y(nn).ne.0.d0) then 
               chi= sin(ph(2,nn))*tanh(bt(2,nn))
               chi= chi* (q2e(nn)/q2y(nn))/tau2
               else !this happens when everything is zero
               chi= 0.d0
               end if
            end if
CCC   #################################################################
CCC   NaN catchup
      if (iauxi.ge.5) then
	if ((chi+sqrt(chi*chi+uni)).le.0.d0) then
	if ((iauxi.ge.10).or.(chi.gt.-0.58d9)) then
      open(11,file=fnnan(1:nfnnan),access='append')
      write(11,*)'NaN in rung!'
	write(11,'(a,f16.8)')'(chi+sqrt(chi*chi+uni))=',
     &           (chi+sqrt(chi*chi+uni))
      close(11)
      endif
      endif
      endif
CCC   #################################################################
            al(2,nn)= log(chi+sqrt(chi*chi+uni)) +et(2,nn)
      if (iauxi.ge.5) then
      if (chi.lt.-0.58d9) then		!!this also gives log(0.d0) in al
      al(2,nn)= -1.d0/2.d0/chi +et(2,nn)
      endif
      endif
ccc   ///////////////////////////////////////////////////////////////
CCC   #################################################################
CCC   debug output
	      if (iauxi.ge.2) then
            if ((al(2,nn).le.0.d0).or.(al(2,nn).ge.0.d0)) then
	        continue
	      elseif (iauxi.ge.5) then
              open(11,file=fnnan(1:nfnnan),access='append')
		    write(11,*)'nancatchup1 al(2,nn) in rung'
		    write(11,*)'SPH particle number nn=',nn
		    write(11,'(a,3d30.20)') 'xx(2,nn),yy(2,nn),et(2,nn)=',
     &                                 xx(2,nn),yy(2,nn),et(2,nn)
		    write(11,'(a,3d30.20)') 'ph(1,nn),bt(1,nn),al(1,nn)=',
     &                                 ph(1,nn),bt(1,nn),al(1,nn)
		    write(11,'(a,3d30.20)') 'q1x(nn),q1y(nn),q1e(nn)=',
     &                                 q1x(nn),q1y(nn),q1e(nn)
		    write(11,'(a,3d30.20)') 'q2x(nn),q2y(nn),q2e(nn)=',
     &                                 q2x(nn),q2y(nn),q2e(nn)
		    write(11,'(a,5d30.20)') 'vx,vy,ve,u0,chi=',vx,vy,ve,u0,chi
		    write(11,'(a,1i8)') 'iflgce(nn)=',iflgce(nn)
        		close(11)
	        stop 'nancatchup1 al(2,nn) in rung'
	      else
	        stop 'nancatchup1 al(2,nn) in rung'
	      endif
	      endif
CCC   #################################################################
 162     CONTINUE

ccc   2.nd order Runge-Kutta the second part:
CCC   #################################################################
      ELSE IF(oflg.eq.2) THEN  
         Nspp= Nsp(oflg) 

CCC   #################################################################
CCC   debug output
c	if ((tau/0.197d0.le.1.04d0).and.(bugflg.eq.15)) then
c	write(chtau,'(f10.7)')(tau/0.197d0)
c      fname2='char_rk2step'//chtau//'.data'
c      open(11,file=fname2,status='replace')
c      write(11,'(a)')'rr sn q1x q1y pp pp2 ff1 ff2 vv1 vv2 index'
c      write(11,'(a,d16.8)')'t=',tau/0.197d0
c	close(11)
c	end if
CCC   #################################################################
         DO 170 nn=1,Nspp
            vx= tanh(bt(2,nn))*cos(ph(2,nn))/cosh(al(2,nn)-et(2,nn))
            vy= tanh(bt(2,nn))*sin(ph(2,nn))/cosh(al(2,nn)-et(2,nn))
            ve= tanh(al(2,nn)-et(2,nn))/tau2
            u0=cosh(bt(2,nn))*cosh(al(2,nn)-et(2,nn))

            k2x(nn)= vx* dtau
            k2y(nn)= vy* dtau
            k2e(nn)= ve* dtau
            xx(1,nn)= xx(1,nn) + half*(k1x(nn)+k2x(nn))
            yy(1,nn)= yy(1,nn) + half*(k1y(nn)+k2y(nn))
            et(1,nn)= et(1,nn) + half*(k1e(nn)+k2e(nn))
CCC   #################################################################
CCC   debug output
            if ((xx(1,nn).le.0.d0).or.(xx(1,nn).ge.0.d0)) then
	        continue
	      elseif (iauxi.ge.5) then
              open(11,file=fnnan(1:nfnnan),access='append')
		    write(11,*)'nancatchup2 xx(1,nn) in rung'
		    write(11,*)'SPH particle number nn=',nn
		    write(11,'(a,3d30.20)') 'xx(1,nn),yy(1,nn),et(1,nn)=',
     &                                 xx(1,nn),yy(1,nn),et(1,nn)
		    write(11,'(a,3d30.20)') 'ph(2,nn),bt(2,nn),al(2,nn)=',
     &                                 ph(2,nn),bt(2,nn),al(2,nn)
		    write(11,'(a,3d30.20)') 'k2x(nn),k2y(nn),k2e(nn)=',
     &                                 k2x(nn),k2y(nn),k2e(nn)
		    write(11,'(a,3d30.20)') 'k1x(nn),k1y(nn),k1e(nn)=',
     &                                 k1x(nn),k1y(nn),k1e(nn)
		    write(11,'(a,4d30.20)') 'vx,vy,ve,u0=',vx,vy,ve,u0
		    write(11,'(a,1i8)') 'iflgce(nn)=',iflgce(nn)
        		close(11)
	        stop 'nancatchup2 xx(1,nn) in rung'
            else
	        stop 'nancatchup2 xx(1,nn) in rung'
	      end if
CCC   #################################################################
            if ((ihdmode.ne.0).and.(iflgce(nn).eq.2)) then
              ph(1,nn)=ph(2,nn)
              cycle
            end if
            q2x(nn)= q1x(nn) + half*(dq1(nn,1)+dq2(nn,1))*dtau
            q2y(nn)= q1y(nn) + half*(dq1(nn,2)+dq2(nn,2))*dtau
            q2e(nn)= q1e(nn) + half*(dq1(nn,3)+dq2(nn,3))*dtau
            ph(1,nn)= phif(q2x(nn),q2y(nn))
CCC   #################################################################
CCC   debug output
c	if ((tau/0.197d0.le.1.04d0).and.(bugflg.eq.15)) then
c      open(11,file=fname2,access='append')
c	       rr=dsqrt((xx(1,nn))**2+(yy(1,nn))**2)
c	       pp=dsqrt((q1x(nn))**2+(q1y(nn))**2)
c	       pp2=dsqrt((q2x(nn))**2+(q2y(nn))**2)
c	       ff1=dsqrt((dq1(nn,1))**2+(dq1(nn,2))**2)
c	       ff2=dsqrt((dq2(nn,1))**2+(dq2(nn,2))**2)
c	       vv1=dsqrt((k1x(nn))**2+(k1y(nn))**2)
c	       vv2=dsqrt((k2x(nn))**2+(k2y(nn))**2)
c	       write(11,'(10d20.10,i8)')rr/.197d0,sn(1,nn),
c     &               q1x(nn),q1y(nn),pp,pp2,ff1,ff2,
c     &               vv1/.197d0/dtau,vv2/.197d0/dtau,nn
c      close(11)
c	end if
CCC   #################################################################
CCC   debugging output
      if (iauxi.ge.2) then
	if ((q2e(nn).ge.0.d0).or.(q2e(nn).le.0.d0)) then
		continue
	elseif (iauxi.ge.5) then
          open(11,file=fnnan(1:nfnnan),access='append')
		WRITE(11,*)'q2e(nn) has unphysical value cp2'
		WRITE(11,*)'SPH particle number nn=',nn
		WRITE(11,'(a,3d30.20)') 'q2x(nn),q2y(nn),q2e(nn)=',
     &                             q2x(nn),q2y(nn),q2e(nn)
		WRITE(11,'(a,3d30.20)') 'q1x(nn),q1y(nn),q1e(nn)=',
     &                             q1x(nn),q1y(nn),q1e(nn)
		WRITE(11,'(a,3d30.20)') 'dq2(nn,1),dq2(nn,2),dq2(nn,3)=',
     &                             dq2(nn,1),dq2(nn,2),dq2(nn,3)
		WRITE(11,'(a,3d30.20)') 'Pr,edn,ss(1,nn)=',Pr,edn,ss(1,nn)
		WRITE(11,'(a,4d30.20)') 'u0,vx,vy,ve=',u0,vx,vy,ve
     		CLOSE(11)
	    stop 'fatal error at cp2 rung'
	else
	    stop 'fatal error at cp2 rung'
	endif
	endif
CCC   #################################################################
ccc   ///////////////////////////////////////////////////////////////
 170     CONTINUE !! oflg=2 => oflg=1   
         tau2=tau+dtau
         oflg=1
         Nspp=Nsp(oflg)
         CALL mklt()
         orfn= oflg
         tafn= tau2
         nout=0
         DO 172 nn=1,Nspp
            if ((ihdmode.ne.0).and.(iflgce(nn).eq.2)) then
              bt(1,nn)=bt(2,nn)
	        al(1,nn)=al(2,nn)
              cycle
            end if
            eos(2)= ss(2,nn)
            eos(3)= bd(2,nn)
            call eosph(eos,2)
            Tm= eos(5)
            if(Tm.lt.Tfin) then
c     == very low temperature... == 
               al(1,nn)=al(2,nn)
               bt(1,nn)=bt(2,nn)
               goto 172
            end if
            xfn=  xx(1,nn)
            yfn=  yy(1,nn)
            efn=  et(1,nn)
            phfn= ph(1,nn)
            qxfn= q2x(nn)
            qyfn= q2y(nn)
            qefn= q2e(nn)
            select case (iviscosity)
            case (0)
               if(bt(2,nn).gt.1.d-01) then
               x1= bt(2,nn) *0.9500d+00
               x2= bt(2,nn) *1.0500d+00
               xacc= 0.001*bt(2,nn)
               else
               x1=  -1.d-01
               x2=  +1.d-01
               xacc= 2.d-05
               end if
 190        continue
               bt(1,nn)= rtflsp1(findbt,x1,x2,xacc)
               if(lowtflg.ne.1.and.exflg.eq.1) then
               x1=x1*0.950D+00		!!changed from 0.75 and 1.25 to small changes
               x2=x2*1.050d+00
               if((x2.ge.0.d0).or.(x2.le.0.d0)) then
                 continue
               else
                 if (iauxi.ge.8) then
                 open(11,file=fnnan(1:nfnnan),access='append')
                 write(11,*)'nancatchup x2 in rung place 2'
                 write(11,*)'x1,x2=',x1,x2
                 close(11)
                 endif
                 stop 'rung fatal error x2 place 2!'
               endif
               goto 190
               endif
            case(1)
c               if(bt(2,nn).gt.1.d-01) then
c               x1= bt(2,nn) *0.9500d+00
c               x2= bt(2,nn) *1.0500d+00
c               xacc= 0.001*bt(2,nn)
c               else
c               x1=  -1.d-01
c               x2=  +1.d-01
c               xacc= 2.d-05
c               end if
c 193       continue
c               bttemp2= rtflsp1(findbt,x1,x2,xacc)
c               if(lowtflg.ne.1.and.exflg.eq.1) then
c               x1=x1*0.950D+00		!!changed from 0.75 and 1.25 to small changes
c               x2=x2*1.050d+00
c               if((x2.ge.0.d0).or.(x2.le.0.d0)) then
c                 continue
c               else
c                 if (iauxi.ge.8) then
c                 open(11,file=fnnan(1:nfnnan),access='append')
c                 write(11,*)'nancatchup x2 in rung place 2'
c                 write(11,*)'x1,x2=',x1,x2
c                 close(11)
c                 endif
c                 stop 'rung fatal error x2 place 2!'
c               endif
c               goto 193
c               endif
c                 findbt_old= findbt(bttemp2)
c                 bttemp1=0.2d0
c                 findbt_new=0.2d0
                 u0fn=u0
	           bttemp1=bt(2,nn)
                 do lpfbt2=1,nfbt2
	              bttempx= bttemp1
                    bttemp1= findbt2()		!u0fn should already been defined
	              if (abs((bttempx-bttemp1)/(bttempx+bttemp1))
     &                  .le.xaccfbt2) exit
                 enddo
c                 findbt_new= findbt(bttemp1)
                 nout=nout+1
                 bt(1,nn)=bttemp1
c                 if (nout.le.2) then
c                 open(22,file=fnfce(1:nfnfce),access='append')
c                 write(22,*)'  '
c                 write(22,*)'2nd: tau, nout=',tafn,nout
c                 write(22,'(a,2d30.20)')'new bt value, findbt():',
c     &                       bttemp1,findbt_new
c                 write(22,'(a,2d30.20)')'old bt value, findbt():',
c     &                       bttemp2,findbt_old
c                 write(22,*)'neighbor particle number np=',np
c                 write(22,'(a,4d30.20)') 'eos(1-4)',
c     &                       eos(1),eos(2),eos(3),eos(4)
c                 write(22,'(a,3d30.20)') 'eos(5-7)',eos(5),eos(6),eos(7)
c                 write(22,'(a,1d30.20)') ',u0fn=',u0fn
c                 write(22,'(a,3d30.20)') 'xfn,yfn,efn=',xfn,yfn,efn
c                 write(22,'(a,2d30.20,i3)') 'phfn,tafn,orfn=',
c     &                       phfn,tafn,orfn
c                 write(22,'(a,3d30.20)') 'qxfn,qyfn,qefn=',
c     &                       qxfn,qyfn,qefn
c                 close(22)
c                 endif
            case default
              stop 'iviscosity fatal error 2 in rung'
            end select
            if((lowtflg.eq.1.or.lowsflg.eq.1).and.
     &         (ieosmode.eq.1.or.ieosmode.eq.2)) then
c            if(lowtflg.eq.1) then
               bt(1,nn)=bt(2,nn) !!  == very low temperature... == 
            end if
            if(bt(1,nn).lt.zero) then
               bt(1,nn)=abs(bt(1,nn))
               ph(1,nn)= ph(1,nn)+pi
            end if
            if(abs(cos(ph(1,nn))).gt.epsilon) then
               if (q2x(nn).ne.0.d0) then
               chi= cos(ph(1,nn))*tanh(bt(1,nn))
               chi= chi* (q2e(nn)/q2x(nn))/tau2
               else !this happens when everything is zero
               chi= 0.d0 
               end if  
            else
               if (q2y(nn).ne.0.d0) then
               chi= sin(ph(1,nn))*tanh(bt(1,nn))
               chi= chi* (q2e(nn)/q2y(nn))/tau2 
               else !this happens when everything is zero
               chi= 0.d0
               end if
            end if
CCC   #################################################################
CCC   NaN catchup
      if (iauxi.ge.5) then
	if ((chi+sqrt(chi*chi+uni)).le.0.d0) then
	if ((iauxi.ge.10).or.(chi.gt.-0.58d9)) then
      open(11,file=fnnan(1:nfnnan),access='append')
      write(11,*)'NaN in rung!'
	write(11,'(a,f16.8)')'(chi+sqrt(chi*chi+uni))=',
     &           (chi+sqrt(chi*chi+uni))
      close(11)
      endif
      endif
      endif
CCC   #################################################################
            al(1,nn)= log(chi+sqrt(chi*chi+uni))+et(1,nn)

      if (iauxi.ge.5) then
      if (chi.lt.-0.58d9) then		!!this also gives log(0.d0) in alfn
      al(1,nn)= -1.d0/2.d0/chi +et(1,nn)
      endif
      endif
CCC   #################################################################
CCC   debug output
            if ((al(1,nn).le.0.d0).or.(al(1,nn).ge.0.d0)) then
	        continue
	      elseif (iauxi.ge.5) then
              open(11,file=fnnan(1:nfnnan),access='append')
		    write(11,*)'nancatchup2 al(1,nn) in rung'
		    write(11,*)'SPH particle number nn=',nn
		    write(11,'(a,3d30.20)') 'xx(1,nn),yy(1,nn),et(1,nn)=',
     &                                 xx(1,nn),yy(1,nn),et(1,nn)
		    write(11,'(a,3d30.20)') 'ph(2,nn),bt(2,nn),al(2,nn)=',
     &                                 ph(2,nn),bt(2,nn),al(2,nn)
		    write(11,'(a,3d30.20)') 'q2x(nn),q2y(nn),q2e(nn)=',
     &                                 q2x(nn),q2y(nn),q2e(nn)
		    write(11,'(a,3d30.20)') 'q1x(nn),q1y(nn),q1e(nn)=',
     &                                 q1x(nn),q1y(nn),q1e(nn)
		    write(11,'(a,5d30.20)') 'vx,vy,ve,u0,chi=',vx,vy,ve,u0,chi
		    write(11,'(a,1i8)') 'iflgce(nn)=',iflgce(nn)
        		close(11)
	        stop 'nancatchup2 al(1,nn) in rung'
            else
	        stop 'nancatchup2 al(1,nn) in rung'
	      end if
CCC   #################################################################

 172     CONTINUE
       END IF
ccc   ################################################################
 175  CONTINUE


      RETURN 
      END 




CCC   #################################################################
CCC   SUBROUTINE PROGRAM: rung.f 
CCC   'rung' solves equation of motion of SPH-particles:    
CCC   Using results by dqfind(), it renews quantity 'q' and 
CCC   finds the difference of corresponding quantities
CCC   The difinition of dq is the following; 
CCC   #################################################################
CCC   Convention of dq_vis
CCC   1 - u1
CCC   2 - u2
CCC   3 - u3
CCC   4 - entropy ratio (sn/sgmn)
CCC   5 - bulk viscosity ratio (pibn/sgmn)
CCC   6 - 10 shear viscosity
CCC   #################################################################
CCC   last udpated 06122009
CCC   the framework is built
CCC   #################################################################
      SUBROUTINE rung_vis(tau,dq_vis1,dq_vis2)
       
      IMPLICIT NONE
      include 'spherio.inc'
      include 'spherio.eos.inc'
      include 'spherio.quifz.inc'
      include 'spherio.debug.inc'
      include 'spherio.temps.inc'
	include 'spherio.hydro.inc'
      include 'epos.incsf'

      DOUBLE PRECISION dq_vis1(NPL,5),dq_vis2(NPL,5)
      DOUBLE PRECISION q_vis1(NPL,5),q_vis2(NPL,5)
      DOUBLE PRECISION k1x(NPL),k1y(NPL),k1e(NPL)
      DOUBLE PRECISION k2x(NPL),k2y(NPL),k2e(NPL)
      COMMON/QKVIS/q_vis1,q_vis2,k1x,k1y,k1e,k2x,k2y,k2e
      INTEGER Nspp,nn
      DOUBLE PRECISION tau,tau1,tau2
      DOUBLE PRECISION vx,vy,ve,u0
      DOUBLE PRECISION Tm
      DOUBLE PRECISION phif
	INTEGER kflg
      INTEGER exflg,lowtflg,lowbflg,lowsflg
 
      COMMON/LOWT/lowtflg,lowbflg,lowsflg
      COMMON/EXF/exflg
      
      exflg=0
      tau1=tau
      tau2=tau+dtau
      
ccc   2.nd order Runge-Kutta the first part:
ccc   ################################################################
      IF(oflg.eq.1) THEN
      Nspp= Nsp(1)
      DO nn=1,Nspp
            vx= tanh(bt(1,nn))*cos(ph(1,nn))/cosh(al(1,nn)-et(1,nn))
            vy= tanh(bt(1,nn))*sin(ph(1,nn))/cosh(al(1,nn)-et(1,nn))
            ve= tanh(al(1,nn)-et(1,nn))/tau1
            u0= cosh(bt(1,nn))*cosh(al(1,nn)-et(1,nn))

            k1x(nn)= vx* dtau
            k1y(nn)= vy* dtau
            k1e(nn)= ve* dtau
            xx(2,nn)= xx(1,nn) + k1x(nn)
            yy(2,nn)= yy(1,nn) + k1y(nn)
            et(2,nn)= et(1,nn) + k1e(nn)
CCC   #################################################################
CCC   debug output
	      if (iauxi.ge.2) then
            if ((xx(2,nn).le.0.d0).or.(xx(2,nn).ge.0.d0)) then
	        continue
	      elseif (iauxi.ge.5) then
              open(11,file=fnnan(1:nfnnan),access='append')
		    write(11,*)'nancatchup xx(2,nn) in rung'
		    write(11,*)'SPH particle number nn=',nn
		    write(11,'(a,3d30.20)') 'xx(2,nn),yy(2,nn),et(2,nn)=',
     &                                 xx(2,nn),yy(2,nn),et(2,nn)
		    write(11,'(a,3d30.20)') 'ph(1,nn),bt(1,nn),al(1,nn)=',
     &                                 ph(1,nn),bt(1,nn),al(1,nn)
		    write(11,'(a,3d30.20)') 'k1x(nn),k1y(nn),k1e(nn)=',
     &                                 k1x(nn),k1y(nn),k1e(nn)
		    write(11,'(a,4d30.20)') 'vx,vy,ve,u0=',vx,vy,ve,u0
		    write(11,'(a,1i8)') 'iflgce(nn)=',iflgce(nn)
        		close(11)
	        stop 'nancatchup xx(2,nn) in rung_vis'
	      else
	        stop 'nancatchup xx(2,nn) in rung_vis'
	      endif
	      endif
CCC   #################################################################
            q_vis1(nn,1)=vx*u0
            q_vis1(nn,2)=vy*u0
            q_vis1(nn,3)=ve*u0
            q_vis1(nn,4)=sn(1,nn)/sgmn(1,nn)
	      q_vis1(nn,5)=pibn(1,nn)/sgmn(1,nn)
	      do kflg=1,5
            q_vis2(nn,kflg)=q_vis1(nn,kflg)+dq_vis1(nn,kflg)*dtau
            enddo
CCC   #################################################################
CCC   debugging output
      if (iauxi.ge.2) then
	if ((q_vis2(nn,1).ge.0.d0).or.(q_vis2(nn,1).le.0.d0)) then
		continue
	elseif (iauxi.ge.5) then
          open(11,file=fnnan(1:nfnnan),access='append')
		WRITE(11,*)'q_vis2(nn,1) has unphysical value cp1'
		WRITE(11,*)'SPH particle number nn=',nn
		WRITE(11,'(a,1d30.20)') 'q_vis2(nn,1)=',q_vis2(nn,1)
		WRITE(11,'(a,1d30.20)') 'q_vis1(nn,1)=',q_vis1(nn,1)
		WRITE(11,'(a,1d30.20)') 'dq_vis1(nn,1)=',dq_vis1(nn,1)
		WRITE(11,'(a,4d30.20)') 'u0,vx,vy,ve=',u0,vx,vy,ve
     		CLOSE(11)
	    stop 'fatal error at cp1 rung_vis'
	else
	    stop 'fatal error at cp1 rung_vis'
	endif
	endif
CCC   #################################################################
ccc   ///////////////////////////////////////////////////
      ENDDO ! of nn of the 1st of the first part of RK
      oflg=2
      CALL mklt()
      DO nn=1,Nspp
CCC   #################################################################
            ph(2,nn)= phif(q_vis2(nn,1),q_vis2(nn,2))
	      u0= dsqrt(q_vis2(nn,1)**2+q_vis2(nn,2)**2
     &        +tau2**2*q_vis2(nn,3)**2+1.d0)
	      vx= q_vis2(nn,1)/u0
	      vy= q_vis2(nn,2)/u0
	      ve= q_vis2(nn,3)/u0
            eos(2)= ss(1,nn)
            eos(3)= bd(1,nn)
            call eosph(eos,2)
            Tm= eos(5)
            if(Tm.lt.Tfin) then
c     == very low temperature... == 
               al(2,nn)=al(1,nn)
               bt(2,nn)=bt(1,nn)
               sn(2,nn)=sn(1,nn)
               pibn(2,nn)=pibn(1,nn)
               cycle
            end if
            select case (iviscosity)
            case (2,3)
              al(2,nn)= half*dlog((uni+(tau2*ve))/(uni-(tau2*ve)))
              al(2,nn)= al(2,nn)+ et(2,nn)
              bt(2,nn)= dsqrt(q_vis2(nn,1)**2+q_vis2(nn,2)**2)
              bt(2,nn)= dlog(dsqrt(1.d0+bt(2,nn)**2)+bt(2,nn))
              sn(2,nn)= sgmn(2,nn)*q_vis2(nn,4)
              pibn(2,nn)= sgmn(2,nn)*q_vis2(nn,5)
            case default
              write(*,*) ' iviscosity fatal error in rung_vis'
              stop ' iviscosity fatal error in rung_vis'
            end select	
      ENDDO !of nn of the 2nd part of the first part of RK

ccc   2.nd order Runge-Kutta the second part:
CCC   #################################################################
      ELSE IF(oflg.eq.2) THEN  
      Nspp= Nsp(oflg) 
      DO nn=1,Nspp
            vx= tanh(bt(2,nn))*cos(ph(2,nn))/cosh(al(2,nn)-et(2,nn)) !it is necessary since the script is called the second time
            vy= tanh(bt(2,nn))*sin(ph(2,nn))/cosh(al(2,nn)-et(2,nn))
            ve= tanh(al(2,nn)-et(2,nn))/tau2
            u0=cosh(bt(2,nn))*cosh(al(2,nn)-et(2,nn))

            k2x(nn)= vx* dtau
            k2y(nn)= vy* dtau
            k2e(nn)= ve* dtau
            xx(1,nn)= xx(1,nn) + half*(k1x(nn)+k2x(nn))
            yy(1,nn)= yy(1,nn) + half*(k1y(nn)+k2y(nn))
            et(1,nn)= et(1,nn) + half*(k1e(nn)+k2e(nn))
CCC   #################################################################
CCC   debug output
            if ((xx(1,nn).le.0.d0).or.(xx(1,nn).ge.0.d0)) then
	        continue
	      elseif (iauxi.ge.5) then
              open(11,file=fnnan(1:nfnnan),access='append')
		    write(11,*)'nancatchup xx(1,nn) in rung'
		    write(11,*)'SPH particle number nn=',nn
		    write(11,'(a,3d30.20)') 'xx(1,nn),yy(1,nn),et(1,nn)=',
     &                                 xx(1,nn),yy(1,nn),et(1,nn)
		    write(11,'(a,3d30.20)') 'ph(2,nn),bt(2,nn),al(2,nn)=',
     &                                 ph(2,nn),bt(2,nn),al(2,nn)
		    write(11,'(a,3d30.20)') 'k2x(nn),k2y(nn),k2e(nn)=',
     &                                 k2x(nn),k2y(nn),k2e(nn)
		    write(11,'(a,3d30.20)') 'k1x(nn),k1y(nn),k1e(nn)=',
     &                                 k1x(nn),k1y(nn),k1e(nn)
		    write(11,'(a,4d30.20)') 'vx,vy,ve,u0=',vx,vy,ve,u0
		    write(11,'(a,1i8)') 'iflgce(nn)=',iflgce(nn)
        		close(11)
	        stop 'nancatchup xx(1,nn) in rung_vis'
            else
	        stop 'nancatchup xx(1,nn) in rung_vis'
	      end if
CCC   #################################################################
	      do kflg=1,5
            q_vis2(nn,kflg)=q_vis1(nn,kflg)+
     &           half*(dq_vis1(nn,kflg)+dq_vis2(nn,kflg))*dtau
            enddo
CCC   #################################################################
CCC   debugging output
CCC   debugging output
      if (iauxi.ge.2) then
	if ((q_vis2(nn,1).ge.0.d0).or.(q_vis2(nn,1).le.0.d0)) then
		continue
	elseif (iauxi.ge.5) then
          open(11,file=fnnan(1:nfnnan),access='append')
		WRITE(11,*)'q_vis2(nn,1) has unphysical value cp1'
		WRITE(11,*)'SPH particle number nn=',nn
		WRITE(11,'(a,1d30.20)') 'q_vis2(nn,1)=',q_vis2(nn,1)
		WRITE(11,'(a,1d30.20)') 'q_vis1(nn,1)=',q_vis1(nn,1)
		WRITE(11,'(a,1d30.20)') 'dq_vis1(nn,1)=',dq_vis1(nn,1)
		WRITE(11,'(a,4d30.20)') 'u0,vx,vy,ve=',u0,vx,vy,ve
     		CLOSE(11)
	    stop 'fatal error at cp2 rung_vis'
	else
	    stop 'fatal error at cp2 rung_vis'
	endif
	endif
CCC   #################################################################
ccc   ///////////////////////////////////////////////////////////////
      ENDDO !of nn of the 2nd part of the second part of RK 
ccc   oflg=2 => oflg=1   
      tau2=tau+dtau
      oflg=1
      Nspp=Nsp(oflg)
      CALL mklt()
      DO nn=1,Nspp
            ph(1,nn)= phif(q_vis2(nn,1),q_vis2(nn,2))
	      u0= dsqrt(q_vis2(nn,1)**2+q_vis2(nn,2)**2
     &        +tau2**2*q_vis2(nn,3)**2+1.d0)
	      vx= q_vis2(nn,1)/u0
	      vy= q_vis2(nn,2)/u0
	      ve= q_vis2(nn,3)/u0
            eos(2)= ss(2,nn)
            eos(3)= bd(2,nn)
            call eosph(eos,2)
            Tm= eos(5)
            if(Tm.lt.Tfin) then
c     == very low temperature... == 
               al(1,nn)=al(2,nn)
               bt(1,nn)=bt(2,nn)
               sn(1,nn)=sn(2,nn)
               pibn(1,nn)=pibn(2,nn)
               cycle
            end if
            select case (iviscosity)
            case (2,3)
              al(1,nn)= half*dlog((uni+(tau2*ve))/(uni-(tau2*ve)))
              al(1,nn)= al(1,nn)+ et(1,nn)
              bt(1,nn)= dsqrt(q_vis2(nn,1)**2+q_vis2(nn,2)**2)
              bt(1,nn)= dlog(dsqrt(1.d0+bt(1,nn)**2)+bt(1,nn))
              sn(1,nn)= sgmn(1,nn)*q_vis2(nn,4)
              pibn(1,nn)= sgmn(1,nn)*q_vis2(nn,5)
            case default
              write(*,*) ' iviscosity fatal error in rung_vis'
              stop ' iviscosity fatal error in rung_vis'
            end select

      ENDDO !of the 2nd of the second part of RK
      END IF
ccc   ################################################################


      RETURN 
      END 




CCC   #################################################################
CCC   subroutine program solvediff.f
CCC   "solvediff" controls subroutines and to solve
CCC    quation of motion of SPH and particle decay and eliminations.
CCC   'ntsp' is the tau-step number
CCC   #################################################################
CCC   last updated 01032009
CCC   modified the code to handle particle low temperature cut-off
CCC   at ntsp=0
CCC
CCC   last updated 01012009
CCC   implemented split control keyword
CCC
CCC   last updated 12272008
CCC   treat zero ic energy
CCC
CCC   last updated 12162008
CCC   add more debug output
CCC
CCC   last updated 09022008
CCC   modified the expression of etot
CCC
CCC   last updated 08192008
CCC   implemented the etot check
CCC
CCC   last updated 07152008
CCC   adopt the new EOM scenario by Yogiro
CCC
CCC   last updated 10092007
CCC   add check output
CCC
CCC   last updated 08312007
CCC   adopt vvxn
CCC
CCC   last updated 08202007
CCC   adopt improved ihdmode check
CCC
CCC   last updated 08162007
CCC   add ihdfull check
CCC   add debug output
CCC
CCC   last updated 06282007
CCC   modifed the output
CCC
CCC   last updated 05212007
CCC   modified some output
CCC   adopt uxn
CCC
CCC   last updated 04272007
CCC   add output of ideco
CCC
CCC   last updated 03022007
CCC   tau2 is modified
CCC
CCC   last updated 02192007
CCC   add more debugging check
CCC
CCC   last updated 01242007
CCC   fixed the bug about solvediff
CCC
CCC   last updated 12192006
CCC   modified to incorporate tempxn in new chemical fo
CCC
CCC   last updated 08152006
CCC   added debug infomation for particle loosing
CCC
CCC   last updated 05042006
CCC   include 'spherio.eos.inc' to incorporate the strangeness
CCC   #################################################################
      SUBROUTINE solvediff(ntsp,tau)

      IMPLICIT NONE 
      include 'spherio.inc'
      include 'spherio.eos.inc'
      include 'spherio.debug.inc'
      include 'spherio.quifz.inc'
      include 'spherio.rfactor.inc'
      include 'spherio.ce.inc'
      include 'spherio.temps.inc'
      include 'spherio.sphsplit.inc'
	include 'spherio.hydro.inc'
      include 'epos.incsf'

      INTEGER Nspp,ntsp,na,nb,nn,mm,k1,k2,kk
      INTEGER indx(NPL) 
      DOUBLE PRECISION tau,tau1,tau2 
      DOUBLE PRECISION xxa,yya,eta   
      DOUBLE PRECISION dq1(NPL,3),dq2(NPL,3)
	DOUBLE PRECISION dq_vis1(NPL,5),dq_vis2(NPL,5)
      DOUBLE PRECISION arr(NPL)
      INTEGER booloss,nacut,natot
      DOUBLE PRECISION gamma,uu3

      IF(ntsp.eq.0) then
        call firststep(tau)
        if (iauxi.ge.6) then
          OPEN(11,FILE=fnztr(1:nfnztr),STATUS='UNKNOWN',ACCESS='APPEND')
            WRITE(11,*) ('-',kk=1,63)
            WRITE(11,'(a,d30.20)')' total entropy=', stot
            WRITE(11,'(a,d30.20)')' total baryon =', btot
            WRITE(11,'(a,d30.20)')' total energy =', etot
            WRITE(11,*) ('-',kk=1,63) 
          CLOSE(11)
        endif
        if (stot.eq.0.d0) then
          OPEN(11,FILE=fnztr(1:nfnztr),STATUS='UNKNOWN',ACCESS='APPEND')
            WRITE(11,*) ('-',kk=1,63)
            WRITE(11,'(a)')' Total IC Entropy is zero'
            WRITE(11,'(a)')' SPheRIO is forced to stop, check the IC!'
            WRITE(11,*) ('-',kk=1,63)
          CLOSE(11)
            WRITE(*,'(a)')' Total IC Entropy is zero'
            WRITE(*,'(a)')' SPheRIO is forced to stop, check the IC!'
            STOP ' Total IC entropy is ZERO, problem with IC'
          return
        endif
            slst=zero
	      slsteta=zero
            blst=zero
	      elst=zero
            sres=zero
	      sreseta=zero
            bres=zero
	      eres=zero
	  goto 7653
      END IF

      if ((ihdfull.ne.0).and.(ihdmode.eq.1).and.(icemode.ne.0))
     &       stop ' ihdfull fatal error 1 in solvediff!'
      if ((ihdfull.ne.1).and.(ihdmode.eq.0))
     &       stop ' ihdfull fatal error 2 in solvediff!'
CCC   #################################################################
CCC   save before step SPH particle information:
CCC   k1=-2, 2 tau step before, k1=1 one tau step before. 
      do 202 k1=-2,-1
CCC   #################################################################
         if(ntsp.eq.1.and.k1.eq.-2) goto 202
         if(k1.eq.-2) k2=-1
         if(k1.eq.-1) k2= 1 
CCC   #################################################################
         do na=1,Nsp(k2)
            xx(k1,na)= xx(k2,na)
            yy(k1,na)= yy(k2,na)
            et(k1,na)= et(k2,na)
            al(k1,na)= al(k2,na)
            bt(k1,na)= bt(k2,na)
            ph(k1,na)= ph(k2,na)
CCC   #################################################################
            sn(k1,na)= sn(k2,na)
            pibn(k1,na)=pibn(k2,na)
            sgmn(k1,na)=sgmn(k2,na)
            br(k1,na)= br(k2,na)
            sr(k1,na)= sr(k2,na)
            qr(k1,na)= qr(k2,na)
CCC   #################################################################
            ss(k1,na)= ss(k2,na)
            pibd(k1,na)=pibd(k2,na)
            sgmd(k1,na)=sgmd(k2,na)
            bd(k1,na)= bd(k2,na)
            sd(k1,na)= sd(k2,na)
            qd(k1,na)= qd(k2,na)
CCC   #################################################################
        end do
        Nsp(k1)=Nsp(k2)
 202  continue
CCC   #################################################################
CCC   SPheRIO Particle decay part...
      select case (ipsplit)
	case (0)
          call dectrl_off(tau)
	case (1)
	    call dectrl(tau)
	case default
	    print*, 'ipsplit error in solvediff'
	    stop 'ipsplit error in solvediff'
      end select
CCC   #################################################################
      oflg=1 
      tau1=tau 
      Nspp=Nsp(oflg)
      CALL mklt()
      do na=1,Nsp(oflg)  
         xxa= xx(oflg,na)
         yya= yy(oflg,na)
         eta= et(oflg,na)
         CALL prcn(xxa,yya,eta)
         CALL fins(tau1,na)
         CALL bsqd(tau1,na)
      end do 
      CALL mklt()
      do na=1,Nspp 
         xxa= xx(oflg,na)
         yya= yy(oflg,na)
         eta= et(oflg,na)
         CALL prcn(xxa,yya,eta)
c         if(npflg.eq.0) goto 325
	   if(((npflg.eq.0).and.(ihdmode.eq.0)).or.
     &      ((npflg.eq.0).and.(ihdmode.ne.0).and.(iflgce(na).eq.0)))
     &                               goto 325     !! error message
         select case (iviscosity)
	   case(0,1)
              CALL dqfn(tau1,na,dq1,1)
         case(2)
              CALL dqfn_bulk(tau1,na,dq_vis1)
         case(3)
              CALL dqfn_shear()
         end select
      end do

      select case (iviscosity)
	case(0,1)
           CALL rung(tau,dq1,dq2)
      case(2,3)
           CALL rung_vis(tau,dq_vis1,dq_vis2)
      end select

CCC   #################################################################

      oflg=2
      Nspp=Nsp(2)
      tau2=tau+dtau
      CALL mklt()
      do na=1,Nspp
         xxa= xx(oflg,na)
         yya= yy(oflg,na)
         eta= et(oflg,na)
         CALL prcn(xxa,yya,eta)
         CALL fins(tau2,na)
         CALL bsqd(tau2,na)
      end do
      CALL mklt()
      do na=1,Nsp(oflg)
         xxa= xx(oflg,na)
         yya= yy(oflg,na)
         eta= et(oflg,na)
         CALL prcn(xxa,yya,eta)
c         if(npflg.eq.0) goto 325
	   if(((npflg.eq.0).and.(ihdmode.eq.0)).or.
     &      ((npflg.eq.0).and.(ihdmode.ne.0).and.(iflgce(na).eq.0)))
     &                               goto 325     !! error message
         select case (iviscosity)
	   case(0,1)
              CALL dqfn(tau2,na,dq2,2)
         case(2)
              CALL dqfn_bulk(tau2,na,dq_vis2)
         case(3)
              CALL dqfn_shear()
         end select
      end do

      select case (iviscosity)
	case(0,1)
           CALL rung(tau,dq1,dq2)
      case(2,3)
           CALL rung_vis(tau,dq_vis1,dq_vis2)
      end select

CCC   #################################################################
      tau=tau+dtau                          !! tau evolution
      oflg=1
      CALL mklt()
      do na=1,Nsp(1) 
         CALL prcn(xx(oflg,na),yy(oflg,na),et(oflg,na))
         CALL fins(tau,na)                  !! entropy renewal
         CALL bsqd(tau,na)
      end do
CCC   #################################################################
      if(ntsp.ge.2) then
         do nn=1, Nsp(-1)
            idp(0,nn)=idp(-1,nn)
         end do
      end if
CCC   #################################################################
7653  continue
CCC   #################################################################
ccc   re-arrenging particles in the descending order of temperature
ccc   using "indexx" in Numerical recipes.
CCC   #################################################################
      do nn=1,Nsp(1)
         eos(2)=ss(1,nn)
         eos(3)=bd(1,nn)
         call eosph(eos,2)
         arr(nn)=eos(5)
      end do

      call indexx(Nsp(1),NPL,arr,indx)

      na=0
      do nn=Nsp(1),1,-1
         na=na+1
         mm=indx(nn)
         idp(-1,na)=mm
         if(mm.le.Nsp(-1)) idp(-1,na)=mm
         if(mm.gt.Nsp(-1)) idp(-1,na)=ndpr(mm)
         if(ntsp.ge.2) idp(-2,na)= idp(0,idp(-1,na))
         nb= idp(-1,na)
         if(ntsp.ge.2) idq(-2,nb)= idp(0,idp(-1,na))

CCC   #################################################################
CCC   Emissao Continua e Quimico fo
         iflgce_aux(na) = iflgce(mm)
         iflgxn_aux(na) = iflgxn(mm)
         mbxn_aux(na)   = mbxn(mm)
         msxn_aux(na)   = msxn(mm)
         tempxn_aux(na) = tempxn(mm)
         ssxn_aux(na)   = ssxn(mm)
         bbxn_aux(na)   = bbxn(mm)
         do kk=0,3
           uxn_aux(kk,na) = uxn(kk,mm)
         enddo
         iflgfp_aux(na) = iflgfp(mm)
CCC   #################################################################
         xx(0,na)= xx(1,mm)
         yy(0,na)= yy(1,mm)
         et(0,na)= et(1,mm)
         al(0,na)= al(1,mm)
         bt(0,na)= bt(1,mm)
         ph(0,na)= ph(1,mm)
CCC   #################################################################
         sn(0,na)= sn(1,mm)
         pibn(0,na)= pibn(1,mm)
         sgmn(0,na)= sgmn(1,mm)
         br(0,na)= br(1,mm)
         sr(0,na)= sr(1,mm)
         qr(0,na)= qr(1,mm)
CCC   #################################################################
         ss(0,na)= ss(1,mm)
         pibd(0,na)= pibd(1,mm)
         sgmd(0,na)= sgmd(1,mm)
         bd(0,na)= bd(1,mm)
         sd(0,na)= sd(1,mm)
         qd(0,na)= qd(1,mm)
CCC   #################################################################
      end do


      do na=1,Nsp(1)
CCC   #################################################################
CCC   Emissao Continua e Quimico fo
CCC   #################################################################
         iflgce(na) = iflgce_aux(na)
         iflgxn(na) = iflgxn_aux(na)
         mbxn(na)   = mbxn_aux(na)
         msxn(na)   = msxn_aux(na)
         tempxn(na) = tempxn_aux(na)
         ssxn(na)   = ssxn_aux(na)
         bbxn(na)   = bbxn_aux(na)
         do kk=0,3
           uxn(kk,na) = uxn_aux(kk,na)
         enddo
         iflgfp(na) = iflgfp_aux(na)
CCC   #################################################################
         xx(1,na)= xx(0,na)
         yy(1,na)= yy(0,na)
         et(1,na)= et(0,na)
         al(1,na)= al(0,na)
         bt(1,na)= bt(0,na)
         ph(1,na)= ph(0,na)
CCC   #################################################################
         sn(1,na)= sn(0,na)
         pibn(1,na)= pibn(0,na)
         sgmn(1,na)= sgmn(0,na)
         br(1,na)= br(0,na)
         sr(1,na)= sr(0,na)
         qr(1,na)= qr(0,na)
CCC   #################################################################
         ss(1,na)= ss(0,na)
         pibd(1,na)= pibd(0,na)
         sgmd(1,na)= sgmd(0,na)
         bd(1,na)= bd(0,na)
         sd(1,na)= sd(0,na)
         qd(1,na)= qd(0,na)
CCC   #################################################################
	end do
CCC   #################################################################
      if (iauxi.ge.17) then
            stot=zero
            btot=zero
            do na=1,Nsp(1)
              stot=stot+sn(1,na)
              btot=btot+br(1,na)
            enddo
          OPEN(11,FILE=fnztr(1:nfnztr),STATUS='UNKNOWN',ACCESS='APPEND')
            WRITE(11,'(a,f6.3,a,i5)') ' tau=',tau,',  Nsp=',Nsp(1)
            WRITE(11,*) ('-',kk=1,63)
            WRITE(11,'(a)')' after particle rearrangement'
            WRITE(11,'(a,d30.20)')' current eom entropy (1)=',stot
            WRITE(11,'(a,d30.20)')' current eom baryon =',btot
            WRITE(11,*) ('-',kk=1,63) 
            CLOSE(11)
      endif
CCC   #################################################################
      stot=zero
      stoteta=zero
      btot=zero
      etot=zero
      sresint=zero
      sresinteta=zero
      bresint=zero
      eresint=zero
      slstint=zero
      slstinteta=zero
      blstint=zero
      elstint=zero
      natot=Nsp(1)
      booloss=0
      nacut=Nsp(1)
CCC   #################################################################
      do na=1,Nsp(1)
         eos(2)= ss(1,na)
         eos(3)= bd(1,na) 
         call eosph(eos,2)
         gamma= cosh(al(1,na)-et(1,na))*cosh(bt(1,na))
         uu3 =  sinh(al(1,na)-et(1,na))*cosh(bt(1,na))/tau
         ez=ez+dtau*tau**2*sn(1,na)/(ss(1,na)*tau*gamma)
     &       *((eos(1)+eos(4))*uu3**2+eos(4)/tau**2)
         etot=etot+sn(1,na)/(ss(1,na)*gamma)
     &       *((eos(1)+eos(4))*gamma**2-eos(4))
CCC   #################################################################
CCC   eliminating low temperature particles
         if(eos(5).lt.Tfin) then
            if(booloss.eq.0) then
               nacut=na-1					!!This particle is also dropped
               booloss=1
            endif
c            goto 124
         end if
CCC   #################################################################
CCC   debugging output to iflgce
               if (booloss.eq.0) then
                 stot=stot+sn(1,na)
                 if (abs(et(1,na)).le.1.d0) stoteta=stoteta+sn(1,na)
                 btot=btot+br(1,na)
               else if (booloss.eq.1) then
                 sresint=sresint+sn(1,na)
	           if (abs(et(1,na)).le.1.d0)
     &                                sresinteta=sresinteta+sn(1,na)
                 bresint=bresint+br(1,na)
                   eresint=eresint
     &             +sn(1,na)/ss(1,na)*((eos(1)+eos(4))*gamma**2-eos(4))
                 if (iflgce(na).eq.0) then
                    slstint=slstint+sn(1,na)
                    if (abs(et(1,na)).le.1.d0)
     &                                slstinteta=slstinteta+sn(1,na)
                    blstint=blstint+br(1,na)
                    elstint=elstint+
     &               sn(1,na)/ss(1,na)*((eos(1)+eos(4))*gamma**2-eos(4))
                 endif
               endif
               if ((iflgce(na).ne.2).and.(iflgce(na).ne.0)) then
                 if (iauxi.ge.6) then
                 open(22,file=fnfce(1:nfnfce),access='append')
                 WRITE(22,*)'fatal iflgce error in solvediff!'
                 WRITE(22,*)'iflgce,na=',iflgce(na),na
                 CLOSE(22)
                 endif
                 stop 'fatal ifgce error in solvediff!'
               endif
CCC   #################################################################
      end do
      Nsp(1)=nacut
CCC   #################################################################
CCC   debugging output to iflgce
      if ((iauxi.ge.6).and.(booloss.eq.1)) then
            OPEN(22,file=fnfce(1:nfnfce),access='append')
            WRITE(22,*)'EOM particle cutoff detected!'
            WRITE(22,'(a,d20.10)')'Tfin=',Tfin
            WRITE(22,'(a,i10,a,i10)')'start from',nacut+1,'  of',natot
            WRITE(22,*)'tau=',tau
              eos(2)= ss(1,nacut+1)
              eos(3)= bd(1,nacut+1)
              call eosph(eos,2)
            WRITE(22,'(a,7d30.20)')'eos(1-7)=',eos(1),eos(2),eos(3),
     &                 eos(4),eos(5),eos(6),eos(7)
            WRITE(22,'(a,d30.20,a,d30.20)')' entropy in EOM=',
     &               stot,'  energy in EOM=',btot
            WRITE(22,'(a,d30.20,a,d30.20)')' entropy dumped=',
     &               sresint,'  baryon number dumped=',bresint
            WRITE(22,'(a,d30.20,a,d30.20)')' entropy lost=',
     &               slstint,'  baryon number lost=',blstint
            CLOSE(22)


          OPEN(11,FILE=fnztr(1:nfnztr),STATUS='UNKNOWN',ACCESS='APPEND')
            WRITE(11,'(a,f6.3,a,i5)') ' tau=',tau,',  Nsp=',Nsp(1)
            WRITE(11,*) ('-',kk=1,63)
            WRITE(11,'(a)')' after temperature filtering'
            WRITE(11,'(a,d30.20)')' last step eom entropy=',stot+sresint
            WRITE(11,'(a,d30.20)')' last step eom baryon =',btot+bresint
c            WRITE(11,'(a,d30.20)')' last step eom energy =',etot+eresint
            WRITE(11,'(a,d30.20)')' total lost entropy=',slst+slstint
            WRITE(11,'(a,d30.20)')' total lost baryon =',blst+blstint
c            WRITE(11,'(a,d30.20)')' total lost energy =',elst+elstint
            WRITE(11,'(a,d30.20)')' current eom entropy (2)=',stot
            WRITE(11,'(a,d30.20)')' current eom baryon =',btot
            WRITE(11,'(a,d30.20)')' current eom energy =',etot+ez
            WRITE(11,'(a,d30.20)')' current ez energy  =',ez
            WRITE(11,'(a,d30.20)')' total dropped entropy=',sres+sresint
            WRITE(11,'(a,d30.20)')' total dropped baryon =',bres+bresint
c            WRITE(11,'(a,d30.20)')' total dropped energy =',eres+eresint
            WRITE(11,'(a,d30.20)')' total entropy=',stot+sres+sresint
            WRITE(11,'(a,d30.20)')' total baryon =',btot+bres+bresint
c            WRITE(11,'(a,d30.20)')' total energy =',etot+eres+eresint
            WRITE(11,*) ('-',kk=1,63)
          CLOSE(11)
	elseif(iauxi.ge.1) then
          OPEN(11,FILE=fnztr(1:nfnztr),STATUS='UNKNOWN',ACCESS='APPEND')
            WRITE(11,'(a,d30.20)')' current eom entropy (3)=',stot
c            WRITE(11,'(a,d30.20)')' current eom energy =',etot+ez
          CLOSE(11)
      end if
            sres=sres+sresint
            bres=bres+bresint
            eres=eres+eresint
            slst=slst+slstint
            blst=blst+blstint
            elst=elst+elstint
CCC   #################################################################
CCC   debugging output to iflgce
      if (iauxi.ge.20) then
            OPEN(22,file=fnfce(1:nfnfce),access='append')
            WRITE(22,'(a,f6.3,a,i5)') ' tau=',tau,',  Nsp=',Nsp(1)
            WRITE(22,*) ('-',kk=1,63)
            WRITE(22,'(a,d30.20)')' current eom entropy (4)=',stot
            WRITE(22,'(a,d30.20)')
     &   ' current eom entropy within the eta interval (-1,1)=',stoteta
            WRITE(22,'(a,d30.20)')
     &   ' current dropped entropy within eta interval (-1,1)=',
     &                                               sreseta+sresinteta
	      WRITE(22,'(a,d30.20)')
     &   ' total entropy within eta interval (-1,1)=',
     &                                       stoteta+sreseta+sresinteta
            CLOSE(22)
            sreseta=sreseta+sresinteta
            slsteta=slsteta+slstinteta
      endif
CCC   #################################################################
 123  FORMAT(4D16.8) 
 
 124  continue
      oflg= 1
      CALL mklt()			
      RETURN 

 325  CONTINUE
      open(11,file=fnnsp(1:nfnnsp),access='append')
      write(11,*) 'np error in solvediff.f \(^^:)/'
	write(11,'(a,2i9)')'na,iflgce(na)=',na,iflgce(na)
      close(11)
      stop 'SPheRIO: np error in solvediff.f'

      END

CCC   #################################################################



CCC   #################################################################
CCC   tranisotropy()
CCC   Compute momentum anisotropy at the given time
CCC   #################################################################
CCC   last updated 09262007
CCC   adopted itrobmd
CCC
CCC   last updated 05102007
CCC   adopted and modified
CCC   #################################################################
      subroutine tranisotropy(tau)
      implicit none
      include 'spherio.inc'
	include 'spherio.eos.inc'
	include 'spherio.debug.inc'
      integer i
      double precision tau,txx,tyy
      double precision sum1,sum2,sumnu
	double precision anisotropy
      double precision u0,u1,u2
      double precision QET,WET
      double precision press,edens


      if (itrobmd.ne.9) return
      sum1=0d0
      sum2=0d0
      sumnu=0d0
      do i=1,Nsp(1)

        eos(2)=ss(1,i)
        eos(3)=bd(1,i)
        call eosph(eos,2)
        edens=eos(1)
        press=eos(4)
        
        QET= dabs(et(1,i))/het         
        CALL krln(QET,WET,1,33)
        
        u0=cosh(bt(1,i))*cosh(al(1,i)-et(1,i))
        u1=sinh(bt(1,i))*cos(ph(1,i))
        u2=sinh(bt(1,i))*sin(ph(1,i)) 
        
        txx=(press+edens)*u1**2+press
        tyy=(press+edens)*u2**2+press
        sum1=sum1+((txx-tyy)*sn(1,i)*WET/ss(1,i)/u0/tau)
        sum2=sum2+((txx+tyy)*sn(1,i)*WET/ss(1,i)/u0/tau)
        sumnu=sumnu+sn(1,i)
      enddo

	anisotropy=0.d0
	if (sum2.ne.0.d0) anisotropy=sum1/sum2

      open(11,file='anisotropy.data',access='append')
      write(11,'(5d20.10)')tau,sum1,sum2,sumnu,anisotropy
      close(11)

      
      return
      end	  
CCC   #################################################################
CCC   fluid observer: transverse & longitudinal
CCC   subroutine trobsrv_ctrl()
CCC   for check of hydrodynamical evolution
CCC   #################################################################
CCC   last updated 05032009
CCC   it is built
CCC   #################################################################
      SUBROUTINE trobsrv_ctrl(ntsp,tau)
      IMPLICIT NONE
	integer ntsp
	double precision tau

      CALL trobsrv(ntsp,tau)
      CALL trobsrv_3(ntsp,tau)
      CALL trobsrv_pion15(ntsp,tau)
	CALL trobsrv_pion16(ntsp,tau)
	CALL tranisotropy(tau)


      RETURN
      END



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



CCC   #################################################################
CCC   trobsrv_old()
CCC   transverse fluid observer: check of hydrodynamical evolution
CCC   this subroutine is obsoleted
CCC   #################################################################
CCC   last updated 09262007
CCC   adopt itrobmd
CCC
CCC   last updated 05132007
CCC   this subroutine is obsoleted
CCC
CCC   last udpated 05102007
CCC   adopt a longer fluid evolution time
CCC
CCC   last updated 03082007
CCC   fix a bug concerning eta distribution
CCC
CCC   last updated 02122007
CCC   fix a bug
CCC
CCC   last updated 01312007
CCC   modified to provide better output
CCC
CCC   last updated 01282007
CCC   fixed a bug about output
CCC
CCC   last updated 01242007
CCC   modified the resolution of dxx,dyy,det as hxy,het
CCC   print info along the x-axis and y-axis rather than rr
CCC   this way of calculation has proved to be very poor
CCC
CCC   last updated 01232007
CCC   inherited from trobsrv2()
CCC   added longitudinal fluid observer
CCC   renamed to _old
CCC   #################################################################
      SUBROUTINE trobsrv_old(ntsp,tau)
      IMPLICIT NONE
      include 'spherio.inc'
	include 'spherio.debug.inc'
	include 'spherio.eos.inc'
	integer ntsp
	double precision tau
	integer nn
	double precision vx,vy,veta
	character fname*25,chtau*5

	double precision xxx
      double precision xxmin,xxmax,dxx
	double precision yyy
      double precision yymin,yymax,dyy
	integer mbin,mm
	double precision nnsph(200)
	double precision totsnsph(200)
	double precision abvtsph(200),vxsph(200),vysph(200)
	double precision ttsph(200),sssph(200)
	double precision vt

	double precision eett
	double precision etmin,etmax,det

      if (itrobmd.ne.9) return
	if ((tau.ne.1.d0).and.(abs(mod(tau,3.d0)).gt.(dtau/1.5d0)))return
	if (tau.gt.20.d0) 
     &   stop 'from trobsrv, bugflg.eq.9 tau.gt.20.d0, program stopped!'

CCC   #################################################################
CCC   raw data
      chtau='00000'
	write(chtau,'(f5.0)')tau
      fname='rawspectrum'//chtau//'.data'
      open(11,file=fname,access='append')
      write(11,'(a)')'i sni x y eta vx vy veta ed ss tt'
	write(11,'(a,i8,d20.10)')'Nsp,tau=',Nsp(1),tau

      do nn=1,Nsp(1)
           vx=dtanh(bt(1,nn))*dcos(ph(1,nn))/dcosh(al(1,nn)-et(1,nn))
           vy=dtanh(bt(1,nn))*dsin(ph(1,nn))/dcosh(al(1,nn)-et(1,nn))
           veta=dsinh(al(1,nn)-et(1,nn))*dcosh(bt(1,nn))/tau
	     veta=veta/dcosh(al(1,nn)-et(1,nn))*dcosh(bt(1,nn))
           eos(2)=ss(1,nn)
           eos(3)=bd(1,nn)
           call eosph(eos,2)
      write(11,'(i5,10d20.10)')nn,sn(1,nn),xx(1,nn),yy(1,nn),et(1,nn),
     &          vx,vy,veta,eos(1),ss(1,nn),eos(5)

      end do
      close(11)

CCC   #################################################################
CCC   the resolution
      dxx=hxy
	dyy=hxy
	det=het
CCC   #################################################################
CCC   x direction
      fname='xxspectrum'//chtau//'.data'
      xxmin=-20.d0
      xxmax=20.d0
      mbin=int((xxmax-xxmin)/dxx)+1
      do mm=1,mbin
	   nnsph(mm)=zero
	   totsnsph(mm)=zero
	   abvtsph(mm)=zero
	   vxsph(mm)=zero
	   vysph(mm)=zero
	   sssph(mm)=zero
	   ttsph(mm)=zero
      end do
      
      do nn=1,Nsp(1)
	     if (abs(et(1,nn)).ge.det) cycle
	     if (abs(yy(1,nn)).ge.dyy) cycle
	     xxx=xx(1,nn)
           mm=int((xxx-xxmin)/dxx)+1
           vx=dtanh(bt(1,nn))*dcos(ph(1,nn))/dcosh(al(1,nn)-et(1,nn))
           vy=dtanh(bt(1,nn))*dsin(ph(1,nn))/dcosh(al(1,nn)-et(1,nn))
           vt=dsqrt(vx**2+vy**2)
	     nnsph(mm)=nnsph(mm)+1.d0
	     totsnsph(mm)=totsnsph(mm)+sn(1,nn)
	     sssph(mm)=sssph(mm)+ss(1,nn)
           eos(2)=ss(1,nn)
           eos(3)=bd(1,nn)
           call eosph(eos,2)
	     ttsph(mm)=ttsph(mm)+eos(5)
           abvtsph(mm)=abvtsph(mm)+vt
	     vxsph(mm)=vxsph(mm)+vx
	     vysph(mm)=vysph(mm)+vx
      end do

      open(12,file=fname,access='append')
      write(12,'(a)')'xx mindex vt abvt sphnumber totsn ss tt'
	write(12,'(a,i8,d20.10)')'Nsp,tau=',Nsp(1),tau
      do mm=1,mbin
	   if (nnsph(mm).ge.1.d0) then
	     vxsph(mm)=vxsph(mm)/nnsph(mm)
	     vysph(mm)=vysph(mm)/nnsph(mm)
	     abvtsph(mm)=abvtsph(mm)/nnsph(mm)
           ttsph(mm)=ttsph(mm)/nnsph(mm)
           sssph(mm)=sssph(mm)/nnsph(mm)
	     xxx= ((mm-1)*dxx+xxmin+(mm)*dxx+xxmin)/2.d0
	     write(12,'(d20.10,i8,6d20.10)')xxx,mm,
     &       dsqrt(vxsph(mm)**2+vysph(mm)**2),
     &       abvtsph(mm),nnsph(mm),totsnsph(mm),
     &       sssph(mm),ttsph(mm)
	   elseif (totsnsph(mm).ne.zero) then
	     stop 'nnsph error in trobsrv_pionbag'
         endif
      end do
      close(12)

CCC   #################################################################
CCC   y direction
      fname='yyspectrum'//chtau//'.data'
      yymin=-20.d0
      yymax=20.d0
      mbin=int((yymax-yymin)/dyy)+1

      do mm=1,mbin
	   nnsph(mm)=zero
	   totsnsph(mm)=zero
	   abvtsph(mm)=zero
	   vxsph(mm)=zero
	   vysph(mm)=zero
	   sssph(mm)=zero
	   ttsph(mm)=zero
      end do
      
      do nn=1,Nsp(1)
	     if (abs(et(1,nn)).ge.det) cycle
	     if (abs(xx(1,nn)).ge.dxx) cycle
	     yyy=yy(1,nn)
           mm=int((yyy-yymin)/dyy)+1
           vx=dtanh(bt(1,nn))*dcos(ph(1,nn))/dcosh(al(1,nn)-et(1,nn)) 
           vy=dtanh(bt(1,nn))*dsin(ph(1,nn))/dcosh(al(1,nn)-et(1,nn))            
           vt=dsqrt(vx**2+vy**2)
	     nnsph(mm)=nnsph(mm)+1.d0
	     totsnsph(mm)=totsnsph(mm)+sn(1,nn)
	     sssph(mm)=sssph(mm)+ss(1,nn)
           eos(2)=ss(1,nn)
           eos(3)=bd(1,nn)
           call eosph(eos,2)
	     ttsph(mm)=ttsph(mm)+eos(5)
           abvtsph(mm)=abvtsph(mm)+vt
	     vxsph(mm)=vxsph(mm)+vx
	     vysph(mm)=vysph(mm)+vx
      end do

      open(12,file=fname,access='append')
      write(12,'(a)')'yy mindex vt abvt sphnumber totsn ss tt'
	write(12,'(a,i8,d20.10)')'Nsp,tau=',Nsp(1),tau
      do mm=1,mbin
	   if (nnsph(mm).ge.1.d0) then
	     vxsph(mm)=vxsph(mm)/nnsph(mm)
	     vysph(mm)=vysph(mm)/nnsph(mm)
	     abvtsph(mm)=abvtsph(mm)/nnsph(mm)
           ttsph(mm)=ttsph(mm)/nnsph(mm)
           sssph(mm)=sssph(mm)/nnsph(mm)
	     yyy= ((mm-1)*dyy+yymin+(mm)*dyy+yymin)/2.d0
	     write(12,'(d20.10,i8,6d20.10)')yyy,mm,
     &       dsqrt(vxsph(mm)**2+vysph(mm)**2),
     &       abvtsph(mm),nnsph(mm),totsnsph(mm),
     &       sssph(mm),ttsph(mm)
	   elseif (totsnsph(mm).ne.zero) then
	     stop 'nnsph error in trobsrv_pionbag'
         endif
      end do
      close(12)


CCC   #################################################################
CCC   longitudinal direction
      fname='lgspectrum'//chtau//'.data'
      etmin=-20.d0
      etmax=20.d0
      mbin=int((etmax-etmin)/det)+1

      do mm=1,mbin
	   nnsph(mm)=zero
	   sssph(mm)=zero
	   ttsph(mm)=zero
      end do

      do nn=1,Nsp(1)
	     if ((abs(xx(1,nn)).ge.dxx).or.(abs(yy(1,nn)).ge.dyy))
     &      cycle
           mm=int((et(1,nn)-etmin)/det)+1
	     nnsph(mm)=nnsph(mm)+1.d0
	     sssph(mm)=sssph(mm)+ss(1,nn)
           eos(2)=ss(1,nn)
           eos(3)=bd(1,nn)
           call eosph(eos,2)
	     ttsph(mm)=ttsph(mm)+eos(5)
      end do

      open(13,file=fname,access='append')
      write(13,'(a)')'et mindex sphnumber ss tt'
	write(13,'(a,i8,d20.10)')'Nsp,tau=',Nsp(1),tau
      do mm=1,mbin
	   if (nnsph(mm).ge.1.d0) then
           ttsph(mm)=ttsph(mm)/nnsph(mm)
           sssph(mm)=sssph(mm)/nnsph(mm)
	     eett= ((mm-1)*det+etmin+(mm)*det+etmin)/2.d0
	     write(13,'(d20.10,i8,3d20.10)')eett,mm,
     &       nnsph(mm),sssph(mm),ttsph(mm)
	   elseif (ttsph(mm).ne.zero) then
	     stop 'ttsph error in trobsrv_pionbag'
         endif
      end do
      close(13)


      RETURN
      END



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



CCC   #################################################################
CCC   subroutine trobsrv_pion15()
CCC   transverse fluid observer: check of hydrodynamical evolution
CCC   #################################################################
CCC   last updated 10172007
CCC   modified according to finsb_gen()
CCC
CCC   last updated 09262007
CCC   adopt itrobmd
CCC
CCC   last updated 03022007
CCC   fix a bug
CCC
CCC   last updated 02202007
CCC   optimize the output
CCC   evoke outp_raw()
CCC
CCC   last updated 02192007
CCC   modified to adopt findsb_gen()
CCC
CCC   last updated 02122007
CCC   fix a bug
CCC
CCC   last updated 01312007
CCC   minor improve in program style
CCC
CCC   last updated 01282007
CCC   use dtau instead of number
CCC
CCC   last updated 09072006
CCC   #################################################################
      SUBROUTINE trobsrv_pion15(ntsp,tau)
      IMPLICIT NONE
      include 'spherio.inc'
	include 'spherio.debug.inc'
	include 'spherio.eos.inc'
	integer ntsp
	double precision tau
	integer nn,gf
	character fname*30,chtau*5
	double precision rr
      double precision xmin(1:2),xmax(1:2),dx
      integer k
	double precision xxp,yyp,etp,ssp,bbp,ttp,ppp
	integer nloop1,iloop1,nloop2,iloop2
	double precision uu0,uut,vt
      double precision rrmin,rrmax,drr
	integer mbin,mm
	double precision sssph(500),nnsph(500),ttsph(500)
      double precision vx,vy,ve

      select case (itrobmd)
	case (8,15)
	    continue
	case default
	    return
	end select

	if (tau.ge.(16.d0*0.197d0)) stop 'tau calculation complished'
c	if ((abs(tau-10.d0)).gt.(dtau/1.5d0)) return
c	if (ntsp.le.5) call outp_raw(ntsp,tau)
      chtau='000'

c	write(*,*)'abs(tau-1.d0*0.197d0),dtau/1.50d0',
c     &     abs(tau-1.d0*0.197d0),dtau/1.50d0
c	stop 'for dbg'
      if (((abs(tau-5.d0*0.197d0)).gt.(dtau/1.5d0)).and.
     &    ((abs(tau-10.d0*0.197d0)).gt.(dtau/1.5d0)).and.
     &    ((abs(tau-15.d0*0.197d0)).gt.(dtau/1.5d0)).and.
     &    ((abs(tau-1.d0*0.197d0)).gt.(dtau/1.5d0)))
     &return
	write(chtau,'(f5.2)')(tau/0.197d0)
c      xmax(1) = 25.d0*0.197d0
c      if ((abs(tau-5.d0*0.197d0)).gt.(dtau/1.5d0))xmax(1)=15.d0*0.2d0
c      if ((abs(tau-10.d0*0.197d0)).gt.(dtau/1.5d0))xmax(1)=12.d0*0.2d0
c      if ((abs(tau-15.d0*0.197d0)).gt.(dtau/1.5d0))xmax(1)=25.d0*0.2d0
c      if ((abs(tau-1.d0*0.197d0)).gt.(dtau/1.5d0))xmax(1)=10.d0*0.2d0
c      xmin(1) = -xmax(1)
c	xmax(2) = xmax(1)
c	xmin(2) = xmin(1)
      do k=1,2
       xmin(k) = -1.4d0
       xmax(k) = +1.4d0
      end do

      do nn=1,Nsp(1)
         if (xx(1,nn).lt.xmin(1)) xmin(1)=xx(1,nn)
         if (xx(1,nn).gt.xmax(1)) xmax(1)=xx(1,nn)
         if (yy(1,nn).lt.xmin(2)) xmin(2)=yy(1,nn)
         if (yy(1,nn).gt.xmax(2)) xmax(2)=yy(1,nn)
      end do
      do k=1,2
       xmin(k) = xmin(k)-3.0d0*hxy
       xmax(k) = xmax(k)+3.0d0*hxy
      end do      

	etp=0.d0
c      dx = hxy/3.d0
      dx = hxy*2.d0
	nloop1=int((xmax(1)-xmin(1))/dx)
	nloop2=int((xmax(2)-xmin(2))/dx)
      open(22,file='rsph_debug.data',access='append')
      write(22,'(a,4d20.10)')'xmax(1),xmin(1),xmax(2),xmin(2)=',
     &  xmax(1),xmin(1),xmax(2),xmin(2)
      write(22,'(a,2i8)')'nloop1,nloop2=',
     &  nloop1,nloop2
      close(22)

	oflg=1
      call mklt()
 
      fname='char_trspectrum'//chtau//'.data'
      open(12,file=fname,access='append')
      write(12,'(a)')'rr xx yy tt0 ss'
      write(12,'(a,i8,d20.10)')'ntsp,tau=',ntsp,tau/0.197d0
      do iloop1=1,nloop1
	   xxp=xmin(1)+(iloop1-1.d0)/(nloop1-1.d0)*(xmax(1)-xmin(1))
         do iloop2=1,nloop2
c             yyp=0.d0
	       yyp=xmin(2)+(iloop2-1.d0)/(nloop2-1.d0)*(xmax(2)-xmin(2))
             rr=sqrt(xxp**2+yyp**2)
             call prcn(xxp,yyp,etp)
             call finsb_gen(tau,xxp,yyp,etp,ssp,bbp,vx,vy,ve,ppp)
	       eos(2)=ssp				!!eos(2) entropy density
	       eos(3)=bbp				!!eos(3) baryon number density 
	       gf=2
	       call eosph(eos,gf)
	       ttp=eos(5)
             write(12,'(5d20.10)')rr/0.197d0,xxp/0.197d0,yyp/0.197d0,
     &                             ttp,ssp
         end do
      end do
      close(12)

CCC   #################################################################
      fname='char_rawspectrum'//chtau//'.data'
      open(11,file=fname,access='append')
      write(11,'(a)')'rr xx yy et phi T/T0 uu0 uut vt sn ss'
      write(11,'(a,i8,d16.8)')'ntsp,tau=',ntsp,tau/0.197d0
      write(11,'(a,i8)')'Nsp(1)=',Nsp(1)
      do nn=1,Nsp(1)
	       rr=dsqrt(xx(1,nn)**2+yy(1,nn)**2)
	       eos(2)=ss(1,nn)				!!eos(2) entropy density
	       eos(3)=bd(1,nn)				!!eos(3) baryon number density 
	       gf=2
	       call eosph(eos,gf)
	       ttp=eos(5)
             uu0=cosh(al(1,nn)-et(1,nn))*cosh(bt(1,nn))
	       uut=sinh(bt(1,nn))
	       vt=uut/uu0
	       write(11,'(11d20.10)')rr/0.197d0,xx(1,nn)/0.197d0
     &          ,yy(1,nn)/0.197d0,et(1,nn)
     &          ,ph(1,nn),ttp,uu0,uut,vt,sn(1,nn),ss(1,nn)
      end do
      close(11)

CCC   #################################################################

      fname='char_avgspectrum'//chtau//'.data'
      open(11,file=fname,access='append')
      write(11,'(a)')'rr mindex T/T0 ss sphnumber'
      write(11,'(a,i8,d30.20)')'ntsp, tau',ntsp,tau/0.197d0
      rrmin=0.d0
      rrmax=xmax(1)
	drr=1.d0*hxy
	mbin=int((rrmax-rrmin)/drr)+1
      do mm=1,mbin
	   sssph(mm)=zero
	   nnsph(mm)=zero
	   ttsph(mm)=zero
      end do
      do nn=1,Nsp(1)
	     if(dabs(et(1,nn)).ge.het) cycle
	     rr=dsqrt(xx(1,nn)**2+yy(1,nn)**2)
           mm=int((rr-rrmin)/drr)+1
           if(rr.ge.rrmax) mm=mbin
           if(rr.le.rrmin) mm=1
	     sssph(mm)=sssph(mm)+ss(1,nn)
	     nnsph(mm)=nnsph(mm)+1.d0
      end do
      do mm=1,mbin
	   if (nnsph(mm).ge.1.d0) then
	     sssph(mm)=sssph(mm)/nnsph(mm)
	     eos(2)=sssph(mm)
	     eos(3)=zero
	     gf=2
		 call eosph(eos,gf)
	     ttsph(mm)=eos(5)
	     rr= ((mm-1)*drr+rrmin+(mm)*drr+rrmin)/2.d0
	     write(11,'(d20.10,i8,3d20.10)')rr/0.197d0,mm,ttsph(mm),
     &       sssph(mm),nnsph(mm)
	   elseif (sssph(mm).ne.zero) then
	     write(*,*)'mm,rr,nnsph(mm),sssph(mm)=',
     &           mm,rr,nnsph(mm),sssph(mm)
	     stop 'nnsph error in trobrv_pion'
         endif
      end do
      close(11)


      RETURN
      END



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



CCC   #################################################################
CCC   subroutine trobsrv_pion16()
CCC   transverse fluid observer: check of hydrodynamical evolution
CCC   #################################################################
CCC   last updated 05222009
CCC   fixed a bug concering tau output
CCC
CCC   last updated 05032009
CCC   built based on trobsrv_pion15()
CCC   #################################################################
      SUBROUTINE trobsrv_pion16(ntsp,tau)
      IMPLICIT NONE
      include 'spherio.inc'
	include 'spherio.debug.inc'
	include 'spherio.eos.inc'
	integer ntsp
	double precision tau
	integer nn,gf
	character fname*30,chtau*6
	double precision rr
      double precision xmin(1:2),xmax(1:2),dx
      integer k
	double precision xxp,yyp,etp,ssp,eep,bbp,ttp,ppp
	integer nloop1,iloop1,nloop2,iloop2
	double precision uu0,uut,vt
      double precision vx,vy,ve

      select case (itrobmd)
	case (16)
	    continue
	case default
	    return
	end select

      chtau='000'

      if (mod(tau,1.d0).gt.(0.5d0*dtau))
     &return
	write(chtau,'(f6.2)')(tau)

      do k=1,2
       xmin(k) = 0.d0
       xmax(k) = 0.d0
      end do

      do nn=1,Nsp(1)
         if (xx(1,nn).lt.xmin(1)) xmin(1)=xx(1,nn)
         if (xx(1,nn).gt.xmax(1)) xmax(1)=xx(1,nn)
         if (yy(1,nn).lt.xmin(2)) xmin(2)=yy(1,nn)
         if (yy(1,nn).gt.xmax(2)) xmax(2)=yy(1,nn)
      end do
      do k=1,2
       xmin(k) = xmin(k)-1.0d0*hxy
       xmax(k) = xmax(k)+1.0d0*hxy
      end do      

	etp=0.d0
c      dx = hxy/3.d0
      dx = hxy*2.d0
	nloop1=int((xmax(1)-xmin(1))/dx)
	nloop2=int((xmax(2)-xmin(2))/dx)
      open(22,file='rsph_debug.data',access='append')
      write(22,'(a,4d20.10)')'xmax(1),xmin(1),xmax(2),xmin(2)=',
     &  xmax(1),xmin(1),xmax(2),xmin(2)
      write(22,'(a,2i8)')'nloop1,nloop2=',
     &  nloop1,nloop2
      close(22)

	oflg=1
      call mklt()
 
      fname='char_trspectrum'//chtau//'.data'
      open(12,file=fname,access='append')
      write(12,'(a)')'xx yy rr tt0 ss'
      write(12,'(a,i8,d20.10)')'ntsp,tau=',ntsp,tau
      do iloop1=1,nloop1
	   xxp=xmin(1)+(iloop1-1.d0)/(nloop1-1.d0)*(xmax(1)-xmin(1))
         do iloop2=1,nloop2
c             yyp=0.d0
	       yyp=xmin(2)+(iloop2-1.d0)/(nloop2-1.d0)*(xmax(2)-xmin(2))
             rr=sqrt(xxp**2+yyp**2)
             call prcn(xxp,yyp,etp)
             call finsb_gen(tau,xxp,yyp,etp,ssp,bbp,vx,vy,ve,ppp)
	       eos(2)=ssp				!!eos(2) entropy density
	       eos(3)=bbp				!!eos(3) baryon number density 
	       gf=2
	       call eosph(eos,gf)
	       ttp=eos(5)
             write(12,'(5d20.10)')xxp,yyp,rr,ttp,ssp
         end do
      end do
      close(12)

CCC   #################################################################
      fname='char_rawspectrum'//chtau//'.data'
      open(11,file=fname,access='append')
      write(11,'(a)')'rr xx yy et phi T/T0 uu0 uut vt sn ss'
      write(11,'(a,i8,d16.8)')'ntsp,tau=',ntsp,tau
      write(11,'(a,i8)')'Nsp(1)=',Nsp(1)
      do nn=1,Nsp(1)
	       rr=dsqrt(xx(1,nn)**2+yy(1,nn)**2)
	       eos(2)=ss(1,nn)				!!eos(2) entropy density
	       eos(3)=bd(1,nn)				!!eos(3) baryon number density 
	       gf=2
	       call eosph(eos,gf)
	       ttp=eos(5)
             uu0=cosh(al(1,nn)-et(1,nn))*cosh(bt(1,nn))
	       uut=sinh(bt(1,nn))
	       vt=uut/uu0
	       write(11,'(11d20.10)')rr,xx(1,nn)
     &          ,yy(1,nn),et(1,nn)
     &          ,ph(1,nn),ttp,uu0,uut,vt,sn(1,nn),ss(1,nn)
      end do
      close(11)

CCC   #################################################################
      do k=1,2
       xmin(k) = 0.d0
       xmax(k) = 0.d0
      end do

      do nn=1,Nsp(1)
         if (xx(1,nn).lt.xmin(1)) xmin(1)=xx(1,nn)
         if (xx(1,nn).gt.xmax(1)) xmax(1)=xx(1,nn)
         if (yy(1,nn).lt.xmin(2)) xmin(2)=yy(1,nn)
         if (yy(1,nn).gt.xmax(2)) xmax(2)=yy(1,nn)
      end do
      do k=1,2
       xmin(k) = xmin(k)-1.0d0*hxy
       xmax(k) = xmax(k)+1.0d0*hxy
      end do      

	etp=0.d0
      dx = hxy/3.d0
c      dx = hxy*2.d0
	nloop1=int((xmax(1)-xmin(1))/dx)
	nloop2=int((xmax(2)-xmin(2))/dx)

      fname='char_trspectrum_x'//chtau//'.data'
      open(12,file=fname,access='append')
      write(12,'(a)')'xx yy tt0 ss ee'
      write(12,'(a,i8,d20.10)')'ntsp,tau=',ntsp,tau
      do iloop1=1,nloop1
	   xxp=xmin(1)+(iloop1-1.d0)/(nloop1-1.d0)*(xmax(1)-xmin(1))
c         do iloop2=1,nloop2
             yyp=0.d0
c	       yyp=xmin(2)+(iloop2-1.d0)/(nloop2-1.d0)*(xmax(2)-xmin(2))
             rr=sqrt(xxp**2+yyp**2)
             call prcn(xxp,yyp,etp)
             call finsb_gen(tau,xxp,yyp,etp,ssp,bbp,vx,vy,ve,ppp)
	       eos(2)=ssp				!!eos(2) entropy density
	       eos(3)=bbp				!!eos(3) baryon number density 
	       gf=2
	       call eosph(eos,gf)
	       ttp=eos(5)
	       eep=eos(1)
             write(12,'(5d20.10)')xxp,yyp,ttp,ssp,eep
c         end do
      end do
      close(12)


      RETURN
      END



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



CCC   #################################################################
CCC   transverse fluid observer
CCC   subroutine trobsrv()
CCC   for check of hydrodynamical evolution
CCC   #################################################################
CCC   last updated 11302007
CCC   modified dx
CCC
CCC   last udpated 11282007
CCC   add itrobmd 14 options
CCC
CCC   last updated 10172007
CCC   modified according to finsb_gen()
CCC
CCC   last updated 10092007
CCC   modified itrobmd 2, 13 options
CCC
CCC   last updated 10082007
CCC   modified itrobmd 2, 13 options
CCC
CCC   last updated 10022007
CCC   adopt itrobmd 2
CCC
CCC   last udpated 10012007
CCC   more aniso added
CCC
CCC   last updated 09262007
CCC   adopt itrobmd
CCC   calculate aniso
CCC
CCC   last updated 07262007
CCC   adopt bugflg 13
CCC
CCC   last udpated 05132007
CCC   adopt bugflg 9 here also
CCC
CCC   last updated 04182007
CCC   #################################################################
      SUBROUTINE trobsrv(ntsp,tau)
      IMPLICIT NONE
      include 'spherio.inc'
	include 'spherio.debug.inc'
	include 'spherio.eos.inc'
	integer ntsp
	double precision tau
	integer nn,gf
	character fname*30,chtau*5
	double precision rr
      double precision xmin(1:2),xmax(1:2),dx
      integer k
	double precision xxp,yyp,etp,ssp,bbp,ttp,ppp
	integer nloop1,iloop1,nloop2,iloop2
	double precision uu0,uut,vt
	double precision vx,vy,ve
	double precision eep
	double precision aniso


      select case (itrobmd)
      case (1,2)
          if (min((tau-int(tau/1.d0)),(1.d0+int(tau/1.d0)-tau))
     &        .gt.(0.5d0*dtau))return
	case (9)
		if (((abs(tau-1.0d0)).gt.(dtau/1.5d0)).and.
     &        ((abs(tau-4.0d0)).gt.(dtau/1.5d0)).and.
     &        ((abs(tau-7.d0)).gt.(dtau/1.5d0)).and.
     &        ((abs(tau-10.0d0)).gt.(dtau/1.5d0)))
     &    return
	case (12)
		if (((abs(tau-3.0d0)).gt.(dtau/1.5d0)).and.
     &        ((abs(tau-5.0d0)).gt.(dtau/1.5d0)).and.
     &        ((abs(tau-10.d0)).gt.(dtau/1.5d0)).and.
     &        ((abs(tau-1.0d0)).gt.(dtau/1.5d0)))
     &    return
	case (13,14)
	    continue
c	    if (tau.lt.(05.d0)) return
c	    if (tau.gt.(11.d0)) return
	case default
	    return
	end select

c	if (ntsp.le.5) call outp_raw(ntsp,tau)
      chtau='000'
	write(chtau,'(f5.2)')(tau)

      do k=1,2
       xmin(k) = -0.d0
       xmax(k) = +0.d0
      end do

      do nn=1,Nsp(1)
         if (xx(1,nn).lt.xmin(1)) xmin(1)=xx(1,nn)
         if (xx(1,nn).gt.xmax(1)) xmax(1)=xx(1,nn)
         if (yy(1,nn).lt.xmin(2)) xmin(2)=yy(1,nn)
         if (yy(1,nn).gt.xmax(2)) xmax(2)=yy(1,nn)
      end do
      do k=1,2
       xmin(k) = xmin(k)-1.5d0*hxy
       xmax(k) = xmax(k)+1.5d0*hxy
      end do      

	etp=0.d0

      select case (itrobmd)
	case(14)
          dx = hxy/3.d0
	case default
          dx = hxy*2.d0
	end select

	nloop1=int((xmax(1)-xmin(1))/dx)
	nloop2=int((xmax(2)-xmin(2))/dx)
      open(22,file='rsph_debug.data',access='append')
      write(22,'(a,4d20.10)')'xmax(1),xmin(1),xmax(2),xmin(2)=',
     &  xmax(1),xmin(1),xmax(2),xmin(2)
      write(22,'(a,2i8)')'nloop1,nloop2=',
     &  nloop1,nloop2
      close(22)

	oflg=1
      call mklt()
 
	if ((itrobmd.eq.12).or.(itrobmd.eq.14)) then
      fname='dbg_trspectrum'//chtau//'.data'
      open(12,file=fname,access='append')
      write(12,'(a)')'rr xx yy tt ss ee vx vy ve'
      write(12,'(a,i8,d20.10)')'ntsp,tau=',ntsp,tau
      do iloop1=1,nloop1
	   xxp=xmin(1)+(iloop1-1.d0)/(nloop1-1.d0)*(xmax(1)-xmin(1))
         do iloop2=1,nloop2
c             yyp=0.d0
	       yyp=xmin(2)+(iloop2-1.d0)/(nloop2-1.d0)*(xmax(2)-xmin(2))
             rr=sqrt(xxp**2+yyp**2)
             call prcn(xxp,yyp,etp)
             call finsb_gen(tau,xxp,yyp,etp,ssp,bbp,vx,vy,ve,ppp)
	       eos(2)=ssp				!!eos(2) entropy density
	       eos(3)=bbp				!!eos(3) baryon number density
	       gf=2
	       call eosph(eos,gf)
	       ttp=eos(5)
	       eep=eos(1)
             write(12,'(9d20.10)')rr,xxp,yyp,ttp,ssp,eep,vx,vy,ve
         end do
      end do
      close(12)
	end if
CCC   #################################################################
      if (itrobmd.eq.13) then
      fname='dbg_rawspectrum'//chtau//'.data'
      open(11,file=fname,access='append')
      write(11,'(a)')'rr xx yy et phi T uu0 uut vt sn ss ee vx vy ve'
      write(11,'(a,i8,d16.8)')'ntsp,tau=',ntsp,tau
      write(11,'(a,i8)')'Nsp(1)=',Nsp(1)
      do nn=1,Nsp(1)
	       rr=dsqrt(xx(1,nn)**2+yy(1,nn)**2)
	       eos(2)=ss(1,nn)				!!eos(2) entropy density
	       eos(3)=bd(1,nn)				!!eos(3) baryon number density 
	       gf=2
	       call eosph(eos,gf)
	       ttp=eos(5)
	       eep=eos(1)
             uu0=cosh(al(1,nn)-et(1,nn))*cosh(bt(1,nn))
	       uut=sinh(bt(1,nn))
	       vt=uut/uu0
	       vx=vt*cos(ph(1,nn))
	       vy=vt*sin(ph(1,nn))
	       ve=tanh(al(1,nn)-et(1,nn))/tau
	       write(11,'(15d20.10)')rr,xx(1,nn)
     &          ,yy(1,nn),et(1,nn)
     &          ,ph(1,nn),ttp,uu0,uut,vt,sn(1,nn),ss(1,nn)
     &          ,eep,vx,vy,ve
      end do
      close(11)

CCC   #################################################################
      fname='dbg_xxpectrum'//chtau//'.data'
      open(12,file=fname,access='append')
      write(12,'(a)')'xx rr yy tt0 ss ee vx vy ve aniso'
      write(12,'(a,i8,d20.10)')'ntsp,tau=',ntsp,tau
      do iloop1=1,nloop1
	   xxp=xmin(1)+(iloop1-1.d0)/(nloop1-1.d0)*(xmax(1)-xmin(1))
c         do iloop2=1,nloop2
             yyp=0.d0
c	       yyp=xmin(2)+(iloop2-1.d0)/(nloop2-1.d0)*(xmax(2)-xmin(2))
             rr=sqrt(xxp**2+yyp**2)
             call prcn(xxp,yyp,etp)
             call finsb_gen(tau,xxp,yyp,etp,ssp,bbp,vx,vy,ve,ppp)
	       eos(2)=ssp				!!eos(2) entropy density
	       eos(3)=bbp				!!eos(3) baryon number density
	       gf=2
	       call eosph(eos,gf)
	       ttp=eos(5)
	       eep=eos(1)
                  aniso=0.d0
	       if ((vx**2+vy**2).ne.0.d0) then
                  aniso=(vx**2-vy**2)/(vx**2+vy**2)
	       end if
             write(12,'(10d20.10)')xxp,rr,yyp,ttp,ssp,eep,vx,vy,ve,aniso
c         end do
      end do
      close(12)

      fname='dbg_yypectrum'//chtau//'.data'
      open(12,file=fname,access='append')
      write(12,'(a)')'yy rr xx tt0 ss ee vx vy ve aniso'
      write(12,'(a,i8,d20.10)')'ntsp,tau=',ntsp,tau
c      do iloop1=1,nloop1
c	   xxp=xmin(1)+(iloop1-1.d0)/(nloop1-1.d0)*(xmax(1)-xmin(1))
         xxp=0.d0
         do iloop2=1,nloop2
c             yyp=0.d0
	       yyp=xmin(2)+(iloop2-1.d0)/(nloop2-1.d0)*(xmax(2)-xmin(2))
             rr=sqrt(xxp**2+yyp**2)
             call prcn(xxp,yyp,etp)
             call finsb_gen(tau,xxp,yyp,etp,ssp,bbp,vx,vy,ve,ppp)
	       eos(2)=ssp				!!eos(2) entropy density
	       eos(3)=bbp				!!eos(3) baryon number density
	       gf=2
	       call eosph(eos,gf)
	       ttp=eos(5)
	       eep=eos(1)
                  aniso=0.d0
	       if ((vx**2+vy**2).ne.0.d0) then
                  aniso=(vx**2-vy**2)/(vx**2+vy**2)
	       end if
             write(12,'(10d20.10)')yyp,rr,xxp,ttp,ssp,eep,vx,vy,ve,aniso
         end do
c      end do
      close(12)
	end if

CCC   #################################################################
      if ((itrobmd.eq.1).or.(itrobmd.eq.2)) then
      fname='dbg_xx_3d.data'
      open(12,file=fname,access='append')
      write(12,'(a)')'tau xx tt ss ee vx vy ve aniso ppp'
      do iloop1=1,nloop1
	   xxp=xmin(1)+(iloop1-1.d0)/(nloop1-1.d0)*(xmax(1)-xmin(1))
c         do iloop2=1,nloop2
             yyp=0.d0
c	       yyp=xmin(2)+(iloop2-1.d0)/(nloop2-1.d0)*(xmax(2)-xmin(2))
             rr=sqrt(xxp**2+yyp**2)
             call prcn(xxp,yyp,etp)
             call finsb_gen(tau,xxp,yyp,etp,ssp,bbp,vx,vy,ve,ppp)
	       eos(2)=ssp				!!eos(2) entropy density
	       eos(3)=bbp				!!eos(3) baryon number density
	       gf=2
	       call eosph(eos,gf)
	       ttp=eos(5)
	       eep=eos(1)
                  aniso=0.d0
	       if ((vx**2+vy**2).ne.0.d0) then
                  aniso=(vx**2-vy**2)/(vx**2+vy**2)
	       end if
             write(12,'(10d20.10)')tau,xxp,ttp,ssp,eep,vx,vy,ve,
     &                             aniso,ppp
c             write(12,'(4d20.10)')tau,xxp,ttp,vx
c         end do
      end do
      close(12)

      fname='dbg_yy_3d.data'
      open(15,file=fname,access='append')
      write(15,'(a)')'tau yy tt ss ee vx vy ve aniso ppp'
c      do iloop1=1,nloop1
c	   xxp=xmin(1)+(iloop1-1.d0)/(nloop1-1.d0)*(xmax(1)-xmin(1))
         xxp=0.d0
         do iloop2=1,nloop2
c             yyp=0.d0
	       yyp=xmin(2)+(iloop2-1.d0)/(nloop2-1.d0)*(xmax(2)-xmin(2))
             rr=sqrt(xxp**2+yyp**2)
             call prcn(xxp,yyp,etp)
             call finsb_gen(tau,xxp,yyp,etp,ssp,bbp,vx,vy,ve,ppp)
	       eos(2)=ssp				!!eos(2) entropy density
	       eos(3)=bbp				!!eos(3) baryon number density
	       gf=2
	       call eosph(eos,gf)
	       ttp=eos(5)
	       eep=eos(1)
                  aniso=0.d0
	       if ((vx**2+vy**2).ne.0.d0) then
                  aniso=(vx**2-vy**2)/(vx**2+vy**2)
	       end if
             write(15,'(10d20.10)')tau,yyp,ttp,ssp,eep,vx,vy,ve,
     &                             aniso,ppp
         end do
c      end do
      close(15)
	end if

      RETURN
      END



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



CCC   #################################################################
CCC   transverse fluid observer
CCC   subroutine trobsrv_3()
CCC   for check of hydrodynamical evolution
CCC   #################################################################
CCC   last updated 09222010
CCC   fixed a bug
CCC
CCC   last updated 09212010
CCC   just restored the original subroutine of Osada
CCC   #################################################################
      SUBROUTINE trobsrv_3(ntsp,tau)
      IMPLICIT NONE
      include 'spherio.inc'
      include 'spherio.eos.inc'
      include 'spherio.debug.inc'
      include 'epos.incsf'
	integer ntsp
	double precision tau
	integer jj,kk
	character chtau*5
      double precision etasph,ysph,reta
      double precision ufrs0,ufrs1,ufrs2,ufrs3
      double precision u1,u2,u3,uu,uu0
      integer imaxeta
      parameter (imaxeta=201)
      integer netabin
      parameter (netabin=21)
      double precision etamax,etamin
c      parameter (etamax=10.5d0,etamin=-10.5d0)
      double precision dn_deta_sph(-1:imaxeta),dn_dy_sph(-1:imaxeta)
      double precision ds_deta_sph(-1:imaxeta),ds_dy_sph(-1:imaxeta)
      character*100 nomefila
      double precision xxp,yyp,etp,ttp,eep
      integer gf
      double precision ppp,ssp,bbp,vx,vy,ve

      select case (itrobmd)
      case (3)
          if (min((tau-int(tau/1.d0)),(1.d0+int(tau/1.d0)-tau))
     &        .gt.(0.5d0*dtau))return
	case default
	    return
	end select

c	if (ntsp.le.5) call outp_raw(ntsp,tau)
      chtau='000'
	write(chtau,'(f5.2)')(tau)

      etamin = -0.d0
      etamax = +0.d0

      do jj=1,Nsp(1)
         if (et(oflg,jj).lt.etamin) etamin=et(oflg,jj)
         if (et(oflg,jj).gt.etamax) etamax=et(oflg,jj)
      end do

      etamin = etamin-1.5d0*het
      etamax = etamax+1.5d0*het

      do jj=-1,imaxeta
          dn_deta_sph(jj)=0.d0
          ds_deta_sph(jj)=0.d0
          dn_dy_sph(jj)=0.d0
          ds_dy_sph(jj)=0.d0
      enddo

      do jj=1,Nsp(1)
cxx   transform into four-velocity in x-y-eta-tau
          ufrs0=+cosh(bt(oflg,jj))*cosh(al(oflg,jj)-et(oflg,jj))
          ufrs1=-sinh(bt(oflg,jj))* cos(ph(oflg,jj))
          ufrs2=-sinh(bt(oflg,jj))* sin(ph(oflg,jj)) 
          ufrs3=-cosh(bt(oflg,jj))*sinh(al(oflg,jj)-et(oflg,jj))*tau
cxx   transform into four-velocity in x-y-z-t
          uu0=cosh(et(oflg,jj))*ufrs0-1.d0/tau*sinh(et(oflg,jj))*ufrs3
	    u1=-ufrs1
	    u2=-ufrs2
	    u3=+sinh(et(oflg,jj))*ufrs0-1.d0/tau*cosh(et(oflg,jj))*ufrs3
          uu=sqrt(u1**2+u2**2+u3**2)
          if (uu.eq.0.d0) then
              etasph=0.d0
          else
              etasph=0.5d0*log((uu+u3)/(uu-u3))
          endif
          ysph=0.5d0*log((uu0+u3)/(uu0-u3))
cxx   verifications:      
c      if (etasph.eq.0.d0.and.et(oflg,jj).ne.0.d0)then
c      if (iauxi.ge.5) then
c          open(11,file=fnnan(1:nfnnan),access='append')
c          write(11,'(a)')'reporting from trobsrv_option_3:'
c          write(11,'(a,3i6)')'Nsp(1),oflg,jj=',Nsp(1),oflg,jj
c          write(11,*)'uu0,u1,u2,u3=',uu0,u1,u2,u3
c          write(11,*)'ufrs0,ufrs1,ufrs2,ufrs3=',ufrs0,ufrs1,ufrs2,ufrs3
c          write(11,*)'al,bt,ph=',al(oflg,jj),bt(oflg,jj)
c          write(11,*)'tau,xx,yy,et=',tau,
c     &                xx(oflg,jj),yy(oflg,jj),et(oflg,jj)
c          close(11)
c      endif
c      endif

          kk=int((etasph-etamin)/(etamax-etamin)*(netabin-1.d0)+0.5d0)+1
          if (etasph.lt.etamin.or.etasph.ge.etamax) kk=-1
cxx   verifications:
          if (kk.ne.-1.and.(kk.le.0.or.kk.gt.netabin)) then
              write(*,*)'kk fatal error, kk=',kk
              stop 'kk fatal error in tro_3 1'
          endif
          
          dn_deta_sph(kk)=dn_deta_sph(kk)+1.d0
          dn_deta_sph(0) =dn_deta_sph(0)+1.d0
          ds_deta_sph(kk)=ds_deta_sph(kk)+sn(oflg,jj)

      if (.not.(etasph.le.0.d0.or.etasph.ge.0.d0))then
      if (iauxi.ge.5) then
          open(11,file=fnnan(1:nfnnan),access='append')
          write(11,'(a)')'reporting from trobsrv_option_3'
          write(11,'(a,3i6)')'Nsp(1),oflg,jj=',Nsp(1),oflg,jj
          write(11,*)'uu0,u1,u2,u3=',uu0,u1,u2,u3
          write(11,*)'ufrs0,ufrs1,ufrs2,ufrs3=',ufrs0,ufrs1,ufrs2,ufrs3
          write(11,*)'al,bt,ph=',al(oflg,jj),bt(oflg,jj)
          write(11,*)'tau,xx,yy,et=',tau,
     &                xx(oflg,jj),yy(oflg,jj),et(oflg,jj)
          write(11,*)'eta fatal error in trobsrv_3!'
          close(11)
      endif
c          stop 'eta fatal error in trobsrv_3!'
      endif

          kk=int((ysph-etamin)/(etamax-etamin)*(netabin-1.d0)+0.5d0)+1
          if (ysph.lt.etamin.or.ysph.ge.etamax) kk=-1
          if (kk.ne.-1.and.(kk.le.0.or.kk.gt.netabin)) then
              write(*,*)'kk fatal error, kk=',kk
              stop 'kk fatal error in tro_3 2'
          endif

          if(mod(jj,1000).eq.1)then
          open(11,file='hydrocheck.data',access='APPEND')
          write(11,*)'etamin,etamax=',etamin,etamax
          write(11,*)'jj,eta,y=',jj,etasph,ysph
          write(11,*)'kk,netabin=',kk,netabin
          write(11,'(a,3i6)')'Nsp(1),oflg,jj=',Nsp(1),oflg,jj
          write(11,'(a,4d20.10)')'uu0,u1,u2,u3=',uu0,u1,u2,u3
          write(11,'(a,4d20.10)')
     &     'ufrs0,ufrs1,ufrs2,ufrs3=',ufrs0,ufrs1,ufrs2,ufrs3
          write(11,'(a,3d20.10)')
     &     'al,bt,ph=',al(oflg,jj),bt(oflg,jj),ph(oflg,jj)
          write(11,'(a,4d20.10)')'tau,xx,yy,et=',tau,
     &                xx(oflg,jj),yy(oflg,jj),et(oflg,jj)
          write(11,'(a,2d20.10)'),'sn,ss=',sn(oflg,jj),ss(oflg,jj)
          close(11)
          endif

          dn_dy_sph(kk)=dn_dy_sph(kk)+1.d0
          dn_dy_sph(0) =dn_dy_sph(0)+1.d0
          ds_dy_sph(kk)=ds_dy_sph(kk)+sn(oflg,jj)
      enddo !of jj(1->Nsp(1))

c      open(11,file=fnnan(1:nfnnan),access='append')
c      write(11,'(a,2d20.10)')'total sph particle number:',
c     &          dn_deta_sph(0),dn_dy_sph(0)
c      close(11)
cxx   output
      nomefila='trobsrv-o3-'//chtau//'.dat'
      open(22,file=nomefila,status='replace')
      write(22,'(a)')'eta dnsph-deta dssph-deta dnsph-dy dssph-dy ss ee'
      xxp=0.d0
      yyp=0.d0
      do jj=1,netabin
	    etp=etamin+(jj-1.d0)*(etamax-etamin)/(netabin-1.d0)
          call prcn(xxp,yyp,etp)
          call finsb_gen(tau,xxp,yyp,etp,ssp,bbp,vx,vy,ve,ppp)
	    eos(2)=ssp				!!eos(2) entropy density
	    eos(3)=bbp				!!eos(3) baryon number density
	    gf=2
	    call eosph(eos,gf)
	    ttp=eos(5)
	    eep=eos(1)
          write(22,'(7d20.10)')etp,dn_deta_sph(jj),ds_deta_sph(jj),
     &                              dn_dy_sph(jj),ds_dy_sph(jj),ssp,eep
      enddo !of jj(1->netabin)
      close(22)

      RETURN
      END



C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



